import QtQuick 2.0
import QtMultimedia 5.0
import QmlVlc 0.1
import QtQuick.Dialogs 1.0
import com.wemx 1.0

WEMXComponent {
    id : wemx
    anchors.fill: parent;
    property var properties: wemx.getProperties();
    
    property var playOnOffTag;
    property var playOnOffTag_RegInfo;    
    property var statusTag;  
    property var showFileDialogTag;
    property var showFileDialogTag_RegInfo;      
    property var pauseResumeTag;
    property var pauseResumeTag_RegInfo;
    property var volumeUpTag;
    property var volumeUpTag_RegInfo;
    property var volumeDownTag;
    property var volumeDownTag_RegInfo;
    property var seekForwardTag;
    property var seekForwardTag_RegInfo;
    property var seekBackwardTag;
    property var seekBackwardTag_RegInfo;
    property var positionTag;
    property var positionTag_RegInfo;
    property var urlTag;

    FileDialog {
        id: fileDialog
        title: "Please choose a file"
        folder: shortcuts.home
        onAccepted: {
            console.log("You chose: " + fileDialog.fileUrls)
            showFileDialogTag.writeInteger(0);
            urlTag.writeString(fileDialog.fileUrls);        
            Qt.quit()
        }
        onRejected: {
            Qt.quit()
        }
        Component.onCompleted: visible = false
    }

    Rectangle {
        color: 'black'
        anchors.fill: parent;
        VlcPlayer {
            id: vlcPlayer;
            mrl: "http://127.0.0.1/mjpg/video.mjpg";
            onMediaPlayerEncounteredError: onErrorVideo() ;
            onMediaPlayerEndReached: onEndVideo() ;
        }
        VlcVideoSurface {
            source: vlcPlayer;
            anchors.fill: parent;
        }
    }

    function onErrorVideo(){
        statusTag.writeInteger(2);
    }
    
    function onEndVideo(){
        statusTag.writeInteger(0);
    }

    function getURL() {
        var playingurl = urlTag.readString();
        if ( !playingurl )   
        {
            playingurl = "http://127.0.0.1/mjpg/video.mjpg";
        }  
        console.log("playingurl:"+playingurl);
        return playingurl;
    }

    function playCallback(parm1, parm2)
    {
        var playing =  playOnOffTag.readInteger();
             if (   playing != 0 ) {
                statusTag.writeInteger(1);
                vlcPlayer.play(getURL());  
            } else {
                statusTag.writeInteger(0);
                vlcPlayer.stop();  
            }     
    }

    function pauseresumeCallback(parm1, parm2)
    {
        var playing =  statusTag.readInteger();
        if (   playing != 0 ) {
            vlcPlayer.togglePause();
        }
    }
    
    function volumeUPCallback(parm1, parm2)
    {
        var volume = vlcPlayer.volume;
        var playing =  statusTag.readInteger();
        if ( playing != 0 ) {
            vlcPlayer.volume = volume+1;
        }
    }    

    function volumeDOWNCallback(parm1, parm2)
    {
        var volume = vlcPlayer.volume;
        var playing =  statusTag.readInteger();
        if ( playing != 0 ) {
            vlcPlayer.volume = volume-1;
        }
    }     

    function seekForwardCallback(parm1, parm2)
    {
        var position = vlcPlayer.position;
        var playing =  statusTag.readInteger();
        if ( playing != 0 ) {
            vlcPlayer.position = position+0.01;
        }
    }     

    function seekBackwardCallback(parm1, parm2)
    {
        var position = vlcPlayer.position;
        var playing =  statusTag.readInteger();
        if ( playing != 0 ) {
            vlcPlayer.position = position-0.01;
        }
    }     

    function positionCallback(parm1, parm2)
    {
        var position = positionTag.readInteger();
        var playing =  statusTag.readInteger();
        if ( position >0 && position <=100 )
        {
            var ratio = position/100.0;
            if ( playing != 0 ) {
                vlcPlayer.position = ratio;
            }            
        }
    }     
     
    function filedialogCallback(parm1, parm2)
    {
        var showing =  showFileDialogTag.readInteger();       
        if ( showing != 0 ) 
        {
            fileDialog.open(); 
        }
    }

    function onItemOpened(){
        var playing; 
                       
        playing = playOnOffTag.readInteger();
        playOnOffTag_RegInfo = playOnOffTag.registerChangeNotify(playCallback, playOnOffTag, 0);
        showFileDialogTag_RegInfo = showFileDialogTag.registerChangeNotify(filedialogCallback, showFileDialogTag, 0);
        pauseResumeTag_RegInfo=pauseResumeTag.registerChangeNotify(pauseresumeCallback, pauseResumeTag, 0);
        
        volumeUpTag_RegInfo = volumeUpTag.registerChangeNotify(volumeUPCallback, volumeUpTag, 0);
        volumeDownTag_RegInfo =  volumeDownTag.registerChangeNotify(volumeDOWNCallback, volumeDownTag, 0);
        seekForwardTag_RegInfo =  seekForwardTag.registerChangeNotify(seekForwardCallback, seekForwardTag, 0);
        seekBackwardTag_RegInfo = seekBackwardTag.registerChangeNotify(seekBackwardCallback, seekBackwardTag, 0);
        positionTag_RegInfo = positionTag.registerChangeNotify(positionCallback, positionTag, 0);
        
        if ( playing == 0 ) {
           statusTag.writeInteger(0);
            vlcPlayer.stop();
        } else {
            vlcPlayer.play(getURL());
            statusTag.writeInteger(1);
        }
    }

    function onItemClosed(){
        vlcPlayer.stop();
        playOnOffTag.unregisterChangeNotify(playOnOffTag_RegInfo);
        showFileDialogTag.unregisterChangeNotify(showFileDialogTag_RegInfo);
        pauseResumeTag.unregisterChangeNotify(pauseResumeTag_RegInfo);
        volumeUpTag.unregisterChangeNotify(volumeUpTag_RegInfo);
        volumeDownTag.unregisterChangeNotify(volumeDownTag_RegInfo);
        seekForwardTag.unregisterChangeNotify(seekForwardTag_RegInfo);
        seekBackwardTag.unregisterChangeNotify(seekBackwardTag_RegInfo);  
        positionTag.unregisterChangeNotify(positionTag_RegInfo);
        
    }        

    Component.onCompleted: {
        vlcPlayer.stop();

        playOnOffTag = wemx.get(properties.OnOff);
        statusTag = wemx.get(properties.Status);
        urlTag = wemx.get(properties.URL);
        pauseResumeTag = wemx.get(properties.PauseResume);
        volumeUpTag = wemx.get(properties.VolumeUp); 
        volumeDownTag = wemx.get(properties.VolumeDown); 
        seekForwardTag = wemx.get(properties.SeekForward); 
        seekBackwardTag = wemx.get(properties.SeekBackward); 
        showFileDialogTag = wemx.get(properties.showfiledialog);
        positionTag = wemx.get(properties.Position);
        
        wemx.conn.itemOpened.connect(onItemOpened);
        wemx.conn.itemClosed.connect(onItemClosed);  
     }

}
