import QtQuick 2.1
import QtQuick.Window 2.2
import QtWebEngine 1.2
import com.wemx 1.0
import QtQuick.Controls 1.0
import QtQuick.Controls.Styles 1.0
import QtQuick.Layouts 1.0
import QtQuick.Dialogs 1.2

WEMXComponent  {
	id: wemx
	property var properties: wemx.getProperties();
	
	property var urlTag: properties.urlTag;
    property var backTag: properties.backTag;   
    property var forewardTag: properties.forewardTag;   
	property var reloadTag: properties.reloadTag;
    property var stopTag: properties.stopTag;   
	
    property var urlTagRegID
    property var backTagRegID
    property var forewardTagRegID	
	property var reloadTagRegID
    property var stopTagRegID
	
	property QtObject defaultProfile: WebEngineProfile {
		storageName: "Default"
		
		// Blink-based (Chromium, Google Chrome, Opera 15+)
		// httpUserAgent: "Mozilla/5.0 (Linux; Android 4.4.2); Nexus 5 Build/KOT49H) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.117 Mobile Safari/537.36 OPR/20.0.1396.72047"
		
		// Internet Explorer ver 9
		//httpUserAgent: "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)"
		
		// Internet Explorer ver 10
		//httpUserAgent: "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)"		

		// Internet Explorer ver 11
		httpUserAgent: "Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0)"
		
		//  Mozilla (Gecko, Firefox)
		//httpUserAgent: "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0"
	}	

	property Item currentWebView: tabs.currentIndex < tabs.count ? tabs.getTab(tabs.currentIndex).item : null
	
    function changedUrlTag(parm1, parm2, value)  {
		console.log("changedUrlTag.......");
		var urlstr = urlTag.readString();
		if ( urlstr )
		{
			var ihttp = urlstr.indexOf("http://");
			var ihttps = urlstr.indexOf("https://");		
			if ( ihttp==0 || ihttps==0)
			{
				//wemxWebView.url = urlstr;
				tabs.writeUrl(urlstr);
			}
			else
			{
				//wemxWebView.url = "http://"+urlstr;
				tabs.writeUrl("http://"+urlstr);			
			}
		}

    }

    function changedBackTag(parm1, parm2, value)  {
		console.log("changedBackTag.......");
		tabs.goBack();
    }
	
    function changedForewardTag(parm1, parm2, value)  {
		console.log("changedForewardTag.......");
		tabs.goForeward();
    }	
	
    function changedReloadTag(parm1, parm2, value)  {
		console.log("changedReloadTag.......");
		tabs.goReload();
    }	

    function changedStopTag(parm1, parm2, value)  {
		console.log("changedStopTag.......");
		tabs.goStop();
    }	

	function connectURITAG() {
		if ( urlTag )
		{
			urlTag.setForceNotify(false);
            urlTagRegID = urlTag.registerChangeNotify(changedUrlTag, urlTag, null);
		}
	}

	function disconnectURITAG() {
        if( urlTag ) {
            urlTag.unregisterChangeNotify(urlTagRegID);
			urlTagRegID=undefined;
        }
	}
	
	TabView {
		id: tabs
		
		style: TabViewStyle {
			property color frameColor: "#999"
			property color fillColor: "#eee"
			frameOverlap: 1
			frame: Rectangle {
				color: "#eee"
				border.color: frameColor
			}
			tab: Rectangle {
				color: styleData.selected ? fillColor : frameColor
				implicitWidth: Math.max(text.width + 24, 80)
				implicitHeight: 20
				Rectangle { height: 1 ; width: parent.width ; color: frameColor}
				Rectangle { height: parent.height ; width: 1; color: frameColor}
				Rectangle { x: parent.width -1; height: parent.height ; width: 1; color: frameColor}
				Text {
					id: text
					anchors.left: parent.left
					anchors.verticalCenter: parent.verticalCenter
					anchors.leftMargin: 6
					text: styleData.title
					color: styleData.selected ? "black" : "white"
				}
				Rectangle {
					anchors.right: parent.right
					anchors.verticalCenter: parent.verticalCenter
					anchors.rightMargin: 4
					implicitWidth: 16
					implicitHeight: 16
					radius: width/2
					color: control.hovered ? "#eee": "#ccc"
					border.color: "gray"
					Text {text: "X" ; anchors.centerIn: parent ; color: "gray"}
					MouseArea {
						anchors.fill: parent
						onClicked: {
							if ( tabs.count > 1 )
								tabs.removeTab(styleData.index)
						}
					}
				}
			}		
		}
		function createEmptyTab(profile) {
			var tab = addTab("", tabComponent)
			// We must do this first to make sure that tab.active gets set so that tab.item gets instantiated immediately.
			tab.active = true
			tab.title = Qt.binding(function() { return tab.item.title })
			tab.item.profile = profile
			return tab
		}	
			
		function writeUrl(str)
		{
			var tab = tabs.getTab(tabs.currentIndex);
			if ( tab.item.url != str)
			{
				console.log("webEngineView.url is changed.");
				console.log("     "+tab.item.url );
				console.log("     "+str);
				//tab.item.triggerWebAction(WebEngineView.Stop);
				tab.item.url=str;
			}
		}
		
		function goBack()
		{
			var tab = tabs.getTab(tabs.currentIndex);
			if ( tab.item.canGoBack )
			{
				tab.item.goBack();
			}
		}		
		
		function goForeward()
		{
			var tab = tabs.getTab(tabs.currentIndex);
			if ( tab.item.canGoForeward )
			{
				tab.item.goForeward();
			}
		}
		
		function goReload()
		{
			var tab = tabs.getTab(tabs.currentIndex);
			tab.item.reload();
		}

		function goStop()
		{
			var tab = tabs.getTab(tabs.currentIndex);
			tab.item.stop();
		}
		
		anchors.fill: parent
		
		Component.onCompleted: createEmptyTab(defaultProfile)	
	
		Component {
			id: tabComponent
			WebEngineView {
				id: wemxWebView
				focus: true	
				url: "https://www.wemx.biz"
		
				onNewViewRequested: {
					if (!request.userInitiated)
						print("Warning: Blocked a popup window.")
					else if (request.destination == WebEngineView.NewViewInTab) {
						var tab = tabs.createEmptyTab(currentWebView.profile)
						tabs.currentIndex = tabs.count - 1
						request.openIn(tab.item)
					} else if (request.destination == WebEngineView.NewViewInBackgroundTab) {
						var tab = tabs.createEmptyTab(currentWebView.profile)
						request.openIn(tab.item)
					} else if (request.destination == WebEngineView.NewViewInDialog) {
						//var dialog = applicationRoot.createDialog(currentWebView.profile)
						//request.openIn(dialog.currentWebView)
						var tab = tabs.createEmptyTab(currentWebView.profile)
						tabs.currentIndex = tabs.count - 1
						request.openIn(tab.item)						
					} else {
						//var window = applicationRoot.createWindow(currentWebView.profile)
						//request.openIn(window.currentWebView)
						var tab = tabs.createEmptyTab(currentWebView.profile)
						tabs.currentIndex = tabs.count - 1
						request.openIn(tab.item)						
					}
				}	
				onLoadingChanged: {
					console.log("onLoadingChanged.....:"+loadRequest.url);
					if ( loadRequest.status == WebEngineView.LoadStartedStatus)
						console.log("WebEngineView.LoadStartedStatus");
					if ( loadRequest.status == WebEngineView.LoadSucceededStatus)
						console.log("WebEngineView.LoadSucceededStatus");
					if ( loadRequest.status == WebEngineView.LoadFailedStatus)
						console.log("WebEngineView.LoadFailedStatus");						
					if ( urlTag &&  loadRequest.status == WebEngineView.LoadStartedStatus )
					{
						urlTag.writeString(loadRequest.url);
					}
				}				
				onWindowCloseRequested: {
					if (tabs.count > 1)
						tabs.removeTab(tabs.currentIndex)
				}	
				
				onCertificateError: {
					error.defer()
					sslDialog.enqueue(error)
				}
				
				onFeaturePermissionRequested: {
					console.log(" onFeaturePermissionRequested:"+ securityOrigin);
					if ( feature == WebEngineView.Geolocation )
						console.log("   WebEngineView.Geolocation");
					if ( feature == WebEngineView.MediaAudioCapture )
						console.log("   WebEngineView.MediaAudioCapture");
					if ( feature == WebEngineView.MediaVideoCapture )
						console.log("   WebEngineView.MediaVideoCapture");
					if ( feature == WebEngineView.MediaAudioVideoCapture )
						console.log("   WebEngineView.MediaAudioVideoCapture");
				}	
				
				onFullScreenRequested: {
					if (request.toggleOn) {				
						console.log("onFullScreenRequested:ON");
					} else {
						console.log("onFullScreenRequested:OFF");
					}
					request.accept()
				}
			}
		}
	}
				
	Rectangle {
		id: downloadView
		color: "lightgray"
		visible: false
		x:0
		y:3*parent.height/4
		width:parent.width
		height:parent.height/4

		ListModel {
			id: downloadModel
			property var downloads: []
		}

		function append(download) {
			downloadModel.append(download)
			downloadModel.downloads.push(download)
		}

		Component {
          id: downloadItemDelegate

          Rectangle {
              width: listView.width
              height: childrenRect.height
              anchors.margins: 10
              radius: 3
              color: "transparent"
              border.color: "black"
              Rectangle {
                  id: progressBar

                  property real progress: downloadModel.downloads[index]
                                         ? downloadModel.downloads[index].receivedBytes / downloadModel.downloads[index].totalBytes : 0

                  radius: 3
                  color: width == listView.width ? "green" : "#2b74c7"
                  width: listView.width * progress
                  height: cancelButton.height

                  Behavior on width {
                      SmoothedAnimation { duration: 100 }
                  }
              }
              Rectangle {
                  anchors {
                      left: parent.left
                      right: parent.right
                      leftMargin: 20
                  }
                  Label {
                      id: label
                      text: path
                      anchors {
                          verticalCenter: cancelButton.verticalCenter
                          left: parent.left
                          right: cancelButton.left
                      }
                  }
                  Button {
                      id: cancelButton
                      anchors.right: parent.right
					  height:30
                      text: "Stop"
                      onClicked: {
                          var download = downloadModel.downloads[index]

                          download.cancel()

                          downloadModel.downloads = downloadModel.downloads.filter(function (el) {
                              return el.id !== download.id;
                          });
                          downloadModel.remove(index)
                      }
                  }
              }
          }
		}
		ListView {
			id: listView
			anchors {
				topMargin: 10
				top: parent.top
				bottom: parent.bottom
				horizontalCenter: parent.horizontalCenter
			}
			width: parent.width - 20
			spacing: 5

			model: downloadModel
			delegate: downloadItemDelegate

			Text {
				visible: !listView.count
				horizontalAlignment: Text.AlignHCenter
				height: 30
				anchors {
					top: parent.top
					left: parent.left
					right: parent.right
				}
				font.pixelSize: 20
				text: "No active downloads."
			}

			Rectangle {
				color: "gray"
				anchors {
					bottom: parent.bottom
					left: parent.left
					right: parent.right
				}
				height: 30
				Button {
					id: okButton
					text: "OK"
					anchors.centerIn: parent
					onClicked: {
						downloadView.visible = false
					}
				}
			}
		}
	}
	
	MessageDialog {
		id: sslDialog

		property var certErrors: []
		icon: StandardIcon.Warning
		standardButtons: StandardButton.No | StandardButton.Yes
		title: "Server's certificate not trusted"
		text: "Do you wish to continue?"
		detailedText: "If you wish so, you may continue with an unverified certificate. " +
                        "Accepting an unverified certificate means " +
                        "you may not be connected with the host you tried to connect to.\n" +
                        "Do you wish to override the security check and continue?"
		onYes: {
			certErrors.shift().ignoreCertificateError()
			presentError()
		}
		onNo: reject()
		onRejected: reject()

		function reject(){
			certErrors.shift().rejectCertificate()
			presentError()
		}
		
		function enqueue(error){
			certErrors.push(error)
			presentError()
		}
		
		function presentError(){
			visible = certErrors.length > 0
		}
	}	
	
	
	function onDownloadRequested(download) {
		console.log("onDownloadRequested.....");
		downloadView.visible = true
		downloadView.append(download)
		download.accept()
	}
	  
	Component.onCompleted: {
         wemx.conn.itemOpened.connect(onItemOpened);
         wemx.conn.itemClosed.connect(onItemClosed);
		 defaultProfile.downloadRequested.connect(onDownloadRequested);
	}

	function onItemOpened() {
	
		connectURITAG();
		
		if ( backTag )
		{
			backTag.setForceNotify(true);
            backTagRegID = backTag.registerChangeNotify(changedBackTag, backTag, null);
		}
		if ( forewardTag )
		{
			forewardTag.setForceNotify(true);
            forewardTagRegID = forewardTag.registerChangeNotify(changedForewardTag, forewardTag, null);
		}	
		if ( reloadTag )
		{
			reloadTag.setForceNotify(true);
            reloadTagRegID = reloadTag.registerChangeNotify(changedReloadTag, reloadTag, null);
		}	
		if ( stopTag )
		{
			stopTag.setForceNotify(true);
            stopTagRegID = stopTag.registerChangeNotify(changedStopTag, stopTag, null);
		}			
	}

	function onItemClosed() {
	
		disconnectURITAG();
   
        if( backTag ) {
            backTag.unregisterChangeNotify(backTagRegID);
			backTagRegID=undefined;
        }   
        if( forewardTag ) {
            forewardTag.unregisterChangeNotify(forewardTagRegID);
			forewardTagRegID=undefined;
        }   
		if ( reloadTag )
		{
            reloadTag.unregisterChangeNotify(reloadTagRegID);
			reloadTagRegID=undefined;
		}	
		if ( stopTag )
		{
            stopTag.unregisterChangeNotify(stopTagRegID);
			stopTagRegID=undefined;
		}		
	}
}
