.import "WEMXRDConst.js" as WConst
.import "LruCache.js" as Cache
.import "LruQueue.js" as LQueue
.import "RDSDeviceSubMain.js" as BlockDriverM




var WEMX_MAX_DRIVER_JOB_NUMBER =     		(512)
var WEMX_MAX_DRIVER_READ_JOB_NUMBER =  		(10240)


var WEMX_SUBURLCACHE_MAX_SIZE = 512;
var lruCache;
var lruQueue;

var driverDevice;
var api2inf;
var slaveEnvList = [];

var pJobList;
var periodTimer;

var immJobList;
var immTimer;

var devProperties;
var maxBlockGap = 0;
var retryCount = 0;

var mIsConnected = false;
var driverState = WConst.WEMXRD_COMMSTATE_NONE;


function parseSubUrl(subUrl, slaveEnvIdx)
{
//console.log("parseSubUrl ------------- >> ", subUrl, slaveEnvIdx);

    var node = lruCache.getNode(subUrl);
    if ( node == null )
    {    
        var addrList = subUrl.split("/");
        if ( addrList.length != 2 )
        {
            return(null);
        }

        var unitId = parseInt(addrList[0]);
        if ( unitId == undefined )
        {
            return(null);
        }

        var lowAddrInfo = api2inf.addressInfo(slaveEnvIdx, addrList[1]);
        if ( lowAddrInfo == null )
        {
            return(null);
        }
        
        if ( lruCache.size < WEMX_SUBURLCACHE_MAX_SIZE )
        {
            node = new lruCache.lrunode();
        }
        else
        {
            node = lruCache.outTailNode();
        }

//        var addrInfo = {};
//        addrInfo.unitId = unitId;
//        addrInfo.lowAddrInfo = lowAddrInfo;

        lowAddrInfo.unitId = unitId;

        node.key = subUrl;
        node.addrInfo = lowAddrInfo;
        
        lruCache.insertHash(node);
    }
    lruCache.setHeadNode(node);
       
    return(node.addrInfo);
}


function Address()
{
}

Address.prototype.init = function(subUrl, offset, mapBitSize, slaveEnvIdx)
{
    this.isActive = false;
    this.isWriting = false;
    this.change(subUrl, offset, mapBitSize, slaveEnvIdx);
}

Address.prototype.change = function(subUrl, offset, mapBitSize, slaveEnvIdx)
{
    // addrInfo.address
    // addrInfo.bitOff
    // addrInfo.addrBitSize
    // addrInfo.devCodeIdx

    var addrInfo = parseSubUrl(subUrl, slaveEnvIdx);
    if ( addrInfo == null )
    {
        this.isValid = false;
        return;
    }

    this.isValid = true;
    this.info = addrInfo;
            
    var locBitStart;
    if ( addrInfo.bitOff >= 0 )
    {
        locBitStart = (addrInfo.address * addrInfo.addrBitSize) + addrInfo.bitOff + offset;
    }
    else
    {
        locBitStart = (addrInfo.address + offset) * addrInfo.addrBitSize;
    }


    this.address = Math.floor(locBitStart / addrInfo.addrBitSize);
    this.bitOff = locBitStart % addrInfo.addrBitSize;

    // nBits는 bitOff에서 mapBitSize를 포함하여 addrBitSize으로 align되어 잇는 크기이다.
    // word나 dword address를 bit로 access할 경우, access할려는 bit영역을 포함하는 word영역전체를 nBits로 읽는다. 
    this.nBits = (((this.bitOff + mapBitSize) + (addrInfo.addrBitSize - 1)) & ~(addrInfo.addrBitSize - 1));    

    if ( this.readBuf == undefined )
    {
        var byteBufSize = ((this.nBits + (16 - 1)) & ~(16 - 1)) >> 3;
        this.readBuf = new ArrayBuffer(byteBufSize);    //new Uint8Array();
    }
    else
    {
        var newByteBufSize = ((this.nBits + (16 - 1)) & ~(16 - 1)) >> 3;
        if ( newByteBufSize > this.readBuf.length )
        {            
            this.readBuf = new ArrayBuffer(newByteBufSize); //new Uint8Array();
        }
    }
}







var readJobs = new Array(WEMX_MAX_DRIVER_READ_JOB_NUMBER);
var startReadJobIdx;
var endReadJobIdx;
var getJobTicks;

function getReadJobList()
{
    startReadJobIdx = endReadJobIdx = 0;

    if ( pJobList.size() == 0 )
    {
        return; //(0);
    }

    getJobTicks = wemx.elapsed();

    pJobList.goFirstIt();
    var rscOpScript = pJobList.curItValue();     
	
	var slaveEnvIdx = rscOpScript.slaveEnvIdx;
	var unitId = rscOpScript.unitId;
    var devCodeIdx = rscOpScript.devCodeIdx;

    readJobs[endReadJobIdx++] = rscOpScript;
    
    var lastRscOpScript = pJobList.lastItValue();

	while( rscOpScript != lastRscOpScript )
	{	
		pJobList.goNextIt();
        rscOpScript = pJobList.curItValue(); 

		if ( (unitId != rscOpScript.unitId) || (slaveEnvIdx != rscOpScript.slaveEnvIdx) 
                || (devCodeIdx != rscOpScript.devCodeIdx) )
		{
			return; //(endReadJobIdx);
		}

        if ( (rscOpScript.expireTime - getJobTicks) > 0 )
        {
            return; //(endReadJobIdx);
        }        

        if ( endReadJobIdx >= WEMX_MAX_DRIVER_READ_JOB_NUMBER )
        {
            return; //(endReadJobIdx);
        }        

        readJobs[endReadJobIdx++] = rscOpScript;
	}

	return; //(endReadJobIdx);
}




var notifyDataList = new Array(WEMX_MAX_DRIVER_READ_JOB_NUMBER);

function readJobDoneNotify(doneCount)
{
    var notifyNum = 0;
    
    pJobList.goFirstIt();

	for (var i = 0; i < doneCount; i++)
	{
        var rscOpScript = pJobList.curItValue();
        pJobList.eraseIt();


        if ( rscOpScript.onceReadCB != null )
        {        
            var onceReadCB = rscOpScript.onceReadCB;
            rscOpScript.onceReadCB = null;
            
            onceReadCB(rscOpScript);
        }
        else
        {        
            //if ( pJob.rscOpDev->readBufState == _WEMX_READBUFSTATE_DIRTY )
            //{
                //   pJob.rscOpDev->readBufState = _WEMX_READBUFSTATE_CLEAR;                
                //rscOpScript.notifyData(rscOpScript.addr.readBuf, rscOpScript.addr.bitOff, rscOpScript.mapBitSize);



                // TODO: dirty값만 notify되도록 수정하는 것이 필요한가???(얼마나 더 효율적일까?)

                /*
                                여기서 바로 rscOpScript.notifyData()를 부르면 같은 tag에 현재의 driver에 속한 여러개의 mapper가 mapping되어 있는 경우,
                                한바퀴 돌아서 doWrite()가 불리수 있다. 그럴 경우,  doWrite()에서는 pJobList.remove()을 불러 pJobList()의 iterator가 깨어질 수 있다.
                                그래서 pJobList()를 iterator하는 동안은 notify할 데이타를 저장해 놓고, iterator완료후 아래에서 notifyData가 수행되게 수정함 
                            */            
            
            if (rscOpScript.addr.isWriting == false)
            {
                notifyDataList[notifyNum++] = rscOpScript;            
            } 
            //}
        }

        rscOpScript.expireTime = getJobTicks + rscOpScript.addr.intervalMs;        
        pJobList.insert(rscOpScript);
    }

    startReadJobIdx += doneCount;



    for(var i=0; i < notifyNum; i++)
    {
        var rscOpScript = notifyDataList[i];
        rscOpScript.notifyData(rscOpScript.addr.readBuf, rscOpScript.addr.bitOff, rscOpScript.mapBitSize);
    }
}


function readJobDoneNoneNotify(doneCount)
{
    //var curTicks = wemx.elapsed();

    pJobList.goFirstIt();
    
	for (var i = 0; i < doneCount; i++)
	{
        var rscOpScript = pJobList.curItValue();
        pJobList.eraseIt();

        rscOpScript.onceReadCB = null;

        rscOpScript.expireTime = getJobTicks + rscOpScript.addr.intervalMs;        
        pJobList.insert(rscOpScript);
    }
    
    startReadJobIdx += doneCount;
}






function getNextTimeMs()
{
    if ( startReadJobIdx != endReadJobIdx )
    {
        return(0);
    }

    var rscOpScript = pJobList.firstItValue();
    var nextTimeMs = rscOpScript.expireTime -  wemx.elapsed();


//if ( nextTimeMs < 0 )
//{
//    console.log("getNextTimeMs >> -------------------- ",  nextTimeMs); //, periodInCount, periodOutCount); 
//}

    return((nextTimeMs < 0) ? 0 : nextTimeMs);
}


//var periodInCount = 0;
//var periodOutCount = 0;

// (1) timer나 기타 정보를 이용하여 주기적으로 읽거나... 
// (2) 독립된 read thread을 만들어 read에 blocking하게 기다렸다가 data가 오면 처리하게 한다. 
//void WEMXRscDriverDeviceThread::slotTransferPeriodJobList()


function doPeriodJob()
{
try {

    if ( mIsConnected == false )
    {
        return;
    }

//console.log(" -------------  doPeriodJob0 : ", pJobList.size());

    var reqCount;
    if ( startReadJobIdx == endReadJobIdx )
    {
        getReadJobList();
        
        reqCount = endReadJobIdx - startReadJobIdx;

//console.log(" -------------  doPeriodJob0 0 : ", reqCount);
        
        if ( reqCount <= 0 )
        {
            return;
        }
    }
    else
    {
        reqCount = endReadJobIdx - startReadJobIdx;
    }



    var rscOpScript = readJobs[startReadJobIdx];
//console.log("doPeriodJob >> 1 ------------------------                             ", wemx.elapsed(), rscOpScript.expireTime, reqCount, rscOpScript.address, wemx.queuedEventCount());

/*
for(var xk=startReadJobIdx; xk < endReadJobIdx; xk++)
{
    var tmpRscOp = readJobs[xk];
    console.log(" INFO >> -----------  check please !!!", tmpRscOp.address, tmpRscOp.unitId);    
}
*/

    var slaveEnvIdx = rscOpScript.slaveEnvIdx;
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv.enable == false )
    {   
        //readJobDoneNoneNotify(reqCount);
        console.log(" ABORT >> --------------------------------------------------------------");
        console.log(" ABORT >> ----------- not exist state !!! check please !!!", rscOpScript.address, rscOpScript.unitId);
        console.log(" ABORT >> --------------------------------------------------------------");
        
        periodTimer.start(getNextTimeMs(), doPeriodJob, false);
        return;
    }


    var iRtn = opzReadData(reqCount);

//console.log(" opzTransferData >> ------------------------------ ", iRtn);

    if ( iRtn >= 0 )
    {
        if ( (slaveEnv.timeoutCount + slaveEnv.commErrCount) > 0 )  // + slaveEnv.recoveryCount) > 0 )
        {
            slaveEnv.timeoutCount = slaveEnv.commErrCount = 0;
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMSTATE_CONNECTED);
        }
        
        readJobDoneNotify(iRtn);

//console.log(" ----------- 111");        
        periodTimer.start(getNextTimeMs(), doPeriodJob, false);
        return;
    }

    if ( iRtn == WConst.WEMXRD_E_NOTCONN )
    {
        disconnectSlaveAll();
        return;            
    } 

    if ( iRtn == WConst.WEMXRD_E_TIMEOUT )
    {
//qDebug() << "slotTransferPeriodJobList WEMXDRV_E_TIMEOUT >> " << slaveInfo->curTimeoutCount << m_drv->pMasterInfo->retryCount;    

        if ( slaveEnv.timeoutCount >= retryCount )
        {
            slaveEnv.commErrCount = 0;

            notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_TIMEOUT|WConst.WEMXRD_COMMSTATE_DISCONNECTED);
            disconnectSlave(slaveEnv);    // slaveEnvList로 stop state로 이동, stop notify

            // disconnectSlave()가 수행된후 pJobList.size()가 0이 되는 경우가 발생할 수 있고
            // 이 경우, getNextTimeMs()가 불리면 pJobList size가 0인 상태게서 pJobMap.last() 가 불리면 죽는다.
            // 그래서 여기서 처리함 
            periodTimer.start(0, doPeriodJob, false);
            return;        
        }
        else
        {
            slaveEnv.commErrCount = 0;
            
            slaveEnv.timeoutCount++;       
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_TIMEOUT|WConst.WEMXRD_COMMSTATE_CONNECTED);
        }

//console.log(" ----------- 222 ", startReadJobIdx, endReadJobIdx);
        periodTimer.start(getNextTimeMs(), doPeriodJob, false);
        return;        
    }
    else    //if ( iRtn == WEMXDRV_E_COMM ) or ( iRtn == WEMXDRV_E_ERROR )
    {
  
        slaveEnv.timeoutCount = 0;
        slaveEnv.commErrCount++;

        notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_DATACORRUPTED|WConst.WEMXRD_COMMSTATE_CONNECTED);

        readJobDoneNoneNotify(reqCount);

//console.log(" ----------- 333");        
        periodTimer.start(getNextTimeMs(), doPeriodJob, false);
        return;
    }

} catch(Err) {
    console.log(" ------- Err: " , Err);
}    
}



function mergeAddress(opzJob, otherRscOp, mergeBufLink)
{
	var othEndAddress = otherRscOp.address + (otherRscOp.addr.nBits / otherRscOp.addr.info.addrBitSize );
	var curEndAddress = opzJob.address + (opzJob.nBits / otherRscOp.addr.info.addrBitSize);
    
	var endAddress = (curEndAddress >= othEndAddress) ? curEndAddress : othEndAddress;
	opzJob.nBits = (endAddress - opzJob.address) * otherRscOp.addr.info.addrBitSize;

    return(otherRscOp.address - opzJob.address);
}

function isReadMergePossible(opzJob, otherRscOp)
{
	var endAddress = opzJob.address + (opzJob.nBits / otherRscOp.addr.info.addrBitSize);

    endAddress += maxBlockGap;
	if ( (otherRscOp.address >= opzJob.address) && (otherRscOp.address <= endAddress) )
	{
		return(true);
	}
	return(false);
}


var readBufLink = new Array(WEMX_MAX_DRIVER_READ_JOB_NUMBER);


var opzDataJob = {}
var bufLink = new Array(WEMX_MAX_DRIVER_JOB_NUMBER);
var opzDataBuf; // = new ArrayBuffer(wemx.wconst.TAG_MAX_DATASIZE);

function opzReadData(count)
{
//try {
    var startRscOpScript = readJobs[startReadJobIdx];
    opzDataJob.address = startRscOpScript.address;
    opzDataJob.nBits = startRscOpScript.addr.nBits;

    readBufLink[0] = 0;

    var bits = [];
    bits[0] = opzDataJob.nBits;

    var opCount = 1;
	for(; opCount < count; opCount++)
	{
		if ( isReadMergePossible(opzDataJob, readJobs[startReadJobIdx+opCount]) == true )
		{
		    readBufLink[opCount] = mergeAddress(opzDataJob, readJobs[startReadJobIdx+opCount]);
		    bits[opCount] = opzDataJob.nBits;
			continue;
		}

        //count = i;
        break;       
	}
	

//    var beforeTick = wemx.elapsed();

//console.log("opzReadData >> ----------- ", count, opzDataJob.address, opzDataJob.nBits);//, wemx.elapsed() - beforeTick);

	var iRtn = api2inf.readData(startRscOpScript.slaveEnvIdx, startRscOpScript.unitId, startRscOpScript.devCodeIdx, 
                                    opzDataJob.address, opzDataBuf, opzDataJob.nBits);
    

	if ( iRtn <= 0 )
	{
		return(iRtn);
	}		

	if ( iRtn < opzDataJob.nBits ) 
	{
	    for (var i = 0; i < opCount; i++)
	    {
	        if ( iRtn < bits[i] ) 
	        {
	            opCount = i;
	            break;
	        }
	    }
	}

    // TODO:     opzDataJob.nBits 요구한 size보다 반환된 iRtn이 작은 경우, iRtn만큼만 처리되게 하는 코드 추가하자.
	for(var n=0; n < opCount; n++)
	{
	    var rscOpScript = readJobs[startReadJobIdx+n];
        
		var locAddrBitSize = rscOpScript.addr.info.addrBitSize;
		if ( locAddrBitSize == 1 )
		{
		    if ( wemx.wbitmemcmp(rscOpScript.addr.readBuf, opzDataBuf, readBufLink[n], rscOpScript.mapBitSize) != 0 )
		    {
		        wemx.wbitmemcpy8(rscOpScript.addr.readBuf, opzDataBuf, readBufLink[n], rscOpScript.mapBitSize);
		    }
		}
		else
		{
		    var copyBytes = rscOpScript.addr.nBits >> 3;
		    if ( wemx.wmemcmp(rscOpScript.addr.readBuf, 0, opzDataBuf, readBufLink[n] * (locAddrBitSize >> 3), copyBytes) != 0 )
		    {   
		        wemx.wmemcpy(rscOpScript.addr.readBuf, 0, opzDataBuf, readBufLink[n] * (locAddrBitSize >> 3), copyBytes);
		    }
		}
	}
    
//}catch(Err)
//{
//    console.log("opzReadData >> Err --------- ", Err);
//}


	return(opCount);
}




var immJobs = new Array(WEMX_MAX_DRIVER_JOB_NUMBER);

function getImmJobList()    
{
	if ( immJobList.size() == 0 )
	{
		// 처리할 job이 없다.
		return(0);
	}

    immJobList.goFirstIt();
    var rscOpScript = immJobList.curItValue();
		
	var slaveEnvIdx = rscOpScript.slaveEnvIdx;
	var unitId = rscOpScript.unitId;
    var devCodeIdx = rscOpScript.devCodeIdx;


    var immJobCount = 1;	
    immJobs[0] = rscOpScript;


    // immJobList에 active가 false인 job은 write가 요청된 상태에서, 해당 rscOpScript가 deactive되거나, 
    // reregisterChangeNotify()되는 중에 rscOpScript isValid가 false가 되면 발생할 수 있을 것 같다.     
    // immJobList에 active가 false인 job이 있으면,  
    // active가 false인 job한개를 return하던지, active가 false인 job전까지 모아서 반환한다.
    if ( rscOpScript.addr.isActive == false )
    {
        return(1);
    }

    var lastRscOpScript = immJobList.lastItValue();
	while( rscOpScript != lastRscOpScript )
	{	
		immJobList.goNextIt();
        rscOpScript = immJobList.curItValue(); 

		if ( (unitId != rscOpScript.unitId) || (slaveEnvIdx != rscOpScript.slaveEnvIdx) 
                || (devCodeIdx != rscOpScript.devCodeIdx) || (rscOpScript.addr.isActive == false) )
		{
			return(immJobCount);
		}

		if ( immJobCount >= WEMX_MAX_DRIVER_JOB_NUMBER )
		{
			return(immJobCount);
		}

        immJobs[immJobCount++] = rscOpScript;
	}

	return(immJobCount);
}






function immJobDoneNotify(doneCount)
{
    immJobList.goFirstIt();

	for (var i = 0; i < doneCount; i++)
	{
        var rscOpScript = immJobList.curItValue();
        immJobList.eraseIt();

        rscOpScript.addr.isWriting = false;
    }    
}



function isWriteMergePossible(opzJob, otherRscOp)
{
	var endAddress = opzJob.address + (opzJob.nBits / otherRscOp.addr.info.addrBitSize);
    
	if ( (otherRscOp.address >= opzJob.address) && (otherRscOp.address <= endAddress) )
	{
		return(true);
	}
	return(false);
}

function opzWriteData(count)
{
//try {
    var startRscOpScript = immJobs[0];
    opzDataJob.address = startRscOpScript.address;
    opzDataJob.nBits = startRscOpScript.addr.nBits;

//console.log("opzWriteData 0 >> -- ", count, opzDataJob.address, opzDataJob.nBits);


    bufLink[0] = 0;

    var bits = [];
    bits[0] = opzDataJob.nBits;

    var opCount = 1;
	for(; opCount < count; opCount++)
	{
		if ( isWriteMergePossible(opzDataJob, immJobs[opCount]) == true )
		{
		    bufLink[opCount] = mergeAddress(opzDataJob, immJobs[opCount]);
		    bits[opCount] = opzDataJob.nBits;
			continue;
		}

        //count = i;
        break;       
	}
	
	for(var k=0; k < opCount; k++)
	{
        var rscOpScript = immJobs[k];        
		var locAddrBitSize = rscOpScript.addr.info.addrBitSize;
        

//console.log("opzWriteData 1 >> -- ", count, locAddrBitSize, bufLink[k]);

		if ( locAddrBitSize == 1 )
		{
            rscOpScript.getData(opzDataBuf, bufLink[k]);
		}
		else
		{
//console.log("opzWriteData 2 >> -- ", count, locAddrBitSize, bufLink[k], rscOpScript.addr.nBits, rscOpScript.mapBitSize);

			var copyBytes = rscOpScript.addr.nBits >> 3;
            if ( rscOpScript.addr.nBits == rscOpScript.mapBitSize )
            {
                rscOpScript.getData(opzDataBuf, bufLink[k]*locAddrBitSize);
            }
            else
            {
                if ( slaveEnvList[rscOpScript.slaveEnvIdx].clearBitsInWord == true )
                {
                    wemx.wmemset(opzDataBuf, bufLink[k]*(locAddrBitSize >> 3), 0, copyBytes);
                }
                else
                {
                    wemx.wmemcpy(opzDataBuf, bufLink[k]*(locAddrBitSize >> 3), rscOpScript.addr.readBuf, 0, copyBytes);
                }
                rscOpScript.getData(opzDataBuf, bufLink[k]*locAddrBitSize+rscOpScript.addr.bitOff);
            }                
		}	
	}	


//    var beforeTick = wemx.elapsed();

	var iRtn = api2inf.writeData(startRscOpScript.slaveEnvIdx, startRscOpScript.unitId, startRscOpScript.devCodeIdx, 
                                    opzDataJob.address, opzDataBuf, opzDataJob.nBits);

//console.log("opzWriteData >> ----------- ", iRtn, opzDataJob.address, opzDataJob.nBits, wemx.elapsed() - beforeTick);


	if ( iRtn <= 0 )
	{
		return(iRtn);
	}		

	if ( iRtn < opzDataJob.nBits ) 
	{
	    for (var i = 0; i < opCount; i++)
	    {
	        if ( iRtn < bits[i] ) 
	        {
	            opCount = i;
	            break;
	        }
	    }
	}
        
//}catch(Err)
//{
//    console.log("opzWriteData >> Err --------- ", Err);
//}


	return(opCount);
}




function doImmJob() //isRecovery)
{
//console.log("doImmJob >> 1 -------------------         >>>>>>>>>>> ", wemx.elapsed());

	var reqCount = getImmJobList(); //immJobList->getJobList(&slaveEnvIdx, &unitId, drvJobList);
	if ( reqCount == 0 )
	{
		return;
	}


//qDebug() << " --------- slotTransferImmJobList 1 >> " << reqCount;

    if ( mIsConnected == false )
	{
		immJobDoneNotify(reqCount);
		immTimer.start(0, doImmJob, false);
		return;
    }

    var rscOpScript = immJobs[0];
    if ( rscOpScript.addr.isActive == false )
    {
   		immJobDoneNotify(1);
		immTimer.start(0, doImmJob, false);
		return;
    }


    var slaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];
    //slaveEnv->unitId = unitId;
    if ( slaveEnv.enable == false )
	{
        immJobDoneNotify(reqCount);
		immTimer.start(0, doImmJob, false);
		return;
	}	

//console.log("doImmJob >> ------------------------       ", rscOpScript.expireTime - wemx.elapsed(), wemx.elapsed(), rscOpScript.expireTime, rscOpScript.address);

//	var iRtn = opzTransferData(slaveEnvIdx, unitId, reqCount, drvJobList);
    var iRtn = opzWriteData(reqCount);
	if ( iRtn >= 0 )
	{		
		if ( (slaveEnv.timeoutCount + slaveEnv.commErrCount) > 0 )  // + slaveEnv.recoveryCount) > 0 )
        {
            slaveEnv.timeoutCount = slaveEnv.commErrCount = 0;  //slaveEnv.recoveryCount = 0;
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMSTATE_CONNECTED);
        }
        
		immJobDoneNotify(iRtn);
		immTimer.start(0, doImmJob, false);

//console.log("doImmJob >> 2 --                   <<  <<  <<  << ", wemx.elapsed());

		return;
	}

    if ( iRtn == WConst.WEMXRD_E_NOTCONN )
    {
        disconnectSlaveAll();
        
        immTimer.start(0, doImmJob, false);
		return;            
    }

    if ( iRtn == WConst.WEMXRD_E_TIMEOUT )
    {
        if ( slaveEnv.timeoutCount >= retryCount )
        {
            slaveEnv.commErrCount = 0;
            
            immJobDoneNotify(reqCount);

            notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_TIMEOUT|WConst.WEMXRD_COMMSTATE_DISCONNECTED);
            disconnectSlave(slaveEnv);
        }
        else
        {
            slaveEnv.commErrCount = 0;

            slaveEnv.timeoutCount++;
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_TIMEOUT|WConst.WEMXRD_COMMSTATE_CONNECTED);
        }

        immTimer.start(0, doImmJob, false);
        return;    
    }
    else    //if ( iRtn == WEMXDRV_E_COMM )
    {
        slaveEnv.timeoutCount = 0;  //slaveEnv.recoveryCount = 0;
        slaveEnv.commErrCount++;

        notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_DATACORRUPTED|WConst.WEMXRD_COMMSTATE_CONNECTED);
        
        immJobDoneNotify(reqCount);
        immTimer.start(0, doImmJob, false);
        return;
    }
}


function writePriv(rscOpScript)
{
    if ( immJobList.size() == 0 )
    {
        immTimer.start(0, doImmJob, false);
    } 

    immJobList.insert(rscOpScript);
}


function setOnceReadCallback(onceReadCB, rscOpScript)
{
    startReadJobIdx = endReadJobIdx = 0;
    pJobList.remove(rscOpScript);

    // expireTime 0을 설정하여 제일 먼저 read하게 만든다.
    rscOpScript.expireTime = 0;   //WeMX::runtime->t.elapsed() + rscOpDev->interval;        
    rscOpScript.onceReadCB = onceReadCB;
    pJobList.insert(rscOpScript); 

    // read interval이 길어도 바로 write되게 하기 위하여 
    periodTimer.start(0, doPeriodJob, false);;
}


DriverDevice.prototype.doWrite = function(rscOpScript, blockDriver)
{
    // 윗단에서 driver에  write()를 고속으로 여러번 실행하면 doWrite signal을 여러번 발생시키지 않고
    // rscOpScript.isWriteReqPending value를 check하여 false일 경우에만 signal을 발생시킨다.
    // 즉 driver thread가 signal을 받아 invoke되어 rscOpScript.isWriteReqPending value를 false로 설정하면
    // write()시 signal을 발생시켜 doWrite() signal 발생했수를 줄인다.
    rscOpScript.isWriteReqPending = false;    

//console.log("SCRIPT DRIVER write ------------------> ", rscOpScript.address, mIsConnected);

//try {


    // doRegisterChangeNotify()나 notifyData()등으로 magicNumber가 WEMXRD_NO_UPDATE_MAGIC로 할당될 경우, 
    // driver에서 tag system으로 notify하지 않으나, notify가 enable되어야 할 시점에 반드시 doWrite가 오게 되고
    // doWrite에서 magicNumber를 update하여 driver에서 tag system으로 notify를 재개하게 한다.
    // mIsConnected나 rscOpScript.addr.isValid등의 값에 상관없이 무조건 updateMagicNumber하는 이유는 
    // driver의 state에 상관없이 magicNumber는 update되어야 mIsConnected나 rscOpScript.addr.isValid값이 정상값으로 돌아왔을때,
    // notify가 정상적으로 동작하게 된다. 
    rscOpScript.updateMagicNumber();

	if ( mIsConnected == false )
	{
		return;
	}
          
    if ( rscOpScript.addr.isValid == false )
    {
        return;
    }


    if ( rscOpScript.addr.isActive == false )
    {
        // write는 tag active시에만 가능하다. 
        console.log("WARNING> RDSDeviceMain::doWrite() try to write at deactive tag..");
        return;
    }

    if ( rscOpScript.addr.isWriting == false )
    {
        // isWriting이 true이면 이전 writing요청을 위한 iJob이 아직 처리되지 않았거나... 다른 iJob을 처리때문에 대기중이므로 ...
        // 중복으로 iJobMap.insertMulti()가 되지 않게 무시한다.
        // 무시해도... 최종 write시에는 rscOpDev->tagMapper->tagBase->getData()를 통하여 최신 tagbase data를 가져와서 
        // write하므로 최신 값이 write된다. 
        rscOpScript.addr.isWriting = true;

//console.log(" write 1 ------------- > ", clearBitsInWord,     rscOpScript.addr.nBits,    rscOpScript.mapBitSize);

        if ( slaveEnvList[rscOpScript.slaveEnvIdx].clearBitsInWord || (rscOpScript.addr.nBits == rscOpScript.mapBitSize) ) 
        {
            writePriv(rscOpScript);
        }
        else
        {
            setOnceReadCallback(writePriv, rscOpScript);
        }
    }
//} catch(err)
//
//    console.log("SCRIPT DRIVER write ERROR ----------  ", err);    
//}

//console.log("SCRIPT DRIVER write ----------             <<");    
}





function doActivate(rscOpScript)
{       
//console.log("SCRIPT DRIVER activate ------------------> ", rscOpScript.slaveEnvIdx, rscOpScript.address, rscOpScript.addr.nBits, rscOpScript.addr.isActive, rscOpScript.addr.isValid);

    if ( rscOpScript.addr.isActive == true )
    {
        return;
    }

    if ( rscOpScript.addr.isValid == false )
    {
        //if ( rscOpDev->magicNumber != WEMX_RSCDRIVER_NO_UPDATE_MAGIC )
        //{        
        
        //WEMXNotifyData nData(rscOpDev->tagMapper->mapType(), rscOpDev->magicNumber, nullptr, 0, WEMXRD_E_INVALID);
        //rscOpDev->magicNumber = rscOpDev->nFunc(rscOpDev->tagMapper, &nData);
        
        //}
        return;
    }

    rscOpScript.addr.isActive = true;


    
    var slaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];

    slaveEnv.deactiveQ.out(rscOpScript);
    
    if ( slaveEnv.enable == true )
    {
        if ( pJobList.size() == 0 )
        {
            periodTimer.start(0, doPeriodJob, false);
        }

        var elapsedTick = wemx.elapsed();
        rscOpScript.expireTime = elapsedTick + rscOpScript.addr.intervalMs;

    //console.log("------------------- active : ", rscOpScript.expireTime, elapsedTick, rscOpScript.addr.intervalMs);

        pJobList.insert(rscOpScript);
    }
    else
    {
        slaveEnv.stopQ.push_back(rscOpScript); 

        // notifySetError
        rscOpScript.setError(-1);
    }

//console.log("SCRIPT DRIVER activate ---             << << << <<", periodInCount, periodOutCount);

}



function doDeactivate(rscOpScript)
{        
//console.log("SCRIPT DRIVER deactivate ---------------------------------->");

    if ( rscOpScript.addr.isActive == false )
    {
        return;
    }

    rscOpScript.addr.isActive = false;



    var slaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];
    if ( slaveEnv.enable == true )
    {
        startReadJobIdx = endReadJobIdx = 0;
        pJobList.remove(rscOpScript);    
    }
    else
    {
        slaveEnv.stopQ.out(rscOpScript);
    }

    slaveEnv.deactiveQ.push_back(rscOpScript);


//console.log(" SCRIPT DRIVER deactivate 2: ", slaveEnv.deactiveQ.size(), slaveEnv.stopQ.size(), slaveEnv.totalCount);

//    if ( (slaveEnv.deactiveQ.size() + slaveEnv.stopQ.size()) == slaveEnv.totalCount )

    if ( slaveEnv.deactiveQ.size() == slaveEnv.totalCount )
    {
        if ( slaveEnv.enable == true )
        {
            // slaveEnv에 속한 element가 WEMXRD_NOTIFY_DATACORRUPT나 WEMXRD_NOTIFY_TIMEOUT가 발생하는 와중에
            // screen change같은 것이 일어나 slaveEnv에 속한 모든 element가 deactivate되는 경우, 앞에 발생한 error표시상태가 계속해서
            // 표시되게 된다....
            // 이것을 방지하기 위하여 slaveEnv에 속한 모든 element가 deactivate되었으나... slaveEnv가 enable인 상태인 경우, 
            // 해당 slaveEnv는 NOERROR로 notify한다. 
            // 그러면 해당 slaveEnv는 정상상태로 깜박이에서 표시하며... 해당 slaveEnv의 element가 다시 activate될때....
            // retry하게 된다. 
            slaveEnv.timeoutCount = slaveEnv.commErrCount = 0;
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMSTATE_CONNECTED);
        }
    }
}



DriverDevice.prototype.doReregisterChangeNotify = function(rscOpScript, subUrl, offset, intervalMs, blockDriver)
{
//console.log("SCRIPT DRIVER re--------------          >>>>>>>>>>>>>> ", subUrl, offset, intervalMs, blockDriver.elapsed());


    if ( rscOpScript.addr.isActive == false )
    {
        var addr = rscOpScript.addr;
        addr.change(subUrl, offset, rscOpScript.mapBitSize, rscOpScript.slaveEnvIdx);
        
        if ( addr.isValid == true )
        {
            addr.intervalMs = intervalMs;
            rscOpScript.unitId = addr.info.unitId;
            rscOpScript.devCodeIdx = addr.info.devCodeIdx;
            rscOpScript.address = addr.address;
        }        
    }
    else
    {
        doDeactivate(rscOpScript);

        var addr = rscOpScript.addr;
        addr.change(subUrl, offset, rscOpScript.mapBitSize, rscOpScript.slaveEnvIdx);

        if ( addr.isValid == true )
        {
            addr.intervalMs = intervalMs;
            rscOpScript.unitId = addr.info.unitId;
            rscOpScript.devCodeIdx = addr.info.devCodeIdx;
            rscOpScript.address = addr.address;
        }

        doActivate(rscOpScript);
    }    

//console.log("SCRIPT DRIVER re-------------            << << << << <<", subUrl, offset, intervalMs, wemx.elapsed());    
}








DriverDevice.prototype.doActivate = function(rscOpScript, blockDriver)
{
    doActivate(rscOpScript);
}


DriverDevice.prototype.doDeactivate = function(rscOpScript, blockDriver)
{
    doDeactivate(rscOpScript);

}








function connectPriv()
{
	if ( mIsConnected == true )
	{
		return;
	}

	var iRtn = api2inf.connect();
	if ( iRtn < 0 )
	{
		return;
	}

	mIsConnected = true;
}


function disconnectPriv()
{
	if ( mIsConnected == false )
	{
		return;
	}
	
	api2inf.disconnect();
    
	mIsConnected = false;
}




function moveStopListToPJobList(slaveEnv)
{
    var rscOpScript;
    var size = slaveEnv.stopQ.size();
    for(var i=0; i < size; i++)
    {
        rscOpScript = slaveEnv.stopQ.outFirst();

        pJobList.insert(rscOpScript);
    }    
}


function connectSlave(slaveEnv)
{    
//console.log(" -----------------> connectSlave: ", slaveEnv.enable, pJobList.size());

    if ( slaveEnv.enable == true )
    {
        return;
    }

    moveStopListToPJobList(slaveEnv);
    
    if ( pJobList.size() > 0 )
    {
        periodTimer.start(0, doPeriodJob, false);
    }

    slaveEnv.enable = true;
    slaveEnv.timeoutCount = 0;
    slaveEnv.commErrCount = 0;
        
    return;
}





function movePJobListToStopList(slaveEnv)
{
    pJobList.goFirstIt();
    var size = pJobList.size();

//console.log(" movePJobListToSlaveEnv ---------->> ", size);

    for(var i=0; i < size; i++)
    {
        var rscOpScript = pJobList.curItValue();        
        var locSlaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];        

        
        if ( slaveEnv == locSlaveEnv )
        {
            slaveEnv.stopQ.push_back(rscOpScript);
            
            // notifySetError
            rscOpScript.setError(-1);        
            
//console.log(" ---------->> setError:  ", size, i, rscOpScript.address, rscOpScript.unitId);


            pJobList.eraseIt();
        }
        else
        {
            pJobList.goNextIt();
        }
    }
}



function disconnectSlave(slaveEnv)
{        
    if ( slaveEnv.enable == false )
    {
        return(false);
    }

    slaveEnv.enable = false;
    startReadJobIdx = endReadJobIdx = 0;

    movePJobListToStopList(slaveEnv);

    for(var i=0; i < slaveEnvList.length; i++)
    {
        var locSlaveEnv = slaveEnvList[i];
        if ( locSlaveEnv.enable == true )
        {
            return;
        }
    }

    // 모든 slave이 disconnect되면 channel도 끊는다.
    disconnectChannel();    
    return;
}



function movePJobListToStopListAll()
{
    pJobList.goFirstIt();
    var size = pJobList.size();

    for(var i=0; i < size; i++)
    {
        var rscOpScript = pJobList.curItValue();        
        var slaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];        

        
        slaveEnv.stopQ.push_back(rscOpScript);
        
        // notifySetError
        rscOpScript.setError(-1);        
        
//console.log(" ---------->> setError:  ", size, i, rscOpScript.address, rscOpScript.unitId);


        pJobList.eraseIt();
    }
}


function disconnectSlaveAll()   
{    
    startReadJobIdx = endReadJobIdx = 0;
    
    movePJobListToStopListAll();
     
    for(var i=0; i < slaveEnvList.length; i++)
    {
        var locSlaveEnv = slaveEnvList[i];

        if ( locSlaveEnv.enable == true )
        {
            locSlaveEnv.enable = false;
            notifyCommState(locSlaveEnv, WConst.WEMXRD_COMMERROR_PEERDISCONNECTED|WConst.WEMXRD_COMMSTATE_DISCONNECTED);
        }
    }
    
    disconnectChannel();    
}


function connectChannel()
{
	if ( mIsConnected == true )
	{
        return;
    }

    api2inf.changeDevProperties(devProperties);
    connectPriv();
}


function disconnectChannel()
{   
    disconnectPriv();
}


//void WEMXRscDriverDeviceThread::slotDisconnect(int slaveEnvIdx, QSemaphore* pSyncSema)

DriverDevice.prototype.disconnect = function(slaveEnvIdx)
{
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    
    notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_USERDISCONNECTED|WConst.WEMXRD_COMMSTATE_DISCONNECTED);
    disconnectSlave(slaveEnv);
}


//void WEMXRscDriverDeviceThread::slotConnect(int slaveEnvIdx, QSemaphore* pSyncSema, int* pIRtn)

DriverDevice.prototype.connect = function(cb, slaveEnvIdx)
{
//console.log(" ---------------- connect: ",     mIsConnected);

    var slaveEnv = slaveEnvList[slaveEnvIdx];


    notifyCommState(slaveEnv, WConst.WEMXRD_COMMSTATE_CONNECTING);


    if ( mIsConnected == false )
    {
        connectChannel();        
        if ( mIsConnected == false )
        {
            notifyCommState(slaveEnv, WConst.WEMXRD_COMMERROR_NOTREACHABLEHOST|WConst.WEMXRD_COMMSTATE_DISCONNECTED);
            wemx.callback(-1, null, cb);
            return;
        }
    }
    
    notifyCommState(slaveEnv, WConst.WEMXRD_COMMSTATE_CONNECTED);
    connectSlave(slaveEnv);
    
    wemx.callback(0, null, cb);    
    return;
}



function notifyCommState(slaveEnv, state)   //, count)
{
console.log("RDSDeviceMain::notifyCommState >> ................ ",  state.toString(16));

    driverState = state;
    
    var connIdHash = slaveEnv.connIdToNotifyListHash;
    for(var key in connIdHash) 
    {
        var notifyList = connIdHash[key];

        for(var i=0; i < notifyList.length; i++)
        {
            wemx.callback(state, 0, notifyList[i]);
        }
    }
}


/*
function notifyAllCommState(state)
{
//console.log("RDSDeviceMain::notifyAllCommState >> ................ ", state);   

    for(var i=0; i < slaveEnvList.length; i++)
    {
        var slaveEnv = slaveEnvList[i];
        if ( slaveEnv.enable == true )
        {
            notifyCommState(slaveEnv, WConst.WEMXRD_NOTIFY_COMMSTOP, 0);
        }
        slaveEnv.enable = false;

        notifyCommState(slaveEnv, state, 0);        
    }    
}
*/

DriverDevice.prototype.doRegisterChangeNotify = function(rscOpScript, subUrl, offset, intervalMs, blockDriver)
{
//console.log("SCRIPT DRIVER register --------------          >>>>>>>>>>>>>> ", subUrl, offset, intervalMs, blockDriver.elapsed(), blockDriver.data.slaveEnvIdx, rscOpScript.mapBitSize);

    var addr = new Address();

    rscOpScript.slaveEnvIdx = blockDriver.data.slaveEnvIdx;
    addr.init(subUrl, offset, rscOpScript.mapBitSize, rscOpScript.slaveEnvIdx);
    
    rscOpScript.addr = addr;    
    if ( addr.isValid == true )
    {
        addr.intervalMs = intervalMs;
        rscOpScript.unitId = addr.info.unitId;
        rscOpScript.devCodeIdx = addr.info.devCodeIdx;
        rscOpScript.address = addr.address;
    }               

    var slaveEnv = slaveEnvList[rscOpScript.slaveEnvIdx];
    slaveEnv.totalCount++;
    slaveEnv.deactiveQ.push_back(rscOpScript);
}


// // subDriver에서만 call된다. 
DriverDevice.prototype.registerCommBlockStateNotify = function(cb, connId, slaveEnvIdx)
{
//console.log("SCRIPT DRIVER registerStateNotify : --------------  ", connId);

    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv == null )
    {
        return;
    }    

    var notifyList = slaveEnv.connIdToNotifyListHash[connId];
    if ( notifyList == null )
    {
        notifyList = slaveEnv.connIdToNotifyListHash[connId] = [];
//        qDebug() << "RDSDeviceMain::registerStateNotify >> not exist device CB List of connId... " << connId;
//        return;
    }


    notifyList.push(cb);

    // 처음 registerStateNotify 등록시, 현재의 state를 보내고, 이후에는 state변동시에만 보낸다.
    //var state = (slaveEnv.enable == true) ? WConst.WEMXRD_NOTIFY_NOERROR : WConst.WEMXRD_NOTIFY_COMMSTOP;
    wemx.callback(driverState, null, cb);

/*    
    if ( mIsConnected == false )
    {
        wemx.callback(WConst.WEMXRD_NOTIFY_DISCONNECTION, null, cb);
    }*/
}


/*
DriverDevice.prototype.newConnection = function(connId, slaveEnvIdx)
{
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv == null )
    {
        return;
    }

    var notifyList = slaveEnv.connIdToNotifyListHash[connId];
    if ( notifyList == null )
    {
        slaveEnv.connIdToNotifyListHash[connId] = [];
    }    
}
*/


DriverDevice.prototype.openSlaveEnv = function(cbName)
{
    var slaveEnvIdx = api2inf.openSlaveEnv();
    if ( slaveEnvIdx < 0 )
    {
        return(-1);
    }   

    if ( slaveEnvList.length != slaveEnvIdx )
    {
        console.log("FATAL DriverDevice.prototype.openSlaveEnv >> ........", slaveEnvList.length, slaveEnvIdx);
        return(-1);
    }
    
    var slaveEnv = {};

    slaveEnv.stopQ = new LQueue.LruQueue();
    slaveEnv.deactiveQ = new LQueue.LruQueue();

    slaveEnv.totalCount = 0;
    slaveEnv.name = cbName;
    slaveEnv.timeoutCount = 0;
    slaveEnv.commErrCount = 0;
    slaveEnv.enable = false;
    slaveEnv.clearBitsInWord = false;
    slaveEnv.connIdToNotifyListHash = {};
    slaveEnvList.push(slaveEnv);
    
    return(slaveEnvIdx);
}



DriverDevice.prototype.changeNotifyCommBlockExtraSettingMap = function(cbExtraProperties, cb, slaveEnvIdx)
{    
    for(var devCode in cbExtraProperties)
    {
        var subProp = cbExtraProperties[devCode];
        var props = "DeviceCode=" + devCode;
        for(var key2 in subProp)
        {
            props += ";" + key2 + "=" + subProp[key2]
        }

//console.log(" -------- MEM : ", props);        
        api2inf.setProperty(slaveEnvIdx, "MemoryFunction", props);
    }       
    return;
}


DriverDevice.prototype.changeNotifyCommBlockSettingMap = function(cbProperties, cb, slaveEnvIdx)
{    
    slaveEnvList[slaveEnvIdx].clearBitsInWord = (cbProperties.RestOfTheBitsInThisWord == "Clear");

    for(var key in cbProperties)
    {
//        console.log("         ", key, ": ", cbProperties[key]);
        api2inf.setProperty(slaveEnvIdx, key, cbProperties[key]);
    }

    return;
}



// internal only
function changeNotifySettingMap(properties)
{
    devProperties = properties;
    propertiesStrToInt(devProperties);
    
    if ( mIsConnected == true )
    {
        var tmpSlaveEnvList = [];
        
        for(var i=0; i < slaveEnvList.length; i++)
        {
            var slaveEnv = slaveEnvList[i];        
            if ( slaveEnv.enable == true )
            {
                tmpSlaveEnvList.push(slaveEnv);
                disconnectSlave(slaveEnv);
            }
        }

        if ( mIsConnected == false )
        {
            connectChannel();            
            if ( mIsConnected == false )
            {
                return;
            }
        }
        
        for(var i=0; i < tmpSlaveEnvList.length; i++)
        {
            var slaveEnv = tmpSlaveEnvList[i];
            connectSlave(slaveEnv);
        }
    }
}



DriverDevice.prototype.disConnection = function(connId, slaveEnvIdx)
{
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv == null )
    {
        return;
    }

    slaveEnv.connIdToNotifyListHash[connId] = [];
}


DriverDevice.prototype.setProperty = function(slaveEnvIdx, key, value, cb)
{
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv == null )
    {
        return;
    }

    var iRtn = api2inf.setProperty(slaveEnvIdx, key, value);
    wemx.callback(iRtn, null, cb);    
    return;
}


DriverDevice.prototype.getProperty = function(slaveEnvIdx, key, cb)
{
    var slaveEnv = slaveEnvList[slaveEnvIdx];
    if ( slaveEnv == null )
    {
        return;
    }

    var vRtn = api2inf.getProperty(slaveEnvIdx, key);    
    wemx.callback(vRtn, null, cb);
    return;
}



DriverDevice.prototype.executed = function(cmd, args1, args2, cb, connId)
{
//console.log("RDSDevice --------------   ============ >>> ", cmd );

    //"changeNotifySettingMap" command외에는 sub driver로 외부에서 접근하고, sub driver가 
    // parent의 API를 직접 불러 외부 요구에 대응한다. 

    switch(cmd)
    {
/*    
        case "disConnection":
            disConnection(connId);
            break;*/

/*            
        case "registerSettingMapNotify":
            registerSettingMapNotify(args1, cb, connId);
            break;
            */

        // internal only            
        case "changeNotifySettingMap":
            changeNotifySettingMap(args1);
            break;
    }
}



function createServiceBlock(cbName)
{       
    var subCtrStr =     "import QtQuick 2.0;\
                        import com.wemx 1.0;\
                        WEMXBlockDriver {\
                            id: serviceBlockDriver;\
                            property var blockDriver;\
                            function executed(cmd, args1, args2, cb, connId)\
                            {\
                                blockDriver.executed(cmd, args1, args2, cb, connId);\
                            }\
                            Component.onCompleted: {\
                                serviceBlockDriver.conn.executed.connect(executed);\
                            }\
                        }";
                    
    var serviceBlockDriver = Qt.createQmlObject(subCtrStr, wemx);

    var blockDriver = new BlockDriverM.SubDriver(serviceBlockDriver);        

    if ( blockDriver.init(driverDevice, cbName) == false )
    {
        return;
    }

    serviceBlockDriver.blockDriver = blockDriver;

// console.log(" ---------------------------- createSub: ", cbName );

    serviceBlockDriver.setName(cbName);
//console.log("addServiceBlock >> ", cbName, subDriver.subServiceDriver);
    wemx.addServiceBlock(cbName, blockDriver.serviceBlockDriver);
        
    return;
}


function DriverDevice()
{
}


function propertiesStrToInt(devProperties)
{
    maxBlockGap = parseInt(devProperties.MaxBlockGap);
    retryCount = parseInt(devProperties.RetryCount);
}


function init()
{
    opzDataBuf = new ArrayBuffer(wemx.wconst.TAG_MAX_DATASIZE);
    
    driverDevice = new DriverDevice();
    wemx.conn.executed.connect(driverDevice.executed);

    
    var rawDriverPath = wemx.findRawDriver();
    if ( rawDriverPath == null )
    {
        console.log("RDSDeviceMain >> can not find a raw driver ... ");
        return(false);
    }

    console.log(">> start Driver(", wemx.name(), ") / loading ", rawDriverPath, "...");


    switch(wemx.driverType())
    {
        case WConst.WEMXRD_TYPE_CAPI1:
        case WConst.WEMXRD_TYPE_CAPI2:
            var subCtrStr = "import com.wemx 1.0; WEMXJSToCStub {}";   //WEMXJSAPI2Interface {}";                            
            api2inf = Qt.createQmlObject(subCtrStr, wemx); //rootItem);            
            break;
            
        case WConst.WEMXRD_TYPE_JSAPI1:    // simple
            Qt.include("WEMXJSAPI1Stub.js");
            api2inf = new WEMXJSAPI1Stub();
            break;

        case WConst.WEMXRD_TYPE_JSAPI2:    // normal
            Qt.include("WEMXJSAPI2Stub.js");
            api2inf = new WEMXJSAPI1Stub();        
            break;
    }

        
    if ( api2inf.init(wemx, rawDriverPath) < 0 )
    {
        console.log("RDSDeviceMain >> fail to init a raw driver ...", wemx.name());
        return(false);
    }

        
    devProperties = wemx.executeS("getDeviceProperties", null, null);    
    propertiesStrToInt(devProperties);
    if ( api2inf.open(devProperties) < 0 )
    {
        console.log("RDSDeviceMain >> fail to open a raw driver ...", wemx.name());
        return(false);
    }

    var cbNameList = wemx.getBlockNames();
    for(var i=0; i < cbNameList.length; i++)
    {
//console.log(" ------ CB : ",     cbNameList[i] );
        createServiceBlock(cbNameList[i]);
    }

    
    lruCache = new Cache.LruCache();
    pJobList = wemx.createPJobList();
    immJobList = wemx.createImmJobList();

    var wTimerService = wemx.getService("/local/TimerService");
    periodTimer = wTimerService.executeS("getTimer", null, null);
    periodTimer.setSingleShot(true);

    immTimer = wTimerService.executeS("getTimer", null, null);
    immTimer.setSingleShot(true);

    return(true);
}


