import QtQuick 2.0
import com.wemx 1.0

Item {
	id : service
	
	property var media;

    function initService() {
		media = wemx.createMedia();
	}

    function onMessageReceived(sender, msg) {
		wemx.log_d("[WEMXAndroidMediaService] msg kind : " + msg.kind);
		
		switch (msg.kind)
		{
			case "createVideoImage":
				{
					wemx.log_d("[WEMXAndroidMediaService] createVideoImage() msg.key " + msg.key + 
										                         " , msg.path " + msg.path + 
																 " , msg.dir " + msg.dir + 
																 " , msg.height " + msg.height);
				
					var imagePath = service.media.createVideoImage(msg.path, msg.dir, msg.height);
					
					//wemx.log_d("[WEMXAndroidMediaService] createVideoImage() imagePath " + imagePath);
					wemx.sendMessage(sender, getVideoImage(msg.key, imagePath));
				}
				break;
				
			case "openVideo":
				{
					var path = msg.path;
					service.media.openVideo(path);
					//wemx.log_d("[WEMXAndroidMediaService] OpenVideo() path : " + path);
				}
				break;
				
            case "releaseVideo":
				{
					service.media.releaseVideo();
					//wemx.log_d("[WEMXAndroidMediaService] ReleaseVideo()");
				}
				break;
				
			case "videoColor":
				{
					//wemx.log_d("[WEMXAndroidMediaService] videoColor() msg.position : " + msg.position);
				
					var color = service.media.getVideoFrameAverageColor(Number(msg.position));					
					//wemx.log_d("[WEMXAndroidMediaService] videoColor() rgb : " + color);
					
					if (msg.useAddress == true) {
						var decimalColor = toDecimalColor(color);
						var address = wemx.get(msg.address);
						address.writeUWord32(decimalColor);
					} else {
						wemx.sendMessage(sender, getVideoColor(msg.position, color));
					}
				}
				break;
				
			case "imageColor":
				{
					var path = msg.path;
					var width = Number(msg.width);
					var height = Number(msg.height);
					
					//wemx.log_d("[WEMXAndroidMediaService] imageColor() : path " + msg.path + " , width : " + width + " , height : " + height);
					
					var color = service.media.getImageAverageColor(path, width, height);
					//wemx.log_d("[WEMXAndroidMediaService] imageColor() rgb : " + color);
					
					if (msg.useAddress == true) {
						var decimalColor = toDecimalColor(color);
						var address = wemx.get(msg.address);
						address.writeUWord32(decimalColor);
						wemx.log_d("[WEMXAndroidMediaService] msg.address send : " + decimalColor);
					} else {
						wemx.sendMessage(sender, getImageColor(msg.path, color));
					}
				}				
				break;
		}
    }	
	
	function getImageColor(path, color) {
		return { 'kind' : "imageColor", 'path' : path, 'color' : color }
	}
	
	function getVideoColor(position, color) {
		return { 'kind' : "videoColor", 'position' : position, 'color' : color }
	}
	
	function getVideoImage(key , imagePath) {
		return { 'kind' : "createVideoImage", 'key' : key, 'imagePath' : imagePath }
	}
	
	function toDecimalColor(color) {
		return (((Math.floor(color.r * 255.0)) << 16) + ((Math.floor(color.g * 255.0)) << 8) + (Math.floor(color.b * 255.0) << 0));
	}

	Component.onCompleted: {
        wemx.conn.messageReceived.connect(onMessageReceived);
        wemx.setService("WEMXAndroidMediaService", true);
        initService();
    }
	
	Component.onDestruction: {
		wemx.freeMedia(media);
	}
}

