import QtQuick 2.0
import HttpServer 1.0
import com.wemx 1.0
import "service/service.js" as ServiceJS
import "session.js" as Session

Item {
	id: httpServerService;

	property var defaultWebPage : "user";
	property var port : 80;
	property var privateKey;
	property var certificate;

	property var httpServerServiceDir; 	// service directory
	property var webSystemRootDir; 		// system web service root directory
	property var webUserRootDir;   		// user web service root directory
	property var defaultContent : "index.html"; 		// 기본 파일 명
	property var defaultDirectory; 		// 가본 디렉토리 user? system?............................
	property var webServices: new Object(); // api service 목록
	
	property var loginService : "/accounts/login";
	property var sessionCleanTime : 5000; // session 정리 주기 5 seconds

	visible: false;
	
	function process(http, request, response) {
	
		console.log("onNewRequest - " + "url: " + request.url.toString());
		//console.log("onNewRequest - " + "request.remoteAddress: " + request.remoteAddress);
		//console.log("onNewRequest - " + "request.method: " + String(request.method));
		//console.log("onNewRequest - " + "request.path: " + request.path);
		//console.log("onNewRequest - " + "request.httpVersion: " + request.httpVersion);
		//console.log("onNewRequest - " + "request.remotePort: " + request.remotePort);
		//console.log("onNewRequest - " + "request.body: " + request.body);
		//console.log("onNewRequest - " + "request.headers: " + request.headers);
		//console.log("onNewRequest - " + "header content: " + request.getHeaderHash("Content-Type"));
		
		var url = request.url.toString();
		if (url == "/favicon.ico") {
			reponseError(404, response);
			return;
		}
		
		var method = request.method;
		if (method == "HTTP_OPTIONS") {
			//console.log("header Access-Control-Request-Method: " + request.getHeaderHash("Access-Control-Request-Method"));		
			method = "HTTP_" + request.getHeaderHash("Access-Control-Request-Method");
		}
			
		var code = 200;
		var service = getWebService(request.path);
		if (service != undefined) {
			//console.log("find web service method is " + method);
			
			if (request.path.toLowerCase() === loginService) {
				code = service.doGet(http, wemx, getParams(url), request, response);
			} else {				
				if (Session.valid(http, wemx, request)) {
					if (method == "HTTP_GET") // get
						code = service.doGet(http, wemx, getParams(url), request, response);
					else if (method == "HTTP_POST")
						code = service.doPost(http, wemx, getParams(url), request, response);
					else if (method == "HTTP_PUT")
						code = service.doPut(http, wemx, getParams(url), request, response);
					else if (method == "HTTP_DELETE")
						code = service.doDelete(http, wemx, getParams(url), request, response);
					else {
						code = 410;
					}
				} else {
					code = 401;
				}
			}
		} else {
			code = responseWebContents(url, request, response);
		}
		
		if (code != 200) {
			reponseError(code, response);
		}
	}

	HttpServer {
		id: httpServer;
        onNewRequest: process(httpServer, request, response);
    }	
	
	Timer {
		id : sessionTimer
		interval : 5000
		running : false
		repeat : false
		onTriggered : httpServer.cleanSession();
	}
	
	function reponseError(code, response) {
		response.writeHead(code);
		switch (code) {
		case 400: response.write("Bad Request"); break;
		case 401: response.write("Unauthorized"); break;
		case 404: response.write("Not Found"); break;
		case 405: response.write("Method Not Allowed"); break;
		case 410: response.write("Gone"); break;
		case 413: response.write("Request Entity Too Large"); break;
		}
		response.end();
		console.log("reponse error : " + code);
	}

	function responseWebContents(url, request, response) {
		//console.log("responseWebContents() - " + "url: " + url);
	
		var file;
		var filePath;
		var fileContent;
		
		var list = url.split("/");
		if (list.length > 2) {
			var first = list[1].toLowerCase();
			//console.log( "first : " + first);
			if (first == "system") {
				filePath = webSystemRootDir + getUrl(list);
			} else if (first == "user") {
				filePath = webUserRootDir + getUrl(list);
			} else {
				filePath = defaultDirectory + url;
			}
		}
		else 
		{
			if (url === "/") {
				filePath = defaultDirectory + url + defaultContent;
			} else {
				filePath = defaultDirectory + url;
			}
		}
		
		//console.log("responseWebContents() - " + "filePath: " + filePath);
		
		file = wemx.allocBinFile();
		fileContent = file.readAll(filePath);
		wemx.freeBinFile(file);

		var mime = mimeTypes(filePath);

		if (fileContent && fileContent != "") {
			response.setHeader("Content-Type", mime);
			response.writeHead(200);
			
			response.write(fileContent);    
			response.end();
			return 200;
		} else {
			return 404;
		}
	}
	
	function getUrl(list) {
		var url = "/";
		
		for (var i = 2; i < list.length; i++) {
			url += list[i];
			if (i < list.length - 1) {
				url += "/";
			}
		}
		
		//console.log( "remake url : " + url );
		return url;
	}
	
	function getParams(url) {
		//console.log( "getParams");
		// 파라미터가 담길 배열
		var param = [];
	 
		// 현재 페이지의 url
		var url = decodeURIComponent(url);
		
		// url이 encodeURIComponent 로 인코딩 되었을때는 다시 디코딩 해준다.
		url = decodeURIComponent(url);
	 
		var params;
		// url에서 '?' 문자 이후의 파라미터 문자열까지 자르기
		params = url.substring( url.indexOf('?')+1, url.length );
		// 파라미터 구분자("&") 로 분리
		params = params.split("&");

		// params 배열을 다시 "=" 구분자로 분리하여 param 배열에 key = value 로 담는다.
		var size = params.length;
		var key, value;
		for(var i=0 ; i < size ; i++) {
			key = params[i].split("=")[0];
			value = params[i].split("=")[1];

			param[key] = value;
			
			//console.log( key + " :  " + value);
		}

		return param;
	}	

	function mimeTypes(filePath) {
		var fileExtension;
		var filePathSplit = filePath.split('.');
		
		if (filePathSplit.length != 0) {
			fileExtension = filePathSplit[filePathSplit.length - 1];
		}
		
		if (fileExtension == "html" || fileExtension == "htm") {
			return "text/html";
		} else if (fileExtension == "xhtml") {
			return "application/xhtml+xml";
		} else if (fileExtension == "xml") {
			return "application/xml";
		} else if (fileExtension == "png" || fileExtension == "gif") {
			return "image/png";
		} else if (fileExtension == "jpeg") {
			return "image/jpeg";
		} else if (fileExtension == "svg") {
			return "image/svg+xml";
		} else if (fileExtension == "js") {
			return "application/javascript";
		} else if (fileExtension == "css") {
			return "text/css";
		} else if (fileExtension == "opf") {
			return "application/oebps-package+xml";
		} else if (fileExtension == "ncx") {
			return "application/x-dtbncx+xml";
		} else if (fileExtension == "json") {
			return "application/json";
		} else if (fileExtension == "mp4") {
			return "video/mp4";
		} else {
			return "application/octet-stream";
		}
	}

	function getWebService(service) {
		if (!ServiceJS.isService(service)) {
			return undefined;
		}

		if (webServices[service] === undefined) {
			var stubStr = 'import QtQuick 2.0; import com.wemx 1.0; import "service/' + service.slice(1) + ".js" + '" as Js; Item { id: stub; function doGet(http, wemx, parames, request, reponse) { return Js.doGet(http,wemx, parames, request, reponse);} function doPut(http,wemx, parames, request, reponse) { return Js.doPut(http,wemx, parames, request, reponse);} function doPost(http,wemx, parames, request, reponse) { return Js.doPost(http,wemx, parames, request, reponse);} function doDelete(http,wemx, parames, request, reponse) { return Js.doDelete(http,wemx, parames, request, reponse);}}';						
			webServices[service] = Qt.createQmlObject(stubStr, httpServerService);
			//console.log("Creadted service : " + service);
		}
		
		return webServices[service]
	}

	function init() {
		var serverRoot  = wemx.getSettings().runtimeRootFolder() + "/services/WEMXHttpService/";
		var projectRoot = wemx.getSettings().projectFolder();
		
		httpServerServiceDir = serverRoot + "service/";
		webSystemRootDir 	 = serverRoot + "webroot";
		webUserRootDir 		 = projectRoot + "/webroot";
		
		loadIni();
		
		if (wemx.getSettings().getEnableSSL()) {
			httpServer.setSslFilePath(privateKey, certificate);
		}
		httpServer.listen(port);
		
		sessionTimer.repeat = true;
		sessionTimer.start();
	}

	function loadIni() {
		var projectRoot = wemx.getSettings().projectFolder();
		
		defaultWebPage = wemx.getSettings().getDefaultWebPage();
		port = wemx.getSettings().getConnectPort();
		privateKey = wemx.getSettings().getPrivateKey();
		certificate = wemx.getSettings().getCertificate();		
		
		if (defaultWebPage == "system") {
			if (!wemx.getSettings().securityEnabled()) {
				defaultContent = "main.html";
			}
			defaultDirectory = webSystemRootDir;
		} else {
			defaultDirectory = webUserRootDir;
		}
		
		if (isEmpty(privateKey)) {
			privateKey = "";
		} else {
			privateKey = projectRoot + "/" + privateKey;
		}
		
		if (isEmpty(certificate)) {
			certificate = "";
		} else {
			certificate = projectRoot + "/" + certificate;
		}

		console.log("WEMX Http Server defaultWebPage: " + defaultWebPage);
		console.log("WEMX Http Server Port: " + port);
		//console.log("WEMX Http Server privateKey: " + privateKey);
		//console.log("WEMX Http Server certificate: " + certificate);
	}
	
	function isEmpty(str) {
		return (!str || 0 === str.length);
	}

	Component.onCompleted: {
		console.log(">> start WEMX Http Server Service... ");
		init();
	}
	
	Component.onDestruction: {
		sessionTimer.repeat = false;
		sessionTimer.stop();
	}
}


