.pragma library

var apiVersion = "1.0"

function isEmpty(str) {
    return (!str || 0 === str.length);
}

function getByteLength(s,b,i,c){
	for(b=i=0;c=s.charCodeAt(i++);b+=c>>11?3:c>>7?2:1);
	return b;
}

function valueJSON(value) {
	var json = {};
	
	json["version"] = apiVersion;
	json["value"] = value;
	
	return JSON.stringify(json);
}

function valuesJSON(displayname, type, count, values) {
	var json = {};
	
	json["version"] = apiVersion;
	json["displayname"] = displayname;
	json["type"] = type;
	json["count"] = count;
	json["values"] = values;
	
	return JSON.stringify(json);
}

function writeJSON(displayname, type, count, value) {
	var json = {};
	
	json["version"] = apiVersion;
	json["displayname"] = displayname;
	json["type"] = type;
	json["count"] = count;
	
	if (type == "TEXT") {
		json["value"] = encodeURIComponent(value);
	} else {
		json["value"] = value;
	}
	
	return JSON.stringify(json);
}

function validType(type) {
	var src = type.toUpperCase();
	if (src === "BIT" || 
		src === "INT16" || 
		src === "INT32" || 
		//src === "INT64" || 
		src === "UINT16" || 
		src === "UINT32" || 
		//src === "UINT64" || 
		src === "TEXT" ||
		src === "BIN"
		) {
		return true;
	}
	
	return false;
}

function validWriteType(type) {
	var src = type.toUpperCase();
	if (src === "BIT" || 
		src === "INT16" || 
		src === "INT32" || 
		//src === "INT64" || 
		src === "UINT16" || 
		src === "UINT32" || 
		//src === "UINT64" || 
		src === "TEXT"
		) {
		return true;
	}
	
	return false;
}

function validJobType(type) {
	var src = type.toUpperCase();
	if (src === "BIT" || 
		src === "INT16" || 
		src === "INT32" || 
		//src === "INT64" || 
		src === "UINT16" || 
		src === "UINT32" || 
		//src === "UINT64" || 
		src === "TEXT"
		) {
		return true;
	}
	
	return false;
}

function error(response, displayname, type, count, error) {
	var json = {};
	
	json["version"] = apiVersion;
	json["displayname"] = displayname;
	json["type"] = type;
	if (!isEmpty(count)) {
		json["count"] = count;
	}
	json["error"] = error;
	
	var str = JSON.stringify(json)
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", "Keep-Alive");
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);	
	
	response.write(str);
	response.end();
	
	return 200;
}

/**
GET https://10.0.0.1/feeds/address
?displayname=[PLC1]X000
&type=uint32
&refresh=true
&quite=true

GET https://10.0.0.1/feeds/address
?displayname=[DISPLAY]USR00000.00
&type=bit
&count=4

GET https://10.0.0.1/feeds/address
?displayname=[DISPLAY]USR00000
&type=bin
&count=1024
*/
function doGet(http, wemx, parames, request, response) {
	
	var displayname = parames["displayname"];
	var type = parames["type"];
	//var refresh = parames["refresh"];
	var quite = parames["quite"];
	var count = parames["count"];
	var isArray = false;
	
	if (isEmpty(displayname)) {
		console.log("405 Not found displayname");
		return 405;
	}
	
	if (isEmpty(type)) {
		console.log("405 Not found type");
		return 405;
	}	
	
	type = type.toUpperCase();
	if (type === "TEXT" || type === "BIN") {
		if (isEmpty(count)) {
			console.log("405 Not found count");
			return 405;
		}
	}
	
	if (type != "TEXT") {
		if (!isEmpty(count) && Number(count) > 1) {
			isArray = true;
		}
	}
	
	if (!validType(type)) {
		console.log("Wrong parameter type : " + type);
		return error(response, displayname, type, count, "Invalid type");
	}
	
	var addr = wemx.get(displayname);
	if (addr.isNull()) {
		console.log("Invalid address : " + displayname);
		return error(response, displayname, type, count, "Invalid address");
	}
	
	var ut = type.toUpperCase();
	var str;
	if (isArray) {
	
		var values;
		if (ut === "BIT") {
			values = new Array();
			
			var len = (count / 8) + ((count % 8) ? 1 : 0);
			//console.log( " len : " + len);
			
			var bits = new ArrayBuffer(len);
			addr.readBits(0, bits, 0, count);
			
			var u8 = new Uint8Array(bits);
			var index = 0;
			for (var c = 0; c < len && index < count; c++) {
				for (var i = 0; i < 8 && index < count; i++) {
					values[index] = (u8[c] >> i) & 1;
					index++;
				}
			}
			
		} else if (ut === "BIN") {
		
			values = wemx.getBytes(addr, count);
			response.setHeader("Content-Type", "application/octet-stream");
			response.setHeader("Content-Length", count);
			response.writeHead(200);
			
			response.write(values);
			response.end();
			return 200;
			
		} else {
			values = new Array();
			
			var byteArray;
			if (ut === "INT32" || ut === "UINT32") {
				
				var buffer = new ArrayBuffer(count * 8 * 4);
				addr.readBits(0, buffer, 0, count * 8 * 4);
				
				var typeArray;
				if (ut === "UINT32") {					
					typeArray = new Uint32Array(buffer);
				} else {
					typeArray = new Int32Array(buffer);
				}
				
				for (var i = 0; i < count; i++) {
					//console.log( "[" + i + "] : " + typeArray[i] );
					values.push(typeArray[i]);
				}

			} else {
			
				var buffer = new ArrayBuffer(count * 8 * 2);
				addr.readBits(0, buffer, 0, count * 8 * 2);
				
				var typeArray;
				if (ut === "UINT16") {					
					typeArray = new Uint16Array(buffer);
				} else {
					typeArray = new Int16Array(buffer);
				}
				
				for (var i = 0; i < count; i++) {
					//console.log( "[" + i + "] : " + typeArray[i] );
					values.push(typeArray[i]);
				}
			}
		}
		
		str = valuesJSON(displayname, type, Number(count), values);
	} else {
		var value;
		if (ut === "BIT") {
			value = addr.readBit();
		} else if (ut === "INT16") {
			value = addr.readWord16();
		} else if (ut === "INT32") {
			value = addr.readWord32();
		//} else if (ut === "INT64") {
		} else if (ut === "UINT16") {
			value = addr.readUWord16();
		} else if (ut === "UINT32") {
			value = addr.readUWord32();
		//} else if (ut === "UINT64") {
		} else if (ut === "TEXT") {
			value = addr.readString(Number(count));
			value = encodeURIComponent(value);
		} else {
			value = 0;
		}
		
		str = valueJSON(value);
	}
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);	
	
	//console.log("response body :" + str);
	response.write(str);	
	response.end();
	
	return 200;
}

/**
Content-Type: application/json; charset=UTF-8

PUT https://10.0.0.1/feeds/address
?displayname=[DISPLAY]USR00000
&type=text

{
    "value": "yoyo"
}

Content-Type: application/json; charset=UTF-8

PUT https://10.0.0.1/feeds/address
?displayname=[DISPLAY]USR00000
&type=int16
&count=2

{
    "values": [123, -4565]
}
*/
function doPut(http, wemx, parames, request, response) {
	var displayname = parames["displayname"];
	var type = parames["type"];	
	//var refresh = parames["refresh"];
	var quite = parames["quite"];
	var count = parames["count"];
	//var value = parames["value"];
	//var values = parames["values"];
	var isArray = false;
	
	if (isEmpty(displayname)) {
		console.log("405 Not found displayname");
		return 405;
	}
	
	if (isEmpty(type)) {
		console.log("405 Not found type");
		return 405;
	}	
	
	type = type.toUpperCase();
	if (type === "TEXT") {
		if (!isEmpty(value) && isEmpty(count)) {
			console.log("405 Not found count");
			return 405;
		}
	}
	
	if (!isEmpty(count) && Number(count) > 1) {
		if (type != "TEXT")
			isArray = true;
	}
	
	if (!validWriteType(type)) {
		console.log("Wrong parameter type : " + type);
		return error(response, displayname, type, count, "Invalid type");
	}
	
	var addr = wemx.get(displayname);
	if (addr.isNull()) {
		console.log("Invalid address : " + displayname);
		return error(response, displayname, type, count, "Invalid address");
	}	
	
	//var json = JSON.parse(request.body);
	
	var str;
	if (isArray) {
		var values = parames["values"];
		//console.log("values : " + values
		var valueList = JSON.parse(values);
		var resultValues = new Array();
		if (type === "BIT") {
			
			var len = Math.floor((count / 8) + ((count % 8) ? 1 : 0));
			//console.log( " len : " + len);
			
			var bits = new ArrayBuffer(len);
			var u8 = new Uint8Array(bits);
			
			var c = 0;
			var shift = 0;
			for (var i = 0; i < valueList.length; i++)
			{
				c = Math.floor(i / 8);				
				shift = i % 8;
				//console.log("valueList[" + i + "] : " + valueList[i]);
				u8[c] = u8[c] | (valueList[i] == "true" ? 1 : ((valueList[i] == 1) ? 1 : 0)) << shift;
				//console.log("u8[" + c + " ] : " + u8[c]);
				resultValues.push(valueList[i]);
			}
			
			addr.writeBits(0, bits, 0, count);
			
		} else {
			
			var byteArray;
			if (type === "INT32" || type === "UINT32") {
				
				var buffer = new ArrayBuffer(count * 8 * 4);
				var typeArray
				if (type === "UINT32") {
					typeArray = new Uint32Array(buffer);
				} else {
					typeArray = new Int32Array(buffer);
				}
				
				for (var i = 0; i < valueList.length; i++) {
					typeArray[i] = Number(valueList[i]);
					//console.log("typeArray[" + i + " ] : " + typeArray[i]);
				}
				
				
				addr.writeBits(0, buffer, 0, count * 8 * 4);

			} else if (type === "INT16" || type === "UINT16") {
			
				var buffer = new ArrayBuffer(count * 8 * 2);
				var typeArray
				if (type === "UINT16") {
					typeArray = new Uint16Array(buffer);
				} else {
					typeArray = new Int16Array(buffer);
				}
				
				for (var i = 0; i < valueList.length; i++) {
					typeArray[i] = Number(valueList[i]);
					//console.log("typeArray[" + i + " ] : " + typeArray[i]);
				}
				
				
				addr.writeBits(0, buffer, 0, count * 8 * 2);
			}
			else {
				return error(response, displayname, type, count, "Invalid type");
			}
		}
		
		str = valuesJSON(displayname, type, Number(count), resultValues);
	} else {
		var value = parames["value"];
		if (type === "BIT") {
			var boolVal = value == "true" ? true : ((value == 1) ? true : false);
			addr.writeBit(boolVal);
			value = boolVal;
		} else if (type === "INT16") {
			addr.writeWord16(value);
			value = addr.readWord16();
		} else if (type === "INT32") {
			addr.writeWord32(value);
			value = addr.readWord32();
		//} else if (type === "INT64") {
		} else if (type === "UINT16") {
			addr.writeUWord16(value);
			value = addr.readUWord16();
		} else if (type === "UINT32") {
			addr.writeUWord32(value);
			value = addr.readUWord32();
		//} else if (type === "UINT64") {
		} else if (type === "TEXT") {
			
			var min = 0;
			if (isEmpty(value)) {
				var json = JSON.parse(request.body);
				value = decodeURIComponent(json["value"]);
				//console.log("value : " + value);
				min = count = json["count"];
				
				//console.log("count : " + count);
			} else {
				//min = count < value.length ? count : value.length;
				if (count <= 0) {
					count = getByteLength(value);
				} else {
					min = count;
				}
			}			
			
			//console.log("min : " + min);
			addr.writeString(min, value);
			value = addr.readString(min);
			//console.log("value : " + value);
		} else {
			return error(response, displayname, type, count, "Invalid type");
		}
		
		if (quite == "true") {
			if (type != "TEXT") {
				str = String(value);
			} else {
				str = encodeURIComponent(value);
			}
		} else {
			str = writeJSON(displayname, type, 1, value);
		}
	}
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);
	
	response.write(str);
	//console.log("response body :" + str);
	response.end();

	return 200
}

function errorJob(jobObj, error) {
	var json = {};
	
	json["job"] = jobObj.job;
	json["displayname"] = jobObj.displayname;
	json["type"] = jobObj.type;
	json["count"] = jobObj.count;
	
	if (!isEmpty(jobObj.job) && jobObj.job.toLowerCase() == "write") {
		if (count == 1 && !isEmpty(jobObj.value)) {
			json["value"] = jobObj.value;
		} else if (count > 1 && !isEmpty(jobObj.values)) {
			json["values"] = jobObj.values;
		}
	}
	json["error"] = error;
	
	return json;
}

function readValues(wemx, jobObj) {

	var displayname = jobObj.displayname;
	var addr = wemx.get(displayname);
	if (addr.isNull()) {
		return errorReadJob(jobObj, "Invalid address");
	}
	
	var type = jobObj.type.toUpperCase();
	if (!validJobType(type)) {
		return errorReadJob(jobObj, "Invalid type");
	}
	
	var count = jobObj.count;
	if (count <= 0) {
		return errorReadJob(jobObj, "Invalid count");
	}
	
	var values = new Array();
	if (type === "BIT") {
		
		var len = (count / 8) + ((count % 8) ? 1 : 0);
		//console.log( " len : " + len);
		
		var bits = new ArrayBuffer(len);
		addr.readBits(0, bits, 0, count);
		
		var u8 = new Uint8Array(bits);
		var index = 0;
		for (var c = 0; c < len && index < count; c++) {
			for (var i = 0; i < 8 && index < count; i++) {
				values[index] = (u8[c] >> i) & 1;
				index++;
			}
		}
		
	} else if (type == "TEXT") {
		values[0] = encodeURIComponent(addr.readString(Number(count)));
	} else {
		var byteArray;
		if (type === "INT32" || type === "UINT32") {
			
			var buffer = new ArrayBuffer(count * 8 * 4);
			addr.readBits(0, buffer, 0, count * 8 * 4);
			
			var typeArray;
			if (type === "UINT32") {					
				typeArray = new Uint32Array(buffer);
			} else {
				typeArray = new Int32Array(buffer);
			}
			
			for (var i = 0; i < count; i++) {
				//console.log( "[" + i + "] : " + typeArray[i] );
				values.push(typeArray[i]);
			}

		} else if (type === "INT16" || type === "UINT16") {
			var buffer = new ArrayBuffer(count * 8 * 4);
			addr.readBits(0, buffer, 0, count * 8 * 4);
			
			var typeArray;
			if (type === "UINT16") {					
				typeArray = new Uint16Array(buffer);
			} else {
				typeArray = new Int16Array(buffer);
			}
			
			for (var i = 0; i < count; i++) {
				//console.log( "[" + i + "] : " + typeArray[i] );
				values.push(typeArray[i]);
			}
		} else {
			return errorReadJob(jobObj, "Invalid type");
		}		
	}
	
	var rObj = {};
	
	rObj["job"] = "read";
	rObj["displayname"] = displayname;
	rObj["type"] = type;
	rObj["count"] = count;
	
	if (count == 1) {
		rObj["value"] = values[0];
	} else if (count > 1) {
		rObj["values"] = values;
	}
	
	return rObj;
}

function writeValues(wemx, jobObj) {
	var displayname = jobObj.displayname;
	var addr = wemx.get(displayname);
	if (addr.isNull()) {
		return errorReadJob(jobObj, "Invalid address");
	}
	
	var type = jobObj.type.toUpperCase();
	if (!validJobType(type)) {
		return errorReadJob(jobObj, "Invalid type");
	}
	
	var count = jobObj.count;
	if (count <= 0) {
		return errorReadJob(jobObj, "Invalid count");
	}
	
	var resultValues;
	if (type == "TEXT") {
		var value = decodeURIComponent(jobObj.value);
		// 아래 코드는 필요없을듯???
		/*
		if (value.length < 1) {
			return errorReadJob(jobObj, "Invalid value");
		}
		*/
	
		var min = count;
		addr.writeString(min, value);
		resultValues = addr.readString(min);
	} else {
		var values;
		if (count == 1) {
			values = new Array();
			values[0] = jobObj.value;
		} else if (count > 1) {
			values = jobObj.values;
		}
		//console.log("values : " + values);
		
		if (values.length != count) {
			return errorReadJob(jobObj, "Mismatched count and value[s]");
		}
		
		var valueList = values;//JSON.parse(values);
		resultValues = new Array();
		if (type === "BIT") {
			var len = Math.floor((count / 8) + ((count % 8) ? 1 : 0));
			//console.log( " len : " + len);
			
			var bits = new ArrayBuffer(len);
			var u8 = new Uint8Array(bits);
			
			var c = 0;
			var shift = 0;
			for (var i = 0; i < valueList.length; i++)
			{
				c = Math.floor(i / 8);				
				shift = i % 8;
				//console.log("valueList[" + i + "] : " + valueList[i]);
				u8[c] = u8[c] | (valueList[i] == "true" ? 1 : ((valueList[i] == 1) ? 1 : 0)) << shift;
				//console.log("u8[" + c + " ] : " + u8[c]);
				resultValues.push(valueList[i]);
			}
			
			addr.writeBits(0, bits, 0, count);
		} else {
			var byteArray;
			if (type === "INT32" || type === "UINT32") {
				
				var buffer = new ArrayBuffer(count * 8 * 4);
				var typeArray
				if (type === "UINT32") {
					typeArray = new Uint32Array(buffer);
				} else {
					typeArray = new Int32Array(buffer);
				}
				
				for (var i = 0; i < valueList.length; i++) {
					typeArray[i] = Number(valueList[i]);
					//console.log("typeArray[" + i + " ] : " + typeArray[i]);
				}
				
				addr.writeBits(0, buffer, 0, count * 8 * 4);

			} else if (type === "INT16" || type === "UINT16") {
				//console.log("count : " + count + " , valueList.length : " + valueList.length);
				var buffer = new ArrayBuffer(count * 8 * 2);
				var typeArray
				if (type === "UINT16") {
					typeArray = new Uint16Array(buffer);
				} else {
					typeArray = new Int16Array(buffer);
				}
				
				for (var i = 0; i < valueList.length; i++) {
					typeArray[i] = Number(valueList[i]);
					//console.log("typeArray[" + i + " ] : " + typeArray[i]);
				}
				
				addr.writeBits(0, buffer, 0, count * 8 * 2);
			} else {
				return error(response, displayname, type, count, "Invalid type");
			}
		}
	}	
	
	var rObj = {};
	
	rObj["job"] = "write";
	rObj["displayname"] = displayname;
	rObj["type"] = type;
	rObj["count"] = count;
	
	if (type == "TEXT") {
		rObj["value"] = encodeURIComponent(resultValues);
	} else if (count == 1) {
		rObj["value"] = resultValues[0];
	} else if (count > 1) {
		rObj["values"] = resultValues;
	}
	
	return rObj;
}

function processJob(wemx, jobObj) {	
	var job = jobObj.job;
	//console.log ( "job : " + job);
	if (isEmpty(job)) {
		return errorJob(jobObj, "Not found job");
	}
	
	var displayname = jobObj.displayname;
	//console.log ( "displayname : " + displayname);
	if (isEmpty(displayname)) {
		return errorJob(jobObj, "Not found displayname");
	}
	
	var type = jobObj.type;
	//console.log ( "type : " + type);
	if (isEmpty(type)) {
		return errorJob(jobObj, "Not found type");
	}
	
	var count = jobObj.count;
	//console.log ( "count : " + count);
	if (isEmpty(count)) {
		return errorJob(jobObj, "Not found count");
	}
	
	var value = jobObj.value;
	//console.log ( "value : " + value);
	
	var values = jobObj.values;
	//console.log ( "values : " + values);	

	if (job == "read") {
		return readValues(wemx, jobObj);
	} else if (job == "write") {
		if (isEmpty(value) && isEmpty(values)) {
			return errorJob(jobObj, "Not found value or values");
		}
		
		return writeValues(wemx, jobObj);
	} 

	return errorJob(jobObj, "Invalid job");
}

/**
Content-Type: application/json; charset=UTF-8

POST https://10.0.0.1/feeds/address

{
    "jobs":
    [
        {
            "job": "read",
            "displayname": "[DISPLAY]USR00000.00",
            "type": "bit",
            "count": 2
        },
        {
             "job": "read",
            "displayname": "[PLC1]X000",
            "type": "text",
            "count": 7
        },
        {
             "job": "write",
            "displayname": "[PLC2]DN0000",
            "type": "uint32",
            "count": 1,
            "value": 33344334
        }
    ]
}
*/
function doPost(http, wemx, parames, request, response) {
	var json = JSON.parse(request.body);
	
	var jobs = json.jobs;
	if (isEmpty(jobs)) {
		return 405;
	}
	
	if (!Array.isArray(jobs)) {
		return 405;
	}
	
	var results = new Array();
	
	//console.log ( "jobs is array : " + jobs.length);
	for (var i = 0 ; i < jobs.length; i++ ) {
	
		var jobObj = jobs[i];
		//console.log ("jobObj[" + i + "]");
		
		results[i] = processJob(wemx, jobObj);
	}
	
	var json = {};
	json["version"] = apiVersion;
	json["results"] = results;
	
	var body = JSON.stringify(json);
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", body.length);
	response.writeHead(200);
	
	//console.log("response body :" + body);
	response.write(body);
	response.end();

	return 200;
}

function doDelete(http, wemx, parames, request, response) {
	return 410
}