var API_VERSION = "1.0"
var ALARM_DISPLAY_MODE = 2; // active

var alarmService = undefined;
var sResponse = undefined;
var block;
var filterList = undefined;
var alarmID;

function isEmpty(str) {
	if (typeof str === 'string' || str instanceof String)
		return (!str || 0 === str.length);
	return true;
}

function init(wemx, response) {
	if (alarmService === undefined) {
		alarmService = wemx.getService("WEMXAlarmService");
		wemx.conn.messageReceived.connect(onMessageReceived)
	}
	
	this.sResponse = response;
	this.block = 1;
	this.filterList = undefined;
	this.alarmID = undefined;
}

function makeDate(date, time) {
	if (isEmpty(date) || isEmpty(time)) {
		return "";
	}

	var str = "20" + date + " " + time;
	var date = new Date(str);	
	//console.log( "makeDate() : " + date.toISOString());
	
	return date.toISOString();
}

function onMessageReceived(sender, msg) 
{
	if (sResponse === undefined) {
		//console.log("sResponse is undefined");
		return;
	}
	
	//console.log( "onMessageReceived() " + msg.kind);
    switch(msg.kind)
    {
        case "getRecordResult":
            var recordList = msg.result;
			//console.log("msg.dbAlarmCount : " + msg.dbAlarmCount);
			//console.log("recordList.length : " + recordList.length);
			
			var result = [];
            for (var i = 0; i < recordList.length; i++) {
				var record = recordList[i];

				//console.log("record[" + i + "] id - " + record.record_id);
				//console.log("record[" + i + "] status - " + record.status);
				//console.log("record[" + i + "] date - " + record.date);
				//console.log("record[" + i + "] message - " + record.message);
				//console.log("record[" + i + "] occur - " + record.occur);
				//console.log("record[" + i + "] recovery - " + record.recovery);
				//console.log("record[" + i + "] ack - " + record.ack);
				//console.log("record[" + i + "] count - " + record.count);
				//console.log("record[" + i + "] level - " + record.level);
				
				var data = {};
				data["index"] = i;
				data["id"] = record.record_id;
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("message") != -1)) {
					data["message"] = encodeURIComponent(record.message);
				}
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("triggered") != -1)) {
					data["triggered"] = makeDate(record.date, record.occur);
				}
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("acknowledged") != -1)) {
					data["acknowledged"] = makeDate(record.date, record.ack);
				}
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("recovered") != -1)) {
					data["recovered"] = makeDate(record.date, record.recovery);
				}
				
				if (record.status == 1) { // trigger 
					if (filterList == undefined || (filterList != undefined && filterList.indexOf("status") != -1)) {
						data["status"] = "triggered";
					}
				} else if (record.status == 0) { // recovery
					if (filterList == undefined || (filterList != undefined && filterList.indexOf("status") != -1)) {
						data["status"] = "recovered";
					}
				} else { // ack
					if (filterList == undefined || (filterList != undefined && filterList.indexOf("status") != -1)) {
						data["status"] = "acknowledged";
					}
				}
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("count") != -1)) {
					data["count"] = record.count;
				}
				
				if (filterList == undefined || (filterList != undefined && filterList.indexOf("level") != -1)) {
					data["level"] = record.level;
				}
				result[i] = data;
            }

			var json = {};
			
			json["version"] = API_VERSION;
			json["block"] 	= block;
			json["type"] 	= "active";
			json["total"] 	= msg.dbAlarmCount;
			json["count"] 	= recordList.length;
			json["results"] = result;
			
			var str = JSON.stringify(json);
			//console.log("response body :" + str);
			
			sResponse.setHeader("Content-Type", "application/json; charset=UTF-8;");
			sResponse.setHeader("Connection", "Keep-Alive");
			sResponse.setHeader("Keep-Alive", "timeout=5, max=100");
			sResponse.setHeader("Content-Length", str.length);
			sResponse.writeHead(200);
			
			sResponse.write(str);
			sResponse.end();
			sResponse = undefined;
			result = undefined;
            break;
			
			/*
		case "notifyAlarm":
			var json = {};
			
			json["version"] = API_VERSION;
			json["block"] 	= block;
			json["type"] 	= "history";
			json["id"] 	= alarmID;
			
			response.writeHead(200);
			response.setHeader("Content-Type", "application/json; charset=UTF-8");
			var str = JSON.stringify(json);
			console.log("response body :" + str);
			response.write(str);
			response.end();
			break;
			*/
    }
}

/**
GET https://10.0.0.1/feeds/alarm/active
?block=1
&startindex=0
&count=10
&filter=message,status,triggered,acknowledged,recovered,count,level
*/
function doGet(http, wemx, parames, request, response) {

	init(wemx, response);
	
	// block 번호
	var block = parames["block"];

	// startindex 로 부터 가져옴.
	var startindex = parames["startindex"];
	
	// count 개수 만큼 가져옴 
	var count = parames["count"];	
	
	// index, id 항목은 기본으로 가져옴. 필더 항목이 없으면 모든 항목 가져옴.
	var filters = parames["filters"];
	
	
	if (isEmpty(block)) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	if (block < 1 || block > 8) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	this.block = block;	

	// 전체 알람 목록 요청
	if (isEmpty(startindex)) {
		startindex = 0;
	}
	
	if (isEmpty(count)) {
		count = -1;
	}	
	
	if (isEmpty(filters) === false) {
		filters = filters.toLowerCase();
		var list = filters.split(",");
		if (list != undefined && list.length > 0) {
			filterList = list;
		}
	}

    var msg = { 
		kind: "getRecord", 
		blockNumber: block, 
		mode: ALARM_DISPLAY_MODE, 
		startPos: startindex, 
		count: count
	};
	
	wemx.sendFastMessage(alarmService, msg);
	return 200;
}

function doPost(http, wemx, parames, request, response) {
	return 410
}

/**
	PUT https://10.0.0.1/feeds/alarm/active?block=1&id=1
	PUT https://10.0.0.1/feeds/alarm/active?block=1
*/
function doPut(http, wemx, parames, request, response) {

	//init(wemx, response);
	
	// block 번호
	var block = parames["block"];
	
	var id = parames["id"];	
	
	if (isEmpty(block)) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	if (block < 1 || block > 8) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	this.block = block;	
	
	var msg;				
	if (isEmpty(id)) {
		msg = { kind: "ackAllRecord",
				blockNumber: block,
				mode: ALARM_DISPLAY_MODE };
        
	} else {
		this.alarmID = id;
		//console.log("alarmID : " + alarmID);
		
		msg = { kind: "ackRecord",
				blockNumber: block,
				mode: ALARM_DISPLAY_MODE,
				recordId: id };
	}
	
    wemx.sendFastMessage(alarmService, msg);
	
	var json = {};
			
	json["version"] = API_VERSION;
	json["block"] 	= block;
	json["type"] 	= "active";
	
	if (!isEmpty(id)) {
		json["id"] 	= alarmID;
	}
	
	var str = JSON.stringify(json);
	//console.log("response body :" + str);
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);
	
	response.write(str);
	response.end();
	return 200;	
}

/**
	DELETE https://10.0.0.1/feeds/alarm/active?block=1&id=1
	DELETE https://10.0.0.1/feeds/alarm/active?block=1
	DELETE https://10.0.0.1/feeds/alarm/active?block=1&filter=acknowleged
	DELETE https://10.0.0.1/feeds/alarm/active?block=1&filter=recovered
*/
function doDelete(http, wemx, parames, request, response) {
	//init(wemx, response);
	
	// block 번호
	var block = parames["block"];
	
	var id = parames["id"];	
	
	var filters = parames["filters"];
	
	if (isEmpty(block)) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	if (block < 1 || block > 8) {
		console.log( "405 Wrong parameter block : " + block);
		return 405;
	}
	
	this.block = block;	
	
	var msg;
	
	var useFilter = false;	
	if (!isEmpty(filters)) {
		filters = filters.toLowerCase();
		if (filters == "acknowleged") {
		    useFilter = true;
			msg = { kind: "delAckRecord",
				blockNumber: block,
				mode: ALARM_DISPLAY_MODE };
		} else if (filters == "recovered") {
			useFilter = true;
			msg = { kind: "delRecoveryRecord",
				blockNumber: block,
				mode: ALARM_DISPLAY_MODE };
		} else {
			console.log( "405 Wrong parameter filters : " + filters);
			return 405;
		}
	}
	
	if (useFilter == false) {
		if (isEmpty(id)) {			
			msg = { kind: "delAllRecord",
					blockNumber: block,
					mode: ALARM_DISPLAY_MODE };
			
		} else {
			this.alarmID = id;
			//console.log("alarmID : " + alarmID);
			
			msg = { kind: "delRecord",
					blockNumber: block,
					mode: ALARM_DISPLAY_MODE,
					recordId: id};
		}
	}
	
    wemx.sendFastMessage(alarmService, msg);
	
	var json = {};
			
	json["version"] = API_VERSION;
	json["block"] 	= block;
	json["type"] 	= "active";
	
	if (!isEmpty(id)) {
		json["id"] 	= alarmID;
	}
	
	var str = JSON.stringify(json);
	//console.log("response body :" + str);
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);
	
	response.write(str);
	response.end();
	return 200;	
}