/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.0
import QtQuick.Controls 1.0
import "submenus"
import com.wemx 1.0
import QtQuick.Window 2.2

Item {
	id: settingMain;
	//anchors.fill: parent;
	width: 800;
	height: 480;
	visible: true;
	//z: 3;

	// property string fontFamily: "";

	property int touchSoundType: 0;
	property int beforeId: 0;

	property string trSettings: "Settings";
	property string trDisplayDevice: "Display Device";
	property string trPlayer: "Player";
	property string trHMX: "HMX";
	property string trMemory: "Memory";
	property string trEthernet: "Ethernet";
	property string trSerial: "Serial";
	property string trSoundAndDisplay: "Sound & Display";
	property string trLanguageAndTime: "Language & Time";
	property string trEventLog: "Event Log";
	property string trSecurity: "Secutity";
	property string trExit: "Exit";

	property int launchMode: 0;
	property int exitMode: 0;
	
	property real mRatio: 1.0;
	property real mRatioFont: 1.0;
	
	onVisibleChanged : {
		if (visible == false)
			return;
			
		//real dpi = qApp->primaryScreen()->logicalDotsPerInch();
		//console.log("dpi : " + dpi);
			
		console.log("settingMain.width : " + settingMain.width);
		console.log("settingMain.height : " + settingMain.height);
		var height = Math.max(settingMain.width, settingMain.height);
		var width = Math.max(settingMain.width, settingMain.height);
		mRatio = Math.min(height/480.0, width/800.0);
		mRatioFont = Math.min((height*100.0)/(96*480.0), (width*100.0)/(96*800.0))
		
		console.log("mRatio : " + mRatio);
		console.log("mRatioFont : " + mRatioFont);
	}
	
	MouseArea {
		anchors.fill: parent

		onClicked: {
			languageTimeInfo.cancelCombo();
		}
	}

	Image {
		x: 0
		y: 0
		width: parent.width
		height: parent.height
		source: "bg_setting.png";
	}

	Rectangle {
		x: 0
		y: 0
		width: parent.width
		height: 43 * mRatio

		Image {
			x: 0
			y: 0
			width: parent.width
			height: 43 * mRatio
			source: "bg_top.png";
		}

		Image {
			id: settingIcon
			width: 40 * mRatio
			height: 40 * mRatio
			source: "setting_main_icon.png";
		}

		Text {
			x: 40 * mRatio
			width: parent.width - ((41 + 40) * mRatio)
			height: parent.height
			verticalAlignment: Text.AlignVCenter;
			font.bold: true
			font.pointSize: 11 * mRatioFont
			color: "#FFFFFFFF"
			text: trSettings;
		}

		Image {
			id: settingClose
			anchors.right: parent.right			
			width: 41 * mRatio
			height: 41 * mRatio			
			source: settingCloseArea.pressed ? "setting_main_close_icon_sel.png" : "setting_main_close_icon.png";
		}

		MouseArea {
			id: settingCloseArea
			width: 41 * mRatio
			height: 41 * mRatio
			anchors.right: parent.right
			propagateComposedEvents: true
			onPressed: {
				// settingMain.playSound(touchSoundType);
			}
			onClicked: {	
				serviceMain.visible = false; //settingMain.visible = false;
				mouse.accepted = false
			}
		}
	}

	Rectangle {
		y: 43 * mRatio
		width: 190 * mRatio
		height: parent.height - (43 * mRatio)
		color: "#00000000";

		ListModel {
			id: settingMainMenuListModel

			ListElement {
				settingMainMenuListItemId: 1;
				settingMainMenuIcon: "setting_main_display_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_display_info_icon_sel.png";
				isSelectedMenu: false;
			}
			ListElement {
				settingMainMenuListItemId: 2;
				settingMainMenuIcon: "setting_main_player_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_player_info_icon_sel.png";
				isSelectedMenu: false;
			}
			ListElement {
				settingMainMenuListItemId: 3;
				settingMainMenuIcon: "setting_main_hmx_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_hmx_info_icon_sel.png";
				isSelectedMenu: false;
			}
			
			ListElement {
				settingMainMenuListItemId: 4;
				settingMainMenuIcon: "setting_main_memory_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_memory_info_icon_sel.png";
				isSelectedMenu: false;
			}
			
			/*
			ListElement {
				settingMainMenuListItemId: 5;
				settingMainMenuIcon: "setting_main_ethernet_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_ethernet_info_sel.png";
				isSelectedMenu: false;
			}
			*/
			ListElement {
				settingMainMenuListItemId: 6;
				settingMainMenuIcon: "setting_main_serial_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_serial_info_icon_sel.png";
				isSelectedMenu: false;
			}
			/*
			ListElement {
				settingMainMenuListItemId: 7;
				settingMainMenuIcon: "setting_main_sound_display_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_sound_display_info_icon_sel.png";
				isSelectedMenu: false;
			}
			*/
			ListElement {
				settingMainMenuListItemId: 8;
				settingMainMenuIcon: "setting_main_language_time_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_language_time_info_icon_sel.png";
				isSelectedMenu: false;
			}
			ListElement {
				settingMainMenuListItemId: 9;
				settingMainMenuIcon: "setting_main_event_log_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_event_log_info_icon_sel.png";
				isSelectedMenu: false;
			}
			/*
			ListElement {
				settingMainMenuListItemId: 10;
				settingMainMenuIcon: "setting_main_security_info_icon.png";
				settingMainMenuSelectedIcon: "setting_main_security_info_icon_sel.png";
				isSelectedMenu: false;
			}
			*/
			ListElement {
				settingMainMenuListItemId: 11;
				settingMainMenuIcon: "setting_main_close_icon.png";
				settingMainMenuSelectedIcon: "setting_main_close_icon_sel.png";
				isSelectedMenu: false;
			}
		}

		ListView {
			id: settingMainMenuListView;
			anchors.fill: parent;
			orientation: ListView.Vertical;
			clip: true
			model: settingMainMenuListModel;
			delegate: settingMainMenuDelegate;
            interactive : false
		}

		Component {       
			id: settingMainMenuDelegate;

			Item {
				width: parent.width
				height: 40 * mRatio
				visible: settingMainMenuListItemId == 11 && Qt.platform.os == "windows" && exitMode == 0 ? false : true
				
				Image {
					width: 190 * mRatio
					height: 39 * mRatio
					source: isSelectedMenu ? "btn_sel.png" : "btn_nor.png";
				}

				Image {
					width: 40 * mRatio
					height: 39 * mRatio
					anchors.verticalCenter: parent.verticalCenter;
					source: isSelectedMenu ? settingMainMenuSelectedIcon : settingMainMenuIcon;
				}

				Text {
					x: 48 * mRatio
					width: parent.width - (48 * mRatio)
					height: 39 * mRatio
					verticalAlignment: Text.AlignVCenter;
 					font.bold: true
					font.pointSize: 10 * mRatioFont
					color: isSelectedMenu ? "#FFFFFFFF" : "#FF616161"
					text: settingMain.getMainMenuListString(settingMainMenuListItemId);
				}

				MouseArea {
					id: settingMainMenuMouseArea
					anchors.fill: parent
					propagateComposedEvents: true
					onPressed: {
						// settingMain.playSound(touchSoundType);
					}
                    onClicked: {
						for (var i = 0; i < settingMainMenuListModel.count ; i++) {
							if (settingMainMenuListModel.get(i).settingMainMenuListItemId == settingMainMenuListItemId) {
								settingMainMenuListModel.set(i, { 'isSelectedMenu': true });
							}
							else {
								settingMainMenuListModel.set(i, { 'isSelectedMenu': false });
							}

							if (settingMain.beforeId != 0) {
								setVisiblitySettingInfo(settingMain.beforeId, false);
							}

							setVisiblitySettingInfo(settingMainMenuListItemId, true);
							settingMain.beforeId = settingMainMenuListItemId;
						}

						mouse.accepted = false
					}
				}
			}
		}
	}
	
	Item {
		id: contentsArea

		x: 191 * mRatio
		y: 43 * mRatio
		width: parent.width - (191 * mRatio)
		height: parent.height - (43 * mRatio)
		clip: true
		
		SettingDisplayInfo { 
			id: displayInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}

		SettingPlayerInfo { 
			id: playerInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}

		SettingHmxInfo { 
			id: hmxInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		
		SettingMemoryInfo { 
			id: memoryInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		
		/*
		SettingEthernetInfo { 
			id: ethernetInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		*/
		SettingSerialInfo { 
			id: serialInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		/*
		SettingSoundDisplayInfo { 
			id: soundDisplayInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		*/

		SettingLanguageTimeInfo { 
			id: languageTimeInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}

		SettingEventLogInfo { 
			id: eventLogInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}

		/*
		SettingSecurityInfo { 
			id: securityInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
		*/
		SettingExitInfo { 
			id: exitInfo
			width: parent.width; 
			height: parent.height; 
			visible: false
		}
	}

	function getMainMenuListString(itemId)
	{
		switch (itemId)
		{
			case 1:
				return trDisplayDevice;
				break;
			case 2:
				return trPlayer;
				break;
			case 3:
				return trHMX;
				break;
			case 4:
				return trMemory;
				break;
			case 5:
				return trEthernet;
				break;
			case 6:
				return trSerial;
				break;
			case 7:
				return trSoundAndDisplay;
				break;
			case 8:
				return trLanguageAndTime;
				break;
			case 9:
				return trEventLog;
				break;
			case 10:
				return trSecurity;
				break;
			case 11:
				return trExit;
				break;
			default:
				return "";
		}
	}		

	function setVisiblitySettingInfo(itemId, isVisible)
	{
		switch (itemId)
		{
			case 1:
				if (isVisible == true) {
					displayInfo.visible = true;
				}
				else {
					displayInfo.visible = false;
				}
				break;
			case 2:
				if (isVisible == true) {
					playerInfo.visible = true;
				}
				else {
					playerInfo.visible = false;
				}
				break;
			case 3:
				if (isVisible == true) {
					hmxInfo.visible = true;
				}
				else {
					hmxInfo.visible = false;
				}
				break;
			case 4:
				if (isVisible == true) {
					memoryInfo.setMemoryInfos(wemx.getSettings().getMemoryInformations());
					memoryInfo.visible = true;
				}
				else {
					memoryInfo.visible = false;
				}
				break;
			case 5:
				/*
				if (isVisible == true) {
					ethernetInfo.visible = true;
				}
				else {
					ethernetInfo.visible = false;
				}
				*/
				break;
			case 6:
				if (isVisible == true) {
					serialInfo.visible = true;
				}
				else {
					serialInfo.visible = false;
				}
				break;
			case 7:
				/*
				if (isVisible == true) {
					soundDisplayInfo.visible = true;
				}
				else {
					soundDisplayInfo.visible = false;
				}
				*/
				break;
			case 8:
				if (isVisible == true) {
					languageTimeInfo.setLanguageTimeInfos(wemx.getSettings().getLanguageTimeInformations());
					languageTimeInfo.visible = true;
				}
				else {
					languageTimeInfo.visible = false;
				}
				break;
			case 9:
				if (isVisible == true) {
					eventLogInfo.visible = true;
				}
				else {
					eventLogInfo.visible = false;
				}
				break;
			case 10:
				/*
				if (isVisible == true) {
					securityInfo.visible = true;
				}
				else {
					securityInfo.visible = false;
				}
				*/
				break;
			case 11:
				if (isVisible == true) {
					if (Qt.platform.os != "windows")
					{
						exitInfo.visible = true;
					}
					else
					{
						if (launchMode == 3) {
							exitInfo.visible = true;
						} else {
							exitInfo.visible = false;
						}
					}
				}
				else {
					exitInfo.visible = false;
				}
				break;
			default:
				break;
		}
	}

/*
	function onMessageReceived(sender, msg) {
		switch (msg.kind) {
            case "MOUSE.LONG_CLICK":
				settingMain.visible = true;
				break;
		}
    }
*/

	function exitPlayer() {
		wemx.exitPlayer();
	}

	function restartPlayer() {
		wemx.restartPlayer();
	}

	function setEventLogInformation(infoName, isDisplay) {
		wemx.getSettings().setEventLogInformation(infoName, isDisplay);
	}

	Component.onCompleted: {
		//print(">> start WEMX Player Setting Service...");
		
		//wemx.conn.messageReceived.connect(onMessageReceived);
		//wemx.setService("WEMXPlayerSettingService", true);

//        print(">> on loading WEMX Player Setting Service...");

		displayInfo.setDisplayInfos(wemx.getSettings().getDeviceInformations());
		playerInfo.setPlayerInfos(wemx.getSettings().getPlayerInformations());
		hmxInfo.setHmxInfos(wemx.getSettings().getHmxInformations());
		memoryInfo.setMemoryInfos(wemx.getSettings().getMemoryInformations());
		serialInfo.setSerialInfos(wemx.getSettings().getSerialInformations());
		languageTimeInfo.setLanguageTimeInfos(wemx.getSettings().getLanguageTimeInformations());
		eventLogInfo.setEventLogInfos(wemx.getSettings().getEventLogInformations());

		launchMode = wemx.getSettings().getLaunchMode();
		if (Qt.platform.os == "windows") {
			if (launchMode == 3) {
				exitMode = 2;
			} else {
				exitMode = 0;
			}
		} else {
			if (Qt.platform.os == "linux") {
				exitMode = 1;
			} else {
				exitMode = 2;
			}
		}

		exitInfo.setExitMode(exitMode);
	}
}

	
