/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.0

Item {
	property int exitMode: 0;

	property string trRestart: "The system is restarted.";
	property string trExit: "Exits the player.";
	property string trProceed: "Do you want to proceed?";
	property string trOK: "OK";

	// property string fontFamily: "";

	Rectangle {
		x: -1 * settingMain.mRatio
		y: 0
		width: parent.width + 2
		height: 168 * settingMain.mRatio
		color: "#4BE6E6E6"
		border.color: "#FFA5A5A5"
		border.width: 1
	}
	
	Text {
		id: restartText
		x: 0
		y: 36 * settingMain.mRatio
		width: parent.width
		height: 26 * settingMain.mRatio
		horizontalAlignment: Text.AlignHCenter;
		verticalAlignment: Text.AlignVCenter;
		// font.family: fontFamily;
		font.bold: true
		font.pointSize: 10 * settingMain.mRatioFont
		color: "#FFFFFFFF"
		visible: exitMode == 1 ? true : false
		text: trRestart;
	}

	Text {
		id: exitText
		x: 0
		y: 36 * settingMain.mRatio
		width: parent.width
		height: 26 * settingMain.mRatio
		horizontalAlignment: Text.AlignHCenter;
		verticalAlignment: Text.AlignVCenter;
		// font.family: fontFamily;
		font.bold: true
		font.pointSize: 10 * settingMain.mRatioFont
		color: "#FFFFFFFF"
		visible: exitMode == 2 ? true : false
		text: trExit;
	}

	Text {
		id: proceedText
		anchors.top: restartText.bottom
		x: 0
		width: parent.width
		height: 26 * settingMain.mRatio
		horizontalAlignment: Text.AlignHCenter;
		verticalAlignment: Text.AlignVCenter;
		// font.family: fontFamily;
		font.bold: true
		font.pointSize: 10 * settingMain.mRatioFont
		color: "#FFFFFFFF"
		text: trProceed;
	}

	Item {
		anchors.top: proceedText.bottom
		anchors.horizontalCenter: parent.horizontalCenter
		x: 0
		anchors.topMargin: 12 * settingMain.mRatio
		width: 72 * settingMain.mRatio
		height: 26 * settingMain.mRatio

		Rectangle {
			anchors.fill: parent;
			color: getPressedColor();
			border.color: getPressedBorderColor();
		}

		Text {
			id: okText
			anchors.fill: parent;
			horizontalAlignment: Text.AlignHCenter;
			verticalAlignment: Text.AlignVCenter;
			// font.family: fontFamily;
			font.bold: true
			font.pointSize: 10 * settingMain.mRatioFont
			color: "#FF000000"
			text: trOK;
		}

		MouseArea {
			id: okMouseArea
			anchors.fill: parent
			onPressed: {
				// settingMain.playSound(settingMain.touchSoundType);
			}
			onClicked: {
				if (Qt.platform.os == "linux")
				{
					settingMain.restartPlayer();
				}
				else
				{
					settingMain.exitPlayer();
				}				
			}
		}
	}
	
	function getPressedColor() {
		if (okMouseArea.pressed) {
			return "#FF50DDFF";
		}
		else {
			return "#FFD9D9D9";
		}
	}

	function getPressedBorderColor() {
		if (okMouseArea.pressed) {
			return "#FF50DDFF";
		}
		else {
			return "#FFB9B9B9";
		}
	}

	function setExitMode(mode)
	{
		exitMode = mode;

		if (exitMode == 1)
		{
			exitText.visible = false;
			restartText.visible = true;
		}
		else if (exitMode == 2)
		{
			exitText.visible = true;
			restartText.visible = false;
		}
	}
}
