.import "Util.js" as Util

var senderTimer;
var serverUrl;
var hmxName;
var hostInfo;
var usePushNotification;
var pushId;
var alarmDataQueue;
var MAX_QUEUE_LENGTH = 128;
var pendingRequestData = "";
var HTTP_STATUS_TYPE = { IDLE:0, REQUESTING:1};
var HTTP_RETRY_DELAY = 10000;
var HTTP_REQUEST_DELAY = 500;

var http_status;

function initService(timer) {
    console.log(">> initService()");

    senderTimer = timer;
    senderTimer.triggered.connect(sendPushData);
    http_status = HTTP_STATUS_TYPE.IDLE;

    alarmDataQueue = new Util.Queue(MAX_QUEUE_LENGTH);

    usePushNotification = wemx.getSettings().getUsePushNotification();

    serverUrl = "https://wemx-player.appspot.com/";

    hmxName = wemx.getSettings().getHmxName();
    console.log(">> hmx name: " + hmxName);

    var hostName = wemx.getSettings().getDeviceInformations().hostName;
    var ipAddress = wemx.getSettings().getDeviceInformations().ipAddress;

    if ( hostName && hostName.length !== 0) {
        hostInfo = hostName + "(" + ipAddress + ")";
    } else {
        hostInfo = "(" + ipAddress + ")";
    }
    console.log(">> host info: " + hostInfo);

    if (usePushNotification == true) {
        console.log(">> use push notification!");

        pushId = wemx.getSettings().getPushId();
        console.log(">> push id: " + pushId);

    }

    wemx.conn.messageReceived.connect(onMessageReceived);

    wemx.setService("WEMXPushNotificationService", true);
}

function onMessageReceived(sender, msg) {
    console.log(">> onMessageReceived:");

    switch(msg.kind) {
    case "appendAlarmPushData" :
        console.log(">> received appendAlarmPushData");
        alarmDataQueue.enqueue(msg.pushData);
        if (http_status == HTTP_STATUS_TYPE.IDLE) {
            senderTimer.interval = HTTP_REQUEST_DELAY;
            senderTimer.restart();
        }
        break;
    }
}

function getRequestDataString() {
    var requestData = {};
    var requestDataString;

    requestData.push_user_id = pushId;

    var itemArray = [];

    while (alarmDataQueue.isEmpty() != true){
        var item = {};
        var alarmData = {};

        alarmData = alarmDataQueue.dequeue();

        item.host = hostInfo;
        item.hmx = hmxName;
        item.block = alarmData.block;
        item.type = alarmData.type;
        item.time = alarmData.time;
        item.status_type = alarmData.status_type;
        item.message = alarmData.message;
        item.count = alarmData.count;
        item.level = alarmData.level;
        itemArray.push(item);
    }

    requestData.data = itemArray;
    requestDataString = JSON.stringify(requestData);

    //wemx.log_e(">> getRequestDataString: " + requestDataString);
    //console.log(">> getRequestDataString: " + requestDataString);
    return requestDataString;
}

function sendPushData() {
    if ((!pendingRequestData || pendingRequestData.length === 0)
            && alarmDataQueue.isEmpty()) {
        return;
    }

    if (pendingRequestData.length == 0 || alarmDataQueue.getLength() >= MAX_QUEUE_LENGTH) {
        pendingRequestData = getRequestDataString();
    }
    http_status = HTTP_STATUS_TYPE.REQUESTING;

    postJson(pendingRequestData, function (status, response) {
        console.log(">> postJson callback");
        console.log(">> status: " + status);

        if (status == 200) {
            if (response || response != undefined) {
                //console.log(">> response: " + response);
                pendingRequestData = "";
            }
        } else {
            if (response || response != undefined) {
                //console.log(">> response: " + response);
            }
        }

        if (pendingRequestData.length > 0) {
            senderTimer.interval = HTTP_RETRY_DELAY;
            senderTimer.restart();
        } else if (alarmDataQueue.getLength() > 0) {
            senderTimer.interval = HTTP_REQUEST_DELAY;
            senderTimer.restart();
        } else {
            http_status = HTTP_STATUS_TYPE.IDLE;
        }
    });
}

function postJson(data, callback) {
    console.log(">>postJson()");

    var requestUrl = serverUrl + "v1/push/user/alarm"

    var xmlhttp = new XMLHttpRequest();

    xmlhttp.open("POST", requestUrl);
    xmlhttp.setRequestHeader("Content-Type", "application/json; charset=utf-8");
    
    xmlhttp.onreadystatechange = function() {
        //console.log(">>readyState: " + xmlhttp.readyState + ", " + "status: " + xmlhttp.status);
        if (xmlhttp.readyState == 4) {
            var responseData;
            if (xmlhttp.status == 200) {
                console.log(">> response: " + JSON.stringify(xmlhttp.responseText));
                responseData = JSON.parse(xmlhttp.responseText);
            }
            console.log(">> status: " + xmlhttp.status);
            callback(xmlhttp.status, responseData);
        }
    }

    //wemx.log_e(">> xmlhttp.send: " + data);
    //console.log(">> xmlhttp.send: " + data);
    xmlhttp.send(data);
}
