.import QtQuick.LocalStorage 2.0 as Sql

var RECIPE_STATE = { RECIPE_STATE_DEFAULT:0, RECIPE_STATE_TRANSMIT:1, RECIPE_STATE_DAY_STOP:2, RECIPE_STATE_SIZE_STOP:3, RECIPE_STATE_RW_ERROR:4, RECIPE_STATE_RW_SUCCESS:5, RECIPE_STATE_ETC_ERROR:99 };
var RECIPE_CONTROL = { IDLE:0, RECIPE_TO_ADDRESS:1, ADDRESS_TO_RECIPE:2, RECIPE_TO_CSV:3, CSV_TO_RECIPE:4 };
var SAVE_PATH = { USB:0, SDCARD:1, INTERNAL:2 };
var LANGUAGE_TYPE = { ASCII: 0, KSC5601:1, UTF8:7, UTF16:8 };

var recipe = {};
var dbReady = false;
var saveStorage = "";
var language = "";
var saveDirectory = "/data/recipe/";

function loadRecipeBlockData(gtx, dataInfo, blockNumber)
{
	var dataList = [];
	var dataDb = gtx.executeSql('SELECT data_index, data FROM recipe_block_data WHERE block_number = ?', [blockNumber]);
	for(var i = 0; i < dataDb.rows.length; i++) 
	{
		var data = {};
		data.blockNumber = blockNumber;
		data.index = dataDb.rows.item(i).data_index;		
		data.data = dataDb.rows.item(i).data;

/*		if ( dataInfo[data.index].m_dataType < 2 )
		{
			data.iVal = parseInt(data.data);		
		}
	*/				
//		console.log(" rawData1 ", blockNumber, data.index, data.data, data.byteLen);
				
		dataList[data.index] = data;
	}
	
	return(dataList);
}


function recipeControlCallback(hAddr, parm2, changedValue)
{
    var rcpControl = recipe.transmitHAddr.readWord16();    
    transferRecipe(rcpControl, recipe.groupHAddr.readWord16(), recipe.blockHAddr.readWord16(), 1, -1);
}


function zeroPadGroupNumber(groupNumber)
{
	if ( groupNumber < 10 )
	{
		return("0" + groupNumber.toString());
	}
	else
	{
		return(groupNumber.toString());
	}
}

function initService() 
{
//	wemx.conn.messageReceived.connect(onMessageReceived);				
//	wemx.setService("WEMXRecipeService", true);		

    recipeService.executed.connect(executed);

console.log(" --- Recipe ", recipeService, executed);

    var localService = wemx.getService("/local");
    localService.setService("RecipeService", recipeService);

		
	recipe.transmitHAddr = wemx.get("$control:RECIPE_TRANSMIT");	
	recipe.stateHAddr = wemx.get("$control:RECIPE_STATE_CONTROL");
	recipe.groupHAddr = wemx.get("$control:RECIPE_GROUP_SELECT");
	recipe.blockHAddr = wemx.get("$control:RECIPE_BLOCK_SELECT");	

	if ( !recipe.transmitHAddr.isNull() )
	{
		recipe.transmitHAddr.registerChangeNotify(recipeControlCallback, recipe.transmitHAddr, null, 16, wemx); 
	}
			
    var db = Sql.LocalStorage.openDatabaseSync("aromaHMI", "1.0", "", 100000000);


    var settings = wemx.getSettings();
    var savePath = settings.getRecipeSavePath();
	
	var wemxDataPath = settings.getWeMXDataPath();
	saveDirectory = wemxDataPath + "/recipe/";

    if ( savePath == SAVE_PATH.USB )
    {
        saveStorage = "USB";
    }
    else if ( savePath == SAVE_PATH.SDCARD )
    {
        saveStorage = "SDCARD";
    }
    else
    {
        saveStorage = "APP_DATA";
    }

    language = settings.getRecipeLanguage();

    //console.log(" Recipe Setting >>  ", saveStorage, ", ", language);

try {
	
	db.transaction(
		function(tx) {

			var rcpGroupDb = tx.executeSql('SELECT group_number, description FROM recipe_group_information ORDER BY group_number ASC');
			recipe.group = [];
			for(var i = 0; i < rcpGroupDb.rows.length; i++) {
				var group = {};
				
				group.number = rcpGroupDb.rows.item(i).group_number;
				group.description = rcpGroupDb.rows.item(i).description;
													
				recipe.group[group.number] = group;
								
				
				group.db = Sql.LocalStorage.openDatabaseSync("WEMXRecipe"+zeroPadGroupNumber(group.number), "1.0", "", 100000000);

				group.db.transaction(
					function(gtx) {
						var dataEleDb = gtx.executeSql('SELECT data_index, data_type, sign_type, address_id, description FROM recipe_data');
						group.dataInfo = [];
						
						for(var j=0; j < dataEleDb.rows.length; j++)
						{
							var dataInfo = {};

							dataInfo.m_index = dataEleDb.rows.item(j).data_index;							
							dataInfo.m_dataType = dataEleDb.rows.item(j).data_type;
							dataInfo.m_signType = dataEleDb.rows.item(j).sign_type;
							dataInfo.m_description = dataEleDb.rows.item(j).description;							
							var addressId = dataEleDb.rows.item(j).address_id;
							dataInfo.hAddr = wemx.get(addressId);							
							
							var addrDb = tx.executeSql('SELECT address_display_name, value_len FROM address WHERE address_id = ?', [addressId]);
							dataInfo.m_name = addrDb.rows.item(0).address_display_name;
							dataInfo.m_byteLen = addrDb.rows.item(0).value_len * 2;

							group.dataInfo[dataInfo.m_index] = dataInfo;
						}
					
						var blockDb = gtx.executeSql('SELECT block_number, block_name FROM recipe_block');
						group.block = [];
						
						for(var j=0; j < blockDb.rows.length; j++)
						{					
							var block = {};
							
							block.number = blockDb.rows.item(j).block_number;
							block.name = blockDb.rows.item(j).block_name;							
							block.dataList = loadRecipeBlockData(gtx, group.dataInfo, block.number);
							
							group.block[block.number] = block;					
						}					
					}				
				)			
			}

		}
	)
		
} catch (err) {
	//console.log("ERROR ", err);
	dbReady = false;
	return;
}

	dbReady = true;
}




function updateRecipeDb(db, data)
{
	db.transaction(
		function(tx) {
 			var sqlCmd = "UPDATE recipe_block_data SET data = ? WHERE data_index = ? AND block_number = ?";
			tx.executeSql(sqlCmd, [data.data, data.index, data.blockNumber]);
		}
	)		
}

function updateRecipesDb(db, blocks, updateIndex)
{
    db.transaction(
        function(tx) {
            var data;
            var sqlCmd = "UPDATE recipe_block_data SET data = ? WHERE data_index = ? AND block_number = ?";

            for (var i = 1 ; i < blocks.length ; i++)
            {
                data = blocks[i].dataList[updateIndex];
                tx.executeSql(sqlCmd, [data.data, data.index, data.blockNumber]);
            }
        }
    )
}


function transferRecipeToAddress(group, data)
{	
	var dataInfo = group.dataInfo[data.index];

//console.log("transferRecipeToAddress   ", data.index, dataInfo.m_dataType, data.data);


	if ( dataInfo.m_dataType == 0 ) // 16Bit DEC
	{
		var iVal = parseInt(data.data);
		if ( dataInfo.m_signType == Qt.Checked )
		{
			dataInfo.hAddr.writeWord16(iVal);
		}
		else
		{
			dataInfo.hAddr.writeUWord16(iVal);
		}    
	}
	else if ( dataInfo.m_dataType == 1 ) // 32Bit DEC
	{
		var iVal = parseInt(data.data);
		if ( dataInfo.m_signType == Qt.Checked )
		{
			dataInfo.hAddr.writeWord32(iVal);
		}
		else
		{
			dataInfo.hAddr.writeUWord32(iVal);
		}
	}
	else if ( dataInfo.m_dataType == 2 ) // Text
	{
		dataInfo.hAddr.writeString(dataInfo.m_byteLen, data.data);
	}
}



function transferAddressToRecipe(group, data)
{	
	var dataInfo = group.dataInfo[data.index];

	if ( dataInfo.m_dataType == 0 ) // 16Bit DEC
	{
		var iVal;
		if ( dataInfo.m_signType == Qt.Checked )
		{
			iVal = dataInfo.hAddr.readWord16();
		}
		else
		{
			iVal = dataInfo.hAddr.readUWord16();
			
		}    
		data.data = iVal.toString();
	}
	else if ( dataInfo.m_dataType == 1 ) // 32Bit DEC
	{
		var iVal;
		if ( dataInfo.m_signType == Qt.Checked )
		{
			iVal = dataInfo.hAddr.readWord32();
		}
		else
		{
			iVal = dataInfo.hAddr.readUWord32();
		}
		data.data = iVal.toString();		
	}
	else if ( dataInfo.m_dataType == 2 ) // Text
	{
		data.data = dataInfo.hAddr.readString(dataInfo.m_byteLen);
	}
	updateRecipeDb(group.db, data);
}


function transferRecipeToCSV(group)
{	
    var file = wemx.allocFile();

    //console.log(hasStorage + ", " + saveStorage);
    
    if (file.setStorage(saveStorage))
    {
        file.setCodec("EUC-KR");
        var fileName = saveDirectory + "wemx-recipe-" + zeroPadGroupNumber(group.number) + ".csv";
        file.remove(fileName);

        var head = "AROMASOFT\n" +
                    "FUNCTION,RECIPE\n" +
                    "VERSION,1,0\n" +
                    "GROUP_NUMBER," + group.number + "\n" +
                    "DESCRIPTION," + group.description + "\n" +
                    "BLOCK_COUNT," + (group.block.length-1) + "\n" +
                    "Number,Name,Address,Type,Sign,Length(Word)"
        for(var i=1; i < group.block.length; i++)
        {
            head += ", " + group.block[i].name;
        }
        head += "\n";

        var rtn = file.writeAppend(fileName, head);
        if ( rtn < 0 )
        {
            console.log("writeAppend fail : 1");
            wemx.freeFile(file);
            return(false);
        }

        var endIndex = group.block[1].dataList.length;
        for(var i=1; i < endIndex; i++)
        {
            var dataInfo = group.dataInfo[i];

            var storeData = i + ", " + dataInfo.m_description + ", " + dataInfo.m_name + ", " + dataInfo.m_dataType + ", " + dataInfo.m_signType + ", "
                        + (dataInfo.m_byteLen >> 1);
            for(var j=1; j < group.block.length; j++)
            {
                storeData += "," + group.block[j].dataList[i].data;
            }
            storeData += "\n";

            rtn = file.writeAppend(fileName, storeData);
            if ( rtn < 0 )
            {
                console.log("writeAppend fail : 2");
                wemx.freeFile(file);
                return(false);
            }
        }

        console.log("EXPORT SUCCESS : 3");
        wemx.freeFile(file);
        return(true);
    }
    else if(group == undefined)
    {
        console.log("RecipeManager::Invalid group number");
    }
    else
    {
        console.log("RecipeManager::Invalid path(" + saveStorage + ")");
    }
    
    wemx.freeFile(file);

    return(false);
}


function transferCSVToRecipe(group, blockNumber)
{	
    var file = wemx.allocFile();

    if (file.setStorage(saveStorage))
    {
        file.setCodec("EUC-KR");
        var fileName = saveDirectory + "wemx-recipe-" + zeroPadGroupNumber(group.number) + ".csv";

        if ( file.exists(fileName) == false )
        {
            wemx.freeFile(file);
            return(false);
        }

        var content = file.readAll(fileName);
        var lineList = content.split("\n");

        //console.log(" pos1 : ", lineList.length - 8, group.block[1].dataList.length - 1);

        if ( (lineList.length - 8) != (group.block[1].dataList.length-1) )
        {
            wemx.freeFile(file);
            return(false);
        }

        for(var i=7; i < lineList.length-1; i++)
        {
            var eles = lineList[i].split(",");

            //console.log(" pos2 : ", eles.length - 6, group.block.length-1);

            if ( (eles.length - 6) != (group.block.length-1) )
            {
                wemx.freeFile(file);
                return(false);
            }
        }

        for(var i=7; i < lineList.length-1; i++)
        {
            var eles = lineList[i].split(",");

            if ( blockNumber == 0 )
            {
                for( var j=6; j < eles.length; j++)
                {
                    group.block[j-5].dataList[i-6].data = eles[j];
                }

                updateRecipesDb(group.db, group.block, i-6);
            }
            else
            {
                group.block[blockNumber].dataList[i-6].data = eles[blockNumber+5];

                updateRecipeDb(group.db, group.block[blockNumber].dataList[i-6]);
            }
        }

        wemx.freeFile(file);
        return(true);
    }
    else if(group == undefined)
    {
        console.log("RecipeManager::Invalid group number");
    }
    else
    {
        console.log("RecipeManager::Invalid path(" + saveStorage + ")");
    }
    
    wemx.freeFile(file);

    return(false);
}


function transferRecipe(control, groupNumber, blockNumber, index, count)
{	
	if ( dbReady == false )
	{
        console.log("WEMXRecipeService:: recipe DB not ready...");
		recipe.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR); 
		return;
	}
	
	if ( control == RECIPE_CONTROL.IDLE )
	{
		return;
	}

    if ( (control < RECIPE_CONTROL.RECIPE_TO_ADDRESS) || (control > RECIPE_CONTROL.CSV_TO_RECIPE) )
    {
		recipe.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);   
		recipe.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE);     
        return;
    }
	
	//console.log(".... transferRecipe ... ", control, groupNumber, blockNumber);

	var rcp = recipe;
	if ( (control == 1) || (control == 2) )
	{		
		var group = rcp.group[groupNumber];
		if ( (group == undefined) || (group == null) )
		{
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);   
			rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 
			return;	
		}
		
		var block = group.block[blockNumber];

		if ( (block == undefined) || (block == null) )
		{
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);   
			rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 
			return;	
		}
		
		if ( (index < 1) || (index >= block.dataList.length) )
		{
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);
			rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 
			return;
		}

		var endIndex;
		if ( count < 0 )
		{
			endIndex = block.dataList.length;
		}
		else
		{
			endIndex = ( (index + count) >= block.dataList.length ) ? block.dataList.length : index + count;    
		}		
		
		if ( control == RECIPE_CONTROL.RECIPE_TO_ADDRESS ) 
		{                           // recipe -> address
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_TRANSMIT); 
			
			for(var i=index; i < endIndex; i++)
			{
				transferRecipeToAddress(group, block.dataList[i]);
			}

			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_SUCCESS);
		}
		else if ( control == RECIPE_CONTROL.ADDRESS_TO_RECIPE )
		{                           // address -> recipe
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_TRANSMIT); 
			
			for(var i=index; i < endIndex; i++)
			{
				transferAddressToRecipe(group, block.dataList[i]);
			}

			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_SUCCESS);
		}		
	}
	else
	{
		var group = rcp.group[groupNumber];
		if ( (group == undefined) || (group == null) )
		{
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);   
			rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 
			return;	
		}
	
		if ( control == RECIPE_CONTROL.RECIPE_TO_CSV )
		{  							// recipe -> CSV
			var bRtn;
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_TRANSMIT); 

			try {
				bRtn = transferRecipeToCSV(group);
			} catch (err) {
				console.log("ERROR ", err);
				bRtn = false;
			}

			if ( bRtn == true )
			{
				rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_SUCCESS);
			}
			else
			{
				rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_ERROR);
			}
		}
		else if ( control == RECIPE_CONTROL.CSV_TO_RECIPE )
		{ 							// CSV -> recipe
			var bRtn;

			var block = group.block[blockNumber];
			if ( (blockNumber != 0) && ((block == undefined) || (block == null)) )
			{
				rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_ETC_ERROR);   
				rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 
				return;	
			}
		
			rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_TRANSMIT); 

			try {
				bRtn = transferCSVToRecipe(group, blockNumber);
			} catch (err) {
                wemx.log_d("ERROR ");
				console.log("ERROR ", err);
				bRtn = false;
			}

			if ( bRtn == true )
			{
				rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_SUCCESS);
			}
			else
			{
				rcp.stateHAddr.writeWord16(RECIPE_STATE.RECIPE_STATE_RW_ERROR);
			}		
		}
	}

	rcp.transmitHAddr.writeWord16(RECIPE_CONTROL.IDLE); 	
}


//function onMessageReceived(sender, msg) 
    
function executed(cmd, args1, args2, cb, connId)        
{		
	//console.log("WEMX Recipe Service  onMessageReceived .........", msg.kind);

    if ( cmd == "transfer" )
    {
        transferRecipe(args1.control, args1.group, args1.block, args1.index, args1.count);
	}		
}

