import QtQuick 2.0
import com.wemx 1.0
import QtQuick.LocalStorage 2.0 as Sql
import "WEMXSamplingBackup.js" as Backup

Item {
    //property var samplingGroups: new Array()
    property var samplingGroups: []
    property var samplingGroupsKey: []

    property var notifyList: []

    property var samplingDataDB;

    id: samplingRootItem
    Component.onCompleted: {
        console.log(">> start WEMX Sampling Service... ");

        initService();

        wemx.conn.fastMessageReceived.connect(onFastMessageReceived)
        wemx.conn.messageReceived.connect(onMessageReceived)
		//wemx.conn.screenClosed.connect(onScreenClosed)
		
        wemx.setService("WEMXSamplingService", true);
    }

    /*
    function onScreenClosed(screenNumber)
    {
        console.log(" onScreenClosed1 >>> ", screenNumber);

        for (var i = 0 ; i < samplingGroups.length ; i++)
        {
            unregisterSamplingList(samplingGroups[samplingGroupsKey[i]]);
        }
    }
    */

    function onFastMessageReceived(sender, msg) 
    {
        //console.log(" .................... FAST MESSAGE RECEIVE sampling service.... ");

	    switch(msg.kind)
	    {
            default:
            case "getRecord":
                //console.log(" ----- getRecord ", msg.groupNumber, msg.startPos, msg.count);			
                var resultMsg = { kind: "getRecordResult" };
                resultMsg.result = getTableDataRecord(msg.groupNumber, msg.order, msg.startPos + msg.count, msg.startPos, msg.channels);
                resultMsg.maximum_data_count = getMaximum(msg.groupNumber);

                //console.log("  ----- !!!   SamplingService getRecord ", resultMsg.result.length);
			
            wemx.sendMessage(sender, resultMsg);
            break;
        }
    }

    function getMaximum(groupNumber)
    {
        var maximum = 0;
        samplingDataDB.transaction(
            function(tx) {
                var prepareSqlCmd = 'SELECT Max(data_count) as mc FROM sampling_data WHERE sampling_number = ?';
                var prepareQueryResult = tx.executeSql(prepareSqlCmd, [groupNumber]);

                maximum = prepareQueryResult.rows.item(0).mc;
            }
        );

        return(maximum);
    }

    function getTableDataRecord(groupNumber, order, endPos, startPos, channelList)
    {
        //var samplingGroupNumber = samplingGroups[groupNumber].number;
        var result = [];

        samplingDataDB.transaction(
            function(tx) {
                var orderBy = 'ASC';
                var start = startPos;
                var end = endPos;
                
                //console.log("---------------end, start: ", end, start);

                if (order == 1) // new record order
                {
                    var gap = end - start;
                    var dataCount = getMaximum(groupNumber);
                    //console.log("---------------gap = ", gap);

                    end = dataCount - startPos + 1;
                    //console.log("---------------dataCount - startPos: ", dataCount, startPos);

                    if (end <= 2)
                    {
                        end = dataCount + start;
                    }

                    //console.log("---------------final end: ", end);
                    start = (end - gap < 0 ? 1 : end - gap);
                    orderBy = 'DESC';

                    //console.log("---------------start: ", start);
                    //console.log("---------------end: ", end);
                    //console.log("---------------orderBy: ", orderBy);
                }

                //console.log("---------------end, start: ", end, start);

                for (var i = 0 ; i < channelList.length ; i++)
                {
                    var channelNumber = channelList[i];
                    var sqlCmd = 'SELECT sampling_data_id, sampling_number, address_id, data_count, value, time_stamp, channel_number, data_type, sign FROM sampling_data WHERE sampling_number = ? AND data_count < ? AND data_count >= ? AND channel_number = ? ORDER BY sampling_data_id ' + orderBy;

                    var samplingDataList = tx.executeSql(sqlCmd, [groupNumber, end, start, channelNumber]);

                    for(var rowIndex = 0; rowIndex < samplingDataList.rows.length; rowIndex++) 
                    {
                        var item = result[rowIndex];

                        if (item == undefined)
                        {
                            var channel_data = [];

                            item = {};
                            item.channel_data = channel_data;
                        }

                        var column = item.channel_data;

                        // time stamp
                        item.time_stamp = samplingDataList.rows.item(rowIndex).time_stamp;
                        //console.log("<<<<<<<<<<<<<< samplingDataList::i : ", rowIndex);
                        //console.log("<<<<<<<<<<<<<< samplingDataList::time_stamp : ", item.time_stamp);

                        // data[]
                        var dataItem = {};
                        var data = samplingDataList.rows.item(rowIndex).value;
                        var data_type = samplingDataList.rows.item(rowIndex).data_type;
                        var sign = samplingDataList.rows.item(rowIndex).sign;

                        dataItem.data = data;
                        dataItem.data_type = data_type;
                        dataItem.sign = sign;

                        //console.log("<<<<<<<<<<<<<< samplingDataList::data : ", data);
                        //console.log("<<<<<<<<<<<<<< samplingDataList::data_type : ", data_type);
                        //console.log("<<<<<<<<<<<<<< samplingDataList::sign : ", sign);

                        column.push(dataItem);

                        item.channel_data = column;

                        result[rowIndex] = item;
                    }
                }
            }
        )

        //console.log("<<<<<<<<<<<<<< result : ", result);
        return(result);
    }

    function notifyChangeTableData(groupNumber)
    {
        for(var i=0; i < notifyList.length; i++)
        {
            console.log(" notifyChangeTableData1 >>> ", notifyList);
            if ( (notifyList[i].number == groupNumber))
            {
                var msg = { kind: "notifyChangedTableData" }; //, count: count};
                
                //console.log(" notifyChangeTableData2 >>> ", msg);
                wemx.sendMessage(notifyList[i].targetService, msg);
            }
        }
    }

    function insertHmiSamplingData(samplingGroup)
    {
        samplingDataDB.transaction(
            function(tx) {
                samplingGroup.data_count++;
                var samplingDatas = [];

                for(var i = 0; i < samplingGroup.samplingList.length ; i++)
                {
                    var sqlCmd = "INSERT INTO sampling_data VALUES(null, ?, ?, ?, ?, ?, ?, ?, ?)";
                    var addressName = wemx.getAddressName(samplingGroup.samplingList[i].address);
                    var data;
                    var backupData;
                    
                    if (samplingGroup.samplingList[i].data_type == 1/*SAMPLING_16BIT_DEC*/)
                    {
                        if (samplingGroup.samplingList[i].sign == 0)
                        {
                            backupData = samplingGroup.samplingList[i].address.readUWord16();
                        }
                        else if (samplingGroup.samplingList[i].sign == 2)
                        {
                            backupData = samplingGroup.samplingList[i].address.readWord16();
                        }
                    }
                    else if (samplingGroup.samplingList[i].data_type == 2/*SAMPLING_32BIT_DEC*/)
                    {
                        if (samplingGroup.samplingList[i].sign == 0)
                        {
                            backupData = samplingGroup.samplingList[i].address.readUWord32();
                        }
                        else if (samplingGroup.samplingList[i].sign == 2)
                        {
                            backupData = samplingGroup.samplingList[i].address.readWord32();
                        }
                    }
                    else if (samplingGroup.samplingList[i].data_type == 3)
                    {
                        backupData = wemx.getByte(samplingGroup.samplingList[i].address, samplingGroup.samplingList[i].data_type * 16);
                    }

                    data = wemx.getByte(samplingGroup.samplingList[i].address, samplingGroup.samplingList[i].data_type * 16);

                    if(samplingGroup.backup_type == 1)
                    {
                        samplingDatas[i] = backupData;
                        //console.log("************************** INSERT Sampling_data >>> ", backupData);
                    }

                    tx.executeSql(sqlCmd, [samplingGroup.number, addressName, samplingGroup.data_count, data, new Date().getTime(), i+1, samplingGroup.samplingList[i].data_type, samplingGroup.samplingList[i].sign]);
                    //console.log(" INSERT Sampling_data >>> ", samplingGroup.number, addressName, samplingGroup.data_count, data, new Date().getTime(), i+1, samplingGroup.samplingList[i].data_type, samplingGroup.samplingList[i].sign);
                    
                }
                    
                // check using Backup
                if (samplingGroup.backup_type == 1)
                {
                    // check using control Address
                    if (samplingGroup.use_backup_control == 1)
                    {
                        // check control value
                        if (samplingGroup.backupControl == 1)
                        {
                            // write to CSV
                            var time = new Date().getTime();
                            Backup.insertRecord(samplingGroup.number, time, samplingGroup.document_date_format, samplingGroup.document_time_format, samplingDatas);
                        }
                        else if (samplingGroup.backupControl == 2)
                        {
                            // read from CSV(TBD)
                        }
                        else if (samplingGroup.backupControl == 3)
                        {
                            // do nothing
                        }
                    }
                    else
                    {
                        //console.log(" INSERT Sampling_backup 3 >>> ", samplingDatas);
                        var time = new Date().getTime();
                        Backup.insertRecord(samplingGroup.number, time, samplingGroup.document_date_format, samplingGroup.document_time_format, samplingDatas);
                    }
                }
            }
        )

        samplingGroup.confirm_bit_address.writeWord16(samplingGroup.data_count);

        var dataCount = samplingGroup.data_count;
        var samplingCount = samplingGroup.sample_count;
        var saveCount = samplingGroup.save_count;
        var type = samplingGroup.type;

        if (samplingCount <= dataCount)
        {
            unregisterSamplingList(samplingGroup);
            samplingGroup.pull_bit_address.writeBit(1);
        }

        //console.log(" insertHmiSamplingData >>> ", dataCount, ", ", samplingCount);
        //notifyChangeTableData(samplingGroup.number);
    }

    function changeValueNotifyFunc(hAddr, samplingGroup, changedValue)
    {
        //console.log("Sampling control value occur samplingGroup : ", samplingGroup.number, wemx.getAddressName(hAddr));

        if (hAddr == samplingGroup.approval_bit_address)
        {
            var type = samplingGroup.type;
            //console.log("approval_bit_address!! type == ", type);

            if (type == 5)  // Bit Change
            {
                insertHmiSamplingData(samplingGroup);

                //console.log("changeValueNotifyFunc Bit Change");
            }
            else if (type == 4) // Bit On
            {
                if (changedValue)
                {
                    insertHmiSamplingData(samplingGroup);
                }
            }
            else if(type == 1) // Time
            {
                // do nothing
            }
            else
            {
                if (type == 3) // Bit on cycle
                {
                    if (changedValue)
                    {
                        //console.log("changeValueNotifyFunc Bit ON ", wemx.getAddressName(hAddr), changedValue, changedValue);
                        
                        registerSamplingList(samplingGroup);
                    }
                    else
                    {
                        //console.log("changeValueNotifyFunc Bit OFF ", wemx.getAddressName(hAddr), changedValue, changedValue);
                        
                        unregisterSamplingList(samplingGroup);
                    }
                }
            }

            //console.log("Sampling control value occur : approvalAddress");
        }
        else if (hAddr == samplingGroup.sample_cycle_address)
        {
            var cycle = samplingGroup.sample_cycle_address.readWord16();
            samplingGroup.sample_cycle = cycle;
        }
        else if (hAddr == samplingGroup.pull_bit_address)
        {
            //console.log("Sampling control value occur : pull_bit_address");
//            if (!changedValue && samplingGroup.data_count < samplingGroup.sample_count && samplingGroup.type != 5)
//            {
                  registerSamplingList(samplingGroup);
//            }
//            else if(changedValue)
//            {
                  unregisterSamplingList(samplingGroup);
//            }
        }
        else if (hAddr == samplingGroup.clear_address)
        {
            if (changedValue)
            {
                deleteSamplingData(samplingGroup);
            }

            //console.log("Sampling control value occur : clear_address");
        }
        else if (hAddr == samplingGroup.confirm_bit_address)
        {
            //console.log("Sampling control value occur : confirm_bit_address");
        }
        else if (hAddr == samplingGroup.backup_state_address)
        {
            //console.log("Sampling control value occur : backup_state_address");
        }
        else if (hAddr == samplingGroup.backup_control_address)
        {
            samplingGroup.backupControl = changedValue;
        }
    }
    
    function unregisterSamplingList(samplingGroup)
    {
        if (samplingGroup.samplingList.length > 0)
        {
            if (samplingGroup.samplingList[0].regId != 0)
            {
                console.log("unregisterSamplingList", samplingGroup.number);
                samplingGroup.samplingList[0].address.unregisterChangeNotify(samplingGroup.samplingList[0].regId);
                samplingGroup.samplingList[0].regId = 0;
            }
        }
    }

    function registerSamplingList(samplingGroup)
    {
        var cycle = samplingGroup.sample_cycle;

        if (samplingGroup.samplingList.length > 0)
        {
            //console.log("registerChangeNotify: ", samplingGroup.number, samplingGroup.samplingList[0].data_type*16, cycle);
            var samplingItem = samplingGroup.samplingList[0];
            samplingItem.regId = samplingGroup.samplingList[0].address.registerChangeNotify(changeSamplingDataNotifyFunc, samplingGroup.samplingList, samplingGroup, samplingGroup.samplingList[0].data_type*16, wemx, cycle);

            samplingGroup.samplingList[0] = samplingItem;
            console.log("registerChangeNotify: ", samplingGroup.samplingList[0].regId);
        }
    }

    function changeSamplingDataNotifyFunc(samplingList, samplingGroup, changedValue)
    {
        var dataCount = samplingGroup.data_count;
        var samplingCount = samplingGroup.sample_count;
        var saveCount = samplingGroup.save_count;
        var type = samplingGroup.type;

        try {
            //console.log(">>>>>>>>>>>>>>>>>>>>> changeSamplingDataNotifyFunc: ", samplingGroup.number);
            if(type == 5 || type == 4) // Bit Change || Bit On
            {
                //if (dataCount < samplingCount)
                //{
                //    insertHmiSamplingData(samplingGroup);
                //    //samplingGroup.timer.stopTimer();
                //    unregisterSamplingList(samplingGroup);
                //}
            }
            else
            {
                if (type == 2)
                {
                    insertHmiSamplingData(samplingGroup);
                }
                else if (!samplingGroup.approval_bit_address.readBit())
                {
                    //console.log("changeSamplingDataNotifyFunc(", samplingGroup.number, ") : Bit OFF, no Cycle");   
                    if(type != 2 && type != 1)
                    {
                        //console.log("changeSamplingDataNotifyFunc(", samplingGroup.number, ") : Bit OFF, no Cycle");    
                        unregisterSamplingList(samplingGroup);
                    }
                }
                else
                {
                    if(type != 2)  // !Cycle
                    {
                        //console.log("changeSamplingDataNotifyFunc2 : dataCount(", dataCount, "), samplingCount(", samplingCount, ")");
                        if (dataCount < samplingCount)
                        {
                            insertHmiSamplingData(samplingGroup);
                        }
                        else
                        {
                            unregisterSamplingList(samplingGroup);
                        }
                    }
                }
            }
        } catch (err) {
	        console.log("ERRRRR ", err);
        }
    }

    function samplingTimeout(samplingGroup)
    {
        var dataCount = samplingGroup.data_count;
        var samplingCount = samplingGroup.sample_count;
        var saveCount = samplingGroup.save_count;
        var type = samplingGroup.type;

        //console.log(">>>>>>>>>>>>>>>>>>>>> samplingTimeout: ", samplingGroup.number);

        // only use "Set Time"
        if (samplingGroup.approval_bit_address.readBit())
        {
            if(type == 1)  // Set Time
            {
                //console.log("samplingTimeout : dataCount(", dataCount, "), samplingCount(", samplingCount, ")");
                if (dataCount < samplingCount)
                {
                    //console.log("insertHmiSamplingData : ", samplingGroup.number);
                    insertHmiSamplingData(samplingGroup);
                }
                else
                {
                    samplingGroup.timer.stopTimer();
                }
            }
        }
    }

    function deleteSamplingData(samplingGroup)
    {
        samplingGroup.data_count = 0;
        samplingDataDB.transaction(
            function(tx) {
                var sqlCmd = 'DELETE FROM sampling_data WHERE sampling_number = ?';
                tx.executeSql(sqlCmd, [samplingGroup.number]);
            }
        )
        samplingGroup.confirm_bit_address.writeWord16(0);
        //samplingGroup.pull_bit_address.writeBit(false);
        samplingGroup.clear_address.writeBit(false);

        if (samplingGroup.approval_bit_address.readBit() && (samplingGroup.type != 5 && samplingGroup.type != 1)) // Bit Change || Time
        {
            unregisterSamplingList(samplingGroup);
            registerSamplingList(samplingGroup);
        }
    }

    function setHmiDailyTimer(timer, startTime)
    {
        var interval;
        var hour = Math.floor(startTime / 3600000);
        var minuteMill = (startTime - (parseInt(hour) * 3600000));
        var minute = (startTime - (parseInt(hour) * 3600000)) / 60000;
        var second = (startTime - ((parseInt(hour) * 3600000) + (parseInt(minute) * 60000))) / 1000;

        var startDate = new Date();

        startDate.setHours(hour);
        startDate.setMinutes(minute);
        startDate.setSeconds(second);

        timer.setStartDate(startDate);
    }

    function initSampling()
    {
        samplingDataDB = Sql.LocalStorage.openDatabaseSync("WEMXSamplingDataDB", "1.0", "sampling data store db", 10000000);

        samplingDataDB.transaction(
            function(tx) {
                //tx.executeSql('DROP TABLE IF EXISTS sampling_data');
                var sqlCmd = 'CREATE TABLE IF NOT EXISTS sampling_data (sampling_data_id INTEGER PRIMARY KEY AUTOINCREMENT, sampling_number INTEGER, address_id TEXT, data_count INTEGER, value BLOB, time_stamp LONG, channel_number INTEGER, data_type INTEGER, sign INTEGER)';
                //var sqlCmd = 'ALTER TABLE sampling_data ADD data_type INTEGER';
                //var sqlCmd = 'ALTER TABLE sampling_data ADD sign INTEGER';
                tx.executeSql(sqlCmd);
            }
        );

        for(var i = 0; i < samplingGroupsKey.length; i++) {
            var samplingGroup = samplingGroups[samplingGroupsKey[i]];
            var stubStr = 'import QtQuick 2.0; WEMXSamplingStub { id:stub; function startStub() { console.log("SamplingStub!!!!!!!!!!!!!!!!!!");}}';
            var timerStub = Qt.createQmlObject(stubStr, samplingRootItem);
            var cycle;
            
            samplingGroup.data_count = getMaximum(samplingGroup.number);

            samplingGroup.timer = timerStub;
            timerStub.setGroup(samplingGroup);

            if(samplingGroup.sample_cycle_type == 2)    // Address
            {
                cycle = samplingGroup.sample_cycle_address.readWord16();
            }
            else
            {
                cycle = samplingGroup.sample_cycle;
                //console.log("cycle : ", cycle);
            }

            //console.log("cycle : ", cycle);
            timerStub.setInterval(cycle);

            if(samplingGroup.type == 1 ||       // Time
                    samplingGroup.type == 2 ||  // Cycle
                    samplingGroup.type == 3)    // Bit on cycle
            {

                if (samplingGroup.type == 1)    // Time
                {
                    var startTime = samplingGroup.sample_start_time;
                    setHmiDailyTimer(timerStub, startTime);
                }
                else if(samplingGroup.type == 2)    // Cycle
                {
                    //timerStub.startTimer();
                    registerSamplingList(samplingGroup);
                }
                else
                {
                    if(wemx.get(samplingGroup.approval_bit_address).readBit())
                    {
                        //timerStub.startTimer();
                        registerSamplingList(samplingGroup);
                    }
                }
            }
            else
            {
                if (samplingGroup.type == 4 && wemx.get(samplingGroup.approval_bit_address).readBit())    // Bit on
                {
                    if (samplingGroup.data_count < samplingGroup.sample_count)
                    {
                        insertHmiSamplingData(samplingGroup);
                    }
                }
            }

            if(wemx.get(samplingGroup.clear_address).readBit())
            {
                deleteSamplingData(samplingGroup.number);
            }

            samplingGroup.confirm_bit_address.writeWord16(samplingGroup.data_count);

            var dataCount = samplingGroup.data_count;
            var samplingCount = samplingGroup.sample_count;
            var saveCount = samplingGroup.save_count;
            var type = samplingGroup.type;

            if (samplingCount <= dataCount)
            {
                //unregisterSamplingList(samplingGroup);
                samplingGroup.pull_bit_address.writeBit(1);
            }
        }
    }

    function initService() {
        var db = Sql.LocalStorage.openDatabaseSync("aromaHMI", "1.0", "", 1000000);

        db.transaction(
            function(tx) {
                var samplingGroupDb = tx.executeSql('SELECT sampling_number, name, address_count, type, start_address_type, \
                    start_address_id, approval_bit_address_id, sample_start_time, sample_cycle_type, sample_cycle, sample_count, \
                    pull_bit_address_id, clear_address_id, save_type, save_count, confirm_bit_address_id, backup_type, \
                    backup_state_address_id, document_save_storage_type, use_backup_control, backup_control_address_id, \
                    document_date_format, document_time_format, document_limit_day, document_limit_size, document_limit_condition \
                    FROM sampling ORDER BY sampling_number ASC');

                var samplingListIndex = 0;
                for(var i = 0; i < samplingGroupDb.rows.length; i++) {
                    var samplingGroup = {};
                    samplingGroup.number = samplingGroupDb.rows.item(i).sampling_number;
                    if (samplingGroup.number < 1)
                    {
                        continue;
                    }
                    samplingGroup.name = samplingGroupDb.rows.item(i).name;
                    samplingGroup.address_count = samplingGroupDb.rows.item(i).address_count;
                    samplingGroup.type = samplingGroupDb.rows.item(i).type;
                    samplingGroup.start_address_type = samplingGroupDb.rows.item(i).start_address_type;

                    var startAddress = wemx.get(samplingGroupDb.rows.item(i).start_address_id);
                    //startAddress.registerChangeNotify(changeValueNotifyFunc, startAddress, samplingGroup, 16, wemx);
                    samplingGroup.start_address = startAddress;

                    var approvalAddress = wemx.get(samplingGroupDb.rows.item(i).approval_bit_address_id);
                    approvalAddress.registerChangeNotify(changeValueNotifyFunc, approvalAddress, samplingGroup, 1, wemx);
                    samplingGroup.approval_bit_address = approvalAddress;

                    samplingGroup.sample_start_time = samplingGroupDb.rows.item(i).sample_start_time;
                    samplingGroup.sample_cycle_type = samplingGroupDb.rows.item(i).sample_cycle_type;
                    //console.log("sample_cycle_type : ", samplingGroup.sample_cycle_type);
                    if (samplingGroup.sample_cycle_type == 2)   // address
                    {
                        var cycleAddress = wemx.get(samplingGroupDb.rows.item(i).sample_cycle);
                        cycleAddress.registerChangeNotify(changeValueNotifyFunc, cycleAddress, samplingGroup, 16, wemx);
                        samplingGroup.sample_cycle = cycleAddress.readWord16()
                        samplingGroup.sample_cycle_address = cycleAddress;
                    }
                    else
                    {
                        samplingGroup.sample_cycle = samplingGroupDb.rows.item(i).sample_cycle;
                    }
                    samplingGroup.sample_count = samplingGroupDb.rows.item(i).sample_count;

                    var fullAddress = wemx.get(samplingGroupDb.rows.item(i).pull_bit_address_id);
                    fullAddress.registerChangeNotify(changeValueNotifyFunc, fullAddress, samplingGroup, 1, wemx);
                    samplingGroup.pull_bit_address = fullAddress;

                    var clearAddress = wemx.get(samplingGroupDb.rows.item(i).clear_address_id);
                    clearAddress.registerChangeNotify(changeValueNotifyFunc, clearAddress, samplingGroup, 1, wemx);
                    samplingGroup.clear_address = clearAddress;

                    samplingGroup.save_type = samplingGroupDb.rows.item(i).save_type;
                    samplingGroup.save_count = samplingGroupDb.rows.item(i).save_count;

                    var confirmAddress = wemx.get(samplingGroupDb.rows.item(i).confirm_bit_address_id);
                    confirmAddress.registerChangeNotify(changeValueNotifyFunc, confirmAddress, samplingGroup, 16, wemx);
                    samplingGroup.confirm_bit_address = confirmAddress;

                    samplingGroup.backup_type = samplingGroupDb.rows.item(i).backup_type;

                    var backupStateAddress = wemx.get(samplingGroupDb.rows.item(i).backup_state_address_id);
                    backupStateAddress.registerChangeNotify(changeValueNotifyFunc, backupStateAddress, samplingGroup, 16, wemx);
                    samplingGroup.backup_state_address = backupStateAddress;
                    
                    var backupControlAddress = wemx.get(samplingGroupDb.rows.item(i).backup_control_address_id);
                    backupControlAddress.registerChangeNotify(changeValueNotifyFunc, backupControlAddress, samplingGroup, 1, wemx);
                    samplingGroup.backup_control_address = backupControlAddress;
                    samplingGroup.backupControl = samplingGroup.backup_control_address.readBit();

                    samplingGroup.document_save_storage_type = samplingGroupDb.rows.item(i).document_save_storage_type;
                    samplingGroup.use_backup_control = samplingGroupDb.rows.item(i).use_backup_control;
                    samplingGroup.document_date_format = samplingGroupDb.rows.item(i).document_date_format;
                    samplingGroup.document_time_format = samplingGroupDb.rows.item(i).document_time_format;
                    samplingGroup.document_limit_day = samplingGroupDb.rows.item(i).document_limit_day;
                    samplingGroup.document_limit_size = samplingGroupDb.rows.item(i).document_limit_size;
                    samplingGroup.document_limit_condition = samplingGroupDb.rows.item(i).document_limit_condition;

                    samplingGroup.data_count = 0;

                    samplingGroup.samplingList = [];
                    // Make samplingList
                    
                    var sqlCmd ='SELECT sampling_list_id, sampling_number, address_id, data_type, backup_format, sign \
                        FROM sampling_list WHERE sampling_number = ? ORDER BY sampling_list_id ASC';
                    var samplingListDb = tx.executeSql(sqlCmd, [samplingGroup.number]);

                    for (var j = 0 ; j < samplingListDb.rows.length ; j++)
                    {
                        var item = {};
                        //console.log("<< samplingListDb.rows.item(" + j + ").address_id: ", samplingListDb.rows.item(j).address_id);
                        var address = wemx.get(samplingListDb.rows.item(j).address_id);
                        
                        item.sampling_number = samplingListDb.rows.item(j).sampling_number;
                        item.address = address;
                        item.data_type = samplingListDb.rows.item(j).data_type;
                        item.backup_format = samplingListDb.rows.item(j).backup_format;
                        item.sign = samplingListDb.rows.item(j).sign;

                        samplingGroup.samplingList[j] = item;
                    }

                    //var j = 0;
                    //var offset = 0;
                    //do
                    //{
                    //    var address = wemx.getAddress(samplingGroup.start_address, j * offset);
                    //    offset = samplingGroup.start_address_type;
                    //    samplingGroup.samplingList[j] = address;
                    //}while(++j < samplingGroup.address_count);

                    samplingGroup.byteSize = (samplingGroup.start_address_type * 2) * samplingGroup.address_count;

                    samplingGroups[samplingGroup.number] = samplingGroup;
                    samplingGroupsKey[i] = samplingGroup.number;

                    //console.log("<< samplingGroup.samplingList.length : ", samplingGroup.samplingList.length);

                }
            }
        )
        
        initSampling();
        
        Backup.init(samplingGroupsKey, samplingGroups);
    }

    function getRecord(samplingNumber, dataEnd, dataStart)
    {
        //var samplingGroupNumber = samplingGroups[samplingNumber].number;
        var result = [];

        samplingDataDB.transaction(
            function(tx) {
                var sqlCmd = 'SELECT sampling_data_id, sampling_number, address_id, data_count, value, time_stamp, channel_number FROM sampling_data WHERE sampling_number = ? AND data_count <= ? AND data_count >= ? ORDER BY sampling_data_id DESC';
                var samplingDataList = tx.executeSql(sqlCmd, [samplingNumber, dataEnd, dataStart]);

                //console.log("<<<<<<<<<<<<<< samplingNumber : ", samplingNumber);
                //console.log("<<<<<<<<<<<<<< dataStart : ", dataStart);
                //console.log("<<<<<<<<<<<<<< dataEnd : ", dataEnd);
                //console.log("<<<<<<<<<<<<<< samplingDataList.rows.length : ", samplingDataList.rows.length);
                //console.log("<<<<<<<<<<<<<< samplingDataList : ", samplingDataList);

                for(var i = 0; i < samplingDataList.rows.length; i++) {
                    var item = {};
                    item.sampling_data_id = samplingDataList.rows.item(i).sampling_data_id;
                    item.sampling_number = samplingDataList.rows.item(i).sampling_number;
                    item.address_id = samplingDataList.rows.item(i).address_id;
                    item.data_count = samplingDataList.rows.item(i).data_count;
                    item.value = samplingDataList.rows.item(i).value;
                    item.time_stamp = samplingDataList.rows.item(i).time_stamp;
                    item.channel_number = samplingDataList.rows.item(i).channel_number;
                    
                    //console.log("<<<<<<<<<<<<<< item : ", item);

                    result.push(item);
                }
            }
        )

        return(result);
    }

    function onMessageReceived(sender, msg)
    {
        //console.log("......... WEMX Sampling service onMessageReceived .........", msg.kind);

        switch(msg.kind)
        {
			case "getSamplingList":
				var resultMsg = { kind: "getSamplingListResult" };
				console.log("......... getSamplingListResult : ", msg.samplingNumber);
				if (samplingGroups[msg.samplingNumber])
                {
					var samplingGroup = samplingGroups[msg.samplingNumber];
					var result = [];
					for(var i = 0; i < samplingGroup.samplingList.length ; i++)
					{
						var item = {};
						
						item.data_type = samplingGroup.samplingList[i].data_type;
						item.sign = samplingGroup.samplingList[i].sign;
						
						console.log("item[" + i + "]" + " type : " + item.data_type + " , sign : " + item.sign);
						
						result.push(item);
					}
				
                    resultMsg.result = result;
                    resultMsg.samplingNumber = msg.samplingNumber;
                    wemx.sendMessage(sender, resultMsg);
                    console.log("......... return");
                }
				break;
				
            case "getSamplingData":
                var resultMsg = { kind: "getSamplingDataResult" };
                //console.log("......... getSamplingDataResult : ", msg.samplingNumber);

                if (samplingGroups[msg.samplingNumber])
                {
                    resultMsg.result = getRecord(msg.samplingNumber, msg.dataEnd, msg.dataStart);
                    resultMsg.samplingNumber = msg.samplingNumber;
                    wemx.sendMessage(sender, resultMsg);
                    //console.log("......... return");
                }
                break;

            case "getSamplingCount":
                var resultMsg = { kind: "getSamplingCountResult" };
                //console.log("......... getSamplingCount : ", msg.samplingNumber);

                if (samplingGroups[msg.samplingNumber])
                {
                    resultMsg.result = samplingGroups[msg.samplingNumber].data_count;
                    resultMsg.samplingNumber = msg.samplingNumber;
                    wemx.sendMessage(sender, resultMsg);
                    //console.log("......... return");
                }
                break;
            default:
                break;
        }
    }
}

