import QtQuick 2.0
import QtQuick.Controls 1.0
import QtGraphicalEffects 1.0

Item {
	id: coverflowComponent;

	anchors.fill: parent;

	property int baseWidth: 800;
	property int baseHeight: 480;
	property int baseItemWidth: 324;
	property int baseItemHeight: 192;

	property int textHeight: 32;

	property int itemWidth;
	property int itemHeight;

	property int selectedNumber: 0;

	property string screenChangeAnimationImageSource: "";

	MouseArea {
		anchors.fill: parent;
		onPressed: {
			coverflowComponent.parent.hide(false);
		}
	}

	ListModel {
		id: coverflowModel;
	}

	Component {
        id: coverflowDelegate;
        Item {
			property int pathItemWidth: modelPathItemWidth;

			width: pathItemWidth; 
			height: itemHeight * 2;
			z: modelPathViewZ;
			scale: modelPathViewScale;

			property int angleY: modelAngleY;

			transform: Rotation { 
				origin.x: pathItemWidth / 2;
				origin.y: (itemHeight * 2) / 2; 
				axis { x: 0; y: 1; z: 0; } 
				angle: angleY;
			}

			Behavior on angleY {
				SmoothedAnimation {duration: 100;}
			}

			Item {
				id: coverItem;
				width: pathItemWidth;
				height: itemHeight;
				antialiasing: true;

				RectangularGlow {
					y: textHeight;
					width: pathItemWidth;
					height: itemHeight - textHeight;
					glowRadius: pathItemWidth / 2;
					spread: 0;
					color: "#99000000";
					cornerRadius: 5;
				}

				Text {
					width: parent.width;
					height: textHeight;
					anchors.top: parent.top;
					elide: Text.ElideRight;
 					horizontalAlignment: Text.AlignHCenter;
					verticalAlignment: Text.AlignVCenter;
					font.pointSize: 10;
					clip: true;
					font.bold: coverflowView.currentIndex == modelIndex ? true : false;
					color: "#FFFFFFFF";
					text: modelScreenNumber + "_" + modelScreenName;
				}

				Image {
					id: screenImage;	
					y: textHeight;
					width: pathItemWidth;
					height: itemHeight - textHeight;
					source: modelImage;
				}
			}

			Image {
				id: screenReflectionImageMask;
				source: "mask.png";
				visible: false;
			}

			OpacityMask {
				id: reflectionMask;
				y: itemHeight + 3;
				width: pathItemWidth;
				height: itemHeight - textHeight;
				source: coverItem;
				maskSource: screenReflectionImageMask;

				transform: Rotation { 
					origin.y: reflectionMask.height / 2; 
					axis.x:1; axis.y:0; axis.z:0;
					angle: 180;
				}
			}

			Item {
				MouseArea {
					width: pathItemWidth;
					height: itemHeight;
					onClicked: {
						if (coverflowView.currentIndex == modelIndex)
						{
							selectedNumber = modelScreenNumber;
							screenChangeAnimationItem.width = itemWidth;
							screenChangeAnimationItem.height = itemHeight - textHeight;
							screenChangeAnimationItem.x = (coverflowComponent.width - screenChangeAnimationItem.width) / 2;
							screenChangeAnimationItem.y = ((coverflowComponent.height - screenChangeAnimationItem.height) / 2) - textHeight;

							screenChangeAnimationImageSource = modelImage;
							screenChangeAnimationItem.visible = true;

							itemChangeScreenAnimation.start();
						}
						else
						{
							selectedNumber = modelScreenNumber;
							coverflowComponent.parent.selectScreen(modelScreenNumber);			
						}	
					}
				}	
			}
		}
	}

	Item {
		id: coverflowItem;
		x: 80;
		y: (coverflowItem.height / 2) - (itemHeight / 1.36);
		width: parent.width - (80 * 2);
		height: parent.height;
		
		PathView {
			id: coverflowView;
			pathItemCount: 7;
			cacheItemCount: 100;
			anchors.fill: parent;
			highlightRangeMode: PathView.StrictlyEnforceRange;
			preferredHighlightBegin: 0.5;
			preferredHighlightEnd: 0.5;
			focus: true;
			clip: true;
			model: coverflowModel;
			
			delegate: Component {
				Loader {
					z: modelPathViewZ;
					property int modelIndex: index;
					property var modelImage: itemImage;
					property int modelScreenNumber: screenNumber;
					property string modelScreenName: screenName;
					property int modelPathItemWidth: PathView.width;
					property int modelPathViewZ: PathView.pathItemZ;
					property var modelPathViewScale: PathView.pathItemScale;
					property int modelAngleY: PathView.angle;
					sourceComponent: coverflowDelegate;
				}
			}

			path: Path {
				startX: -(itemWidth * 0.36)
				startY: coverflowItem.height / 2 - 12;
				PathAttribute { name: "pathItemZ"; value: 0 }
				PathAttribute { name: "pathItemScale"; value: 0.7 }
				PathAttribute { name: "angle"; value: 56 }
				PathAttribute { name: "width"; value: itemWidth * 0.76 }

				PathLine { x: (coverflowItem.width) * 0.30; y: coverflowItem.height / 2 - 6;  }
				PathAttribute { name: "pathItemZ"; value: 50 }
				PathAttribute { name: "pathItemScale"; value: 0.9 }
				PathAttribute { name: "angle"; value: 40 }
				PathAttribute { name: "width"; value: itemWidth * 0.76 }
				PathPercent { value: 0.40 }

				PathLine { x: (coverflowItem.width) * 0.5; y: coverflowItem.height / 2;  }
				PathAttribute { name: "pathItemZ"; value: 100 }
				PathAttribute { name: "pathItemScale"; value: 1.0 }
				PathAttribute { name: "angle"; value: 0 }
				PathAttribute { name: "width"; value: itemWidth }

				PathLine { x: (coverflowItem.width) * 0.70; y: coverflowItem.height / 2 - 6; }
				PathAttribute { name: "pathItemZ"; value: 50 }
				PathAttribute { name: "pathItemScale"; value: 0.9 }
				PathAttribute { name: "angle"; value: -40 }
				PathAttribute { name: "width"; value: itemWidth * 0.76 }
				PathPercent { value: 0.60 }

				PathLine { x: (coverflowItem.width) + (itemWidth * 0.36); y: coverflowItem.height / 2 - 12; }
				PathAttribute { name: "pathItemZ"; value: 0 }
				PathAttribute { name: "pathItemScale"; value: 0.7 }
				PathAttribute { name: "angle"; value: -56 }
				PathAttribute { name: "width"; value: itemWidth * 0.76 }
				PathPercent { value: 1.0 }
			}

			onDragEnded: {
				selectedNumber = currentItem.modelScreenNumber;
			}

			onFlickEnded: {
				selectedNumber = currentItem.modelScreenNumber;
			}

			onMovementEnded: {	
				selectedNumber = currentItem.modelScreenNumber;
			}
		}
	}

	Item {
		id: screenChangeAnimationItem;

		y: textHeight;
		width: itemWidth; 
		height: itemHeight - textHeight;
		z: 101;
		visible: false;
		
		Image {
			id: screenImage;	
			anchors.fill: parent;
			source: screenChangeAnimationImageSource;
		}
	}

	property bool itemChangeBusy: false;
	ParallelAnimation {
		id: itemChangeScreenAnimation;
		NumberAnimation { target: screenChangeAnimationItem; property: "width"; easing.type: Easing.OutBack; to: coverflowComponent.parent.width; duration: 600 }
		NumberAnimation { target: screenChangeAnimationItem; property: "height"; easing.type: Easing.OutBack; to: coverflowComponent.parent.height; duration: 600 }
		NumberAnimation { target: screenChangeAnimationItem; property: "x"; easing.type: Easing.OutBack; to: 0; duration: 600 }
		NumberAnimation { target: screenChangeAnimationItem; property: "y"; easing.type: Easing.OutBack; to: 0; duration: 600 }
		
		onRunningChanged: {
			if (itemChangeScreenAnimation.running) {
				itemChangeBusy = true;
			} else {
				if (itemChangeBusy) {
					itemChangeBusy = false;
					coverflowComponent.parent.changeScreen(selectedNumber);
					screenChangeAnimationItem.visible = false;
					coverflowComponent.parent.hide(false);
				}            
			}
		}
	}

	Item {
		anchors.left: parent.left;
		width: 80;
		height: coverflowComponent.height / 2;
		anchors.verticalCenter: parent.verticalCenter;

		Image {
			x: (parent.width - 30) / 2;
			anchors.verticalCenter: parent.verticalCenter;
			source: pathLeftMouseArea.pressed ? "prev_sel.png" : "prev_nor.png";
		}

		MouseArea {
			id: pathLeftMouseArea;
			anchors.fill: parent;

			onPressed: {
				movePathLeft();
			}
		}
	}

	function movePathLeft() {
		coverflowView.decrementCurrentIndex();
	}

	Item {
		anchors.right: parent.right;
		width: 80;
		height: coverflowComponent.height / 2;
		anchors.verticalCenter: parent.verticalCenter;

		Image {
			x: (parent.width - 30) / 2;
			anchors.verticalCenter: parent.verticalCenter;
			source: pathRightMouseArea.pressed ? "next_sel.png" : "next_nor.png";
		}

		MouseArea {
			id: pathRightMouseArea;
			anchors.fill: parent;

			onPressed: {
				movePathRight();
			}
		}
	}

	function movePathRight() {
		coverflowView.incrementCurrentIndex();
	}
	
	function setCurrentScreen(number, isChangePosition) {
		selectedNumber = number;
	}

	function setSelectScreen(number) {
		selectedNumber = number;
		for (var i = 0; i < coverflowModel.count; i++) {
			if (number == coverflowModel.get(i).screenNumber) {
				coverflowView.currentIndex = i;
				return;
			}
		}
	}

	function init(width, height, screens) {
		itemWidth = (baseItemWidth * width) / baseWidth;
		itemHeight = ((baseItemHeight * height) / baseHeight) + textHeight;

		coverflowModel.clear();
		for (var i = 0; i < screens.length; i++) {
			var itemImage = "image://screen/" + screens[i].screenNumber;
			coverflowModel.append({ 'screenNumber': screens[i].screenNumber, 'itemImage': itemImage, 'screenName': screens[i].screenName });
		}	
	}
}

