import QtQuick 2.0
import QtQuick.Controls 1.0
import QtGraphicalEffects 1.0

import com.wemx 1.0

Item {
	id: component;
	anchors.centerIn: parent;
	visible: false;
	opacity: 0.01;
	z: 2;
	
	Rectangle {
	anchors.fill: parent;
		color: "#FF000000";
	}

	ScreenChangeCoverflowComponent {
		id: coverflowComponent;
		anchors.fill: parent;
	}
 
	SequentialAnimation {
		id: componentShowAnimation;
		NumberAnimation { target: component; property: "opacity"; to: 1.0; duration: 800 }
    }
	
	function show() {
		var number = wemx.getCurrentScreenNumber();
		if (number < 1)
		{
			return;
		}
		coverflowComponent.setSelectScreen(number);

		component.opacity = 0.01;
		component.visible = true;
		componentShowAnimation.start();
	}

	function hide(isAnimation) {
		component.visible = false;
		component.opacity = 0.01;
	}

	function init() {
		component.opacity = 0.01;

		initRegion();
		var number = wemx.getCurrentScreenNumber();
		var screens = wemx.getScreens();

		coverflowComponent.init(component.width, component.height, screens);
		coverflowComponent.setSelectScreen(number);
	}

	function selectScreen(number) {
		coverflowComponent.setSelectScreen(number);
	}

	function changeScreen(number) {
		wemx.s_call(number);
	} 
	
	function initRegion() {
		var settings = wemx.getSettings();
		var screenWidth = settings.screenWidth();
		var screenHeight = settings.screenHeight();
		var displayScreenWidth = settings.displayScreenWidth();
		var displayScreenHeight = settings.displayScreenHeight();

		if (screenWidth == displayScreenWidth && screenHeight == displayScreenHeight) {
			component.width = screenWidth;
			component.height = screenHeight;
			return;
		}

		var scale = 1.0;
		var widthScale;
		var heightScale;

		if (screenWidth > displayScreenWidth && screenHeight > displayScreenHeight) {
			widthScale = displayScreenWidth / screenWidth;
			heightScale = displayScreenHeight / screenHeight;
			if (widthScale < heightScale) {
				scale = widthScale;
			} else {
				scale = heightScale;
			}
		} else if (screenWidth > displayScreenWidth && screenHeight <= displayScreenHeight) {
			scale = displayScreenWidth / screenWidth;
		} else if (screenHeight > displayScreenHeight && screenWidth <= displayScreenWidth) {
			scale = displayScreenHeight / screenHeight;
		} else if (screenHeight < displayScreenHeight & screenWidth < displayScreenWidth) {
			widthScale = displayScreenWidth / screenWidth;
			heightScale = displayScreenHeight / screenHeight;
			if (widthScale < heightScale) {
				scale = widthScale;
			} else {
				scale = heightScale;
			}
		}

		component.width = screenWidth * scale; 
		component.height = screenHeight * scale;
	}

	function onMessageReceived(sender, msg) {
		switch (msg.kind) {
			case "MOUSE.DOUBLE_CLICK":
				show();
				break;
		}
    }

	Component.onCompleted: {
		if (wemx.getSettings().getScreenChangeServiceMode() == "coverflow") {
			console.log(">> start WEMX Screen Change Coverflow Service...");
		} else {
			return;
		}

		init();
		wemx.conn.messageReceived.connect(onMessageReceived);
		wemx.setService("WEMXScreenChangeCoverflowService", true);
	}
}
