import QtQuick 2.0
//import QtQuick.Controls 1.0

Item {
	id: listComponent;

	property int itemWidth;
	property int itemHeight;
	property int itemSpacing;
	property int listMargin;

	property int selectedNumber: 0;

	Rectangle {
		anchors.fill: parent;
		color: "#99000000";
	}

	ListModel {
		id: listModel;
	}

	ListView {
		id: listView;
		y: 48;
		width: listComponent.width;
		height: listComponent.height - 96;
		spacing: listComponent.itemSpacing;
		smooth: true
		orientation: ListView.Vertical;
		model: listModel;
		anchors.topMargin: listMargin;
		anchors.bottomMargin: listMargin;
		clip: true;

		delegate: Component {
			Loader {
				property int modelIndex: index;
				property var modelImage: itemImage;
				property var modelSelect: itemSelect;
				property int modelScreenNumber: screenNumber;
				property string modelScreenName: screenName;
				property int listX: x - listView.contentX;
				property int listY: y - listView.contentY;
				anchors.horizontalCenter: parent.horizontalCenter;
				sourceComponent: itemDelegate;
			}
		}
	}

	Component {       
		id: itemDelegate;

		Item {
			id: item;
			width: listComponent.itemWidth;
			height: listComponent.itemHeight;
			scale: itemMouseArea.pressed ? 1.1 : 1.0

			property real angleZ: (90 * listY) / listView.height;

			transform: Rotation { 
				origin.x: width * 1.6; 
				origin.y: 30; 
					
				axis { 
					x: 1; 
					y: 0; 
					z: 0.8; 
				} 
				angle: angleZ
			}

			Binding {
				target: item
				property: "angleZ"
				value: 0
				when: !(listView.moving || listView.dragging)
			}

			Behavior on angleZ {
				SmoothedAnimation {duration: 100; to: 0}
				enabled: !(listView.flicking || listView.dragging)
			}

			Behavior on scale {
				SmoothedAnimation { duration: 100 }
			}

			Item {
				anchors.fill: parent;
				anchors.centerIn: parent;
				antialiasing: true;

				Image {
					anchors.fill: parent;
					source: modelImage;
					antialiasing: true;
				}

				Rectangle {
					anchors.fill: parent;
					color: "#00000000"
					border {
						color: "#FF000000"
						width: 1;
					}
				}

				Rectangle {
					width: parent.width;
					height: 20;
					anchors.bottom: parent.bottom;
					color: "#7F000000"
				}

				Text {
					width: parent.width;
					height: 20;
					anchors.bottom: parent.bottom;
					elide: Text.ElideRight
 					verticalAlignment: Text.AlignVCenter;
					font.pointSize: 8
					clip: true
					text: "  " + modelScreenNumber + "_" + modelScreenName;
					color: modelSelect ? "#FF01A1FF" : "#FFC8C8C8"
				}
			}
				
			Rectangle {
				anchors.fill: parent;
				visible: modelSelect
				anchors.centerIn: parent;
				color: "#00000000";
				border.color: "#FF01A1FF";
				border.width: 2;
				antialiasing: true;
			}

			MouseArea {
				id: itemMouseArea
				anchors.fill: parent
				onClicked: {
					if (selectedNumber == modelScreenNumber)
					{
						listComponent.parent.changeScreen(modelScreenNumber);
					}
					else
					{
						selectedNumber = modelScreenNumber;
						listComponent.parent.selectScreen(modelScreenNumber);			
					}	
				}
			}
		}
	}

	Item {
		anchors.top: parent.top;
		width: listComponent.width;
		height: 48;

		Image {
			anchors.centerIn: parent;
			source: listUpMouseArea.pressed ? "prev_2_sel.png" : "prev_2_nor.png";
		}

		MouseArea {
			id: listUpMouseArea;
			anchors.fill: parent;

			onPressed: {
				moveListUp();
			}

			onReleased: {
				listComponent.parent.selectScreen(selectedNumber);		
			}
		}
	}

	Item {
		anchors.bottom: parent.bottom;
		width: listComponent.width;
		height: 48;

		Image {
			anchors.centerIn: parent;
			source: listDownMouseArea.pressed ? "next_2_sel.png" : "next_2_nor.png";
		}

		MouseArea {
			id: listDownMouseArea;
			anchors.fill: parent;

			onPressed: {
				moveListDown();
			}

			onReleased: {
				listComponent.parent.selectScreen(selectedNumber);		
			}
		}
	}

	function moveListUp() {
		var number = -1;
		for (var i = 0; i < listModel.count; i++) {
			if (selectedNumber == listModel.get(i).screenNumber) {
				break;
			}
			else {
				number = listModel.get(i).screenNumber;
			}
		}

		if (number != -1) {
			selectedNumber = number;
			setSelectScreen(selectedNumber);
			// listComponent.parent.selectScreen(selectedNumber);		
		}
	}

	function moveListDown() {
		var number = -1;
		var isFindScreen = false;
		for (var i = 0; i < listModel.count; i++) {
			if (selectedNumber == listModel.get(i).screenNumber) {
				isFindScreen = true;
			}
			else {
				if (isFindScreen && i != listModel.count) {
					number = listModel.get(i).screenNumber;	
					break;
				}					
			}
		}

		if (number != -1) {
			selectedNumber = number;
			setSelectScreen(selectedNumber);
			// listComponent.parent.selectScreen(selectedNumber);		
		}
	}

	function setCurrentScreen(number, isChangePosition) {
		for (var i = 0; i < listModel.count ; i++) {
			if (listModel.get(i).screenNumber == number) {
				if (isChangePosition) {
					listView.positionViewAtIndex(i, ListView.Beginning);
					break;
				}
			}
		}
	}

	function setSelectScreen(number) {
		selectedNumber = number;
		for (var i = 0; i < listModel.count; i++) {
			if (number == listModel.get(i).screenNumber) {
				listModel.set(i, { 'itemSelect': true });
				listView.positionViewAtIndex(i, ListView.Contain);
			}
			else {
				listModel.set(i, { 'itemSelect': false });
			}
		}
	}

	property int baseWidth: 800;
	property int baseHeight: 480;
	property int baseItemWidth: 146;
	property int baseItemHeight: 87;

	function init(width, height, screens) {
		itemWidth = (baseItemWidth * width) / baseWidth;
		itemHeight = (baseItemHeight * height) / baseHeight;

		listModel.clear();
		for (var i = 0; i < screens.length; i++) {
			var itemImage = "image://screen/thumbnail/" + screens[i].screenNumber;
			listModel.append({ 'screenNumber': screens[i].screenNumber, 'itemImage': itemImage, 'screenName': screens[i].screenName, 'itemSelect': false });
		}						
	}
}

