import QtQuick 2.0
//import QtQuick.Controls 1.0
import QtGraphicalEffects 1.0

Item {
	id: mainComponent;

	property int itemX;
	property int itemY;
	property int screenWidth;
	property int screenHeight;
	property int screenNumber;
	property string itemImage;
	
	MouseArea {
		id: invisibleMouseArea
		anchors.fill: parent
		onClicked: {
			mainComponent.parent.hide(true);
		}
	}
	
	Item {
		id: item;
		antialiasing: true;

		property int itemAngle: 0;
        transform: Rotation {
			id: rotation
			origin.x: item.width / 2;
			origin.y: item.height / 2;
			axis {x:0; y:1; z:0}
			angle: item.itemAngle
        }

		RectangularGlow {
			anchors.fill: parent;
			anchors.centerIn: parent;
			glowRadius: 10
			spread: 0
			color: "#FF000000"
			cornerRadius: 5
		}

		Item {
			anchors.fill: parent;
			anchors.centerIn: parent;
			antialiasing: true;

			Image {
				anchors.fill: parent;
				source: itemImage;
				antialiasing: true;
			}
		}

		MouseArea {
			id: itemMouseArea
			anchors.fill: parent
			onClicked: {
				itemFirstSelectAnimation.stop();
				itemSecondSelectAnimation.stop();
				itemChangeScreenAnimation.start();
			}
		}
	}
	
	property bool itemChangeBusy: false;
	ParallelAnimation {
		id: itemChangeScreenAnimation;
		NumberAnimation { target: item; property: "width"; easing.type: Easing.OutBack; to: mainComponent.parent.width; duration: 600 }
		NumberAnimation { target: item; property: "height"; easing.type: Easing.OutBack; to: mainComponent.parent.height; duration: 600 }
		NumberAnimation { target: item; property: "x"; easing.type: Easing.OutBack; to: 0; duration: 600 }
		NumberAnimation { target: item; property: "y"; easing.type: Easing.OutBack; to: 0; duration: 600 }
		
		onRunningChanged: {
			if (itemChangeScreenAnimation.running) {
				itemChangeBusy = true;
				mainComponent.parent.changeScreen(screenNumber);
			} else {
				if (itemChangeBusy) {
					itemChangeBusy = false;
					mainComponent.visible = false;
					item.itemAngle = 0;

					item.width = (baseItemWidth * screenWidth) / baseWidth;
					item.height = (baseItemHeight * screenHeight) / baseHeight;
					item.x = (mainComponent.width - item.width) / 2;
					item.y = (mainComponent.height - item.height) / 2;

					//mainComponent.parent.changeScreen(screenNumber);
					mainComponent.parent.hide(false);
				}            
			}
		}
	}

	property bool itemFirstSelectBusy: false;
	ParallelAnimation {
		id: itemFirstSelectAnimation;
		NumberAnimation { target: item; property: "itemAngle"; from: 0; to: 90; duration: 300; }

		onRunningChanged: {
			if (itemFirstSelectAnimation.running) {
				itemFirstSelectBusy = true;
			} else {
				if (itemFirstSelectBusy) {
					itemFirstSelectBusy = false;
					itemImage = "image://screen/" + screenNumber;
					itemSecondSelectAnimation.start();
				}            
			}
		}
	}

	property bool itemSecondSelectBusy: false;
	ParallelAnimation {
		id: itemSecondSelectAnimation;
		NumberAnimation { target: item; property: "itemAngle"; from: -90; to: 0; duration: 300; }

		onRunningChanged: {
			if (itemSecondSelectAnimation.running) {
				itemSecondSelectBusy = true;
			} else {
				if (itemSecondSelectBusy) {
					itemSecondSelectBusy = false;
					item.itemAngle = 0;
				}            
			}
		}
	}

	property int baseWidth: 800;
	property int baseHeight: 480;
	property int baseItemWidth: 536;
	property int baseItemHeight: 322;

	function init(width, height, number) {
		screenNumber = number;
		itemImage = "image://screen/" + number;

		screenWidth = width;
		screenHeight = height;

		item.width = (baseItemWidth * screenWidth) / baseWidth;
		item.height = (baseItemHeight * screenHeight) / baseHeight;
		item.x = (mainComponent.width - item.width) / 2;
		item.y = (mainComponent.height - item.height) / 2;
		
		mainComponent.visible = true;
	}

	function setSelectScreen(number, isAnimation) {
		screenNumber = number;
		mainComponent.visible = true;

		if (isAnimation) {
			itemFirstSelectAnimation.stop();
			itemSecondSelectAnimation.stop();
			itemFirstSelectAnimation.start();
		} else {
			itemImage = "image://screen/" + number;
		}
	}
}

