import QtQuick 2.0
//import QtQuick.Controls 1.0
//import QtGraphicalEffects 1.0

import com.wemx 1.0

Item {
	id: component;
	anchors.centerIn: parent;
	visible: false;
	//z: 2;

	property int baseWidth: 800;

	property int listComponentWidth;
	property int listItemSpacing: 12;
	property int listItemMargin: 0;

	Rectangle {
		id: mainListComponentBlur;
		anchors.fill: parent;
		color: "#99000000";
	}

	ScreenChangeListComponent {
		id: listComponent;
		x: parent.width
		width: listComponentWidth;
		height: parent.height;
		itemSpacing: listItemSpacing;
		listMargin: listItemMargin;
	}
	
	ScreenChangeMainComponent {
		id: mainComponent;
	}

	ParallelAnimation {
		id: componentShowAnimation;
		SmoothedAnimation { target: listComponent; property: "x"; to: listComponent.parent.width - listComponentWidth; duration: 300 }
		SmoothedAnimation { target: mainComponent; property: "x"; to: 0; duration: 400 }
    }

	property bool hideBusy: false;
	ParallelAnimation {
		id: componentHideAnimation;
		SmoothedAnimation { target: listComponent; property: "x"; to: listComponent.parent.width; duration: 300 }
		SmoothedAnimation { target: mainComponent; property: "x"; to: -(mainComponent.parent.width - listComponentWidth); duration: 400 }

		onRunningChanged: {
			if (componentHideAnimation.running) {
				hideBusy = true;
			} else {
				if (hideBusy) {
					hideBusy = false;
					component.visible = false;

				}            
			}
		}
    }

	function show() {
		var number = wemx.getCurrentScreenNumber();

		if (number < 1)
		{
			return;
		}

		listComponent.setCurrentScreen(number, true);
		listComponent.setSelectScreen(number);
		mainComponent.setSelectScreen(number, false);

		listComponent.x = listComponent.parent.width;
		mainComponent.x = -(mainComponent.parent.width - listComponentWidth);

		component.visible = true;
		mainComponent.visible = true;

		componentHideAnimation.stop();
		componentShowAnimation.start();		
	}

	function hide(isAnimation) {
		if (isAnimation) {
			componentShowAnimation.stop();
			componentHideAnimation.start();
		} else {
			component.visible = false;
            serviceMain.visible = false;
		}
	}

	property int baseListComponentWidth: 186;

	function init() {
		initRegion();

		var number = wemx.getCurrentScreenNumber();
		var screens = wemx.getScreens();
		
		component.listComponentWidth = baseListComponentWidth * component.width / baseWidth;

		listComponent.init(component.width, component.height, screens);		
		listComponent.setCurrentScreen(number, true);
		listComponent.setSelectScreen(number);

		mainComponent.width = component.width - component.listComponentWidth;
		mainComponent.height = component.height;

		mainComponent.init(component.width, component.height, number);
	}

	function selectScreen(number) {
		listComponent.setSelectScreen(number);
		mainComponent.setSelectScreen(number, true);
	}

	function changeScreen(number) {
		wemx.s_call(number);
	} 

	function initRegion() {
		var settings = wemx.getSettings();
		var screenWidth = settings.screenWidth();
		var screenHeight = settings.screenHeight();
		var displayScreenWidth = settings.displayScreenWidth();
		var displayScreenHeight = settings.displayScreenHeight();

		if (screenWidth == displayScreenWidth && screenHeight == displayScreenHeight) {
			component.width = screenWidth;
			component.height = screenHeight;
			return;
		}

		var scale = 1.0;
		var widthScale;
		var heightScale;

		if (screenWidth > displayScreenWidth && screenHeight > displayScreenHeight) {
			widthScale = displayScreenWidth / screenWidth;
			heightScale = displayScreenHeight / screenHeight;
			if (widthScale < heightScale) {
				scale = widthScale;
			} else {
				scale = heightScale;
			}
		} else if (screenWidth > displayScreenWidth && screenHeight <= displayScreenHeight) {
			scale = displayScreenWidth / screenWidth;
		} else if (screenHeight > displayScreenHeight && screenWidth <= displayScreenWidth) {
			scale = displayScreenHeight / screenHeight;
		} else if (screenHeight < displayScreenHeight & screenWidth < displayScreenWidth) {
			widthScale = displayScreenWidth / screenWidth;
			heightScale = displayScreenHeight / screenHeight;
			if (widthScale < heightScale) {
				scale = widthScale;
			} else {
				scale = heightScale;
			}
		}

		component.width = screenWidth * scale; 
		component.height = screenHeight * scale;
	}

/*
	function onMessageReceived(sender, msg) {
		switch (msg.kind) {
			case "MOUSE.DOUBLE_CLICK":
				show();
				break;
		}
    }
*/

	Component.onCompleted: {
		if (wemx.getSettings().getScreenChangeServiceMode() == "list") {
			console.log(">> start WEMX Screen Change Service[list]...");
		} else {
			return;
		}

		init();
		//wemx.conn.messageReceived.connect(onMessageReceived);
		//wemx.setService("WEMXScreenChangeService", true);
	}
}
