import QtQuick 2.0
import com.wemx 1.0
import QtQuick.LocalStorage 2.0 as Sql

WEMXService {
    id: wemx    // eventLogService

	property int logType: 2;
	property var eventLogDB;
	property var logLimit: 5000;

    function initService() {
		eventLogDB = Sql.LocalStorage.openDatabaseSync("aromaHMILog", "1.0", "", 10000000);

		eventLogDB.transaction(
			function(tx) {
				var sqlCmd = "CREATE TABLE IF NOT EXISTS log(log_id INTEGER PRIMARY KEY AUTOINCREMENT, log_type INTEGER, log_time LONG, log_level INTEGER, log_module TEXT, log_message TEXT)";
				tx.executeSql(sqlCmd);
			}
		)
	}

    function executed(cmd, args1, args2, cb, connId)
    {

//console.log(" ------------ WEMXEventLogService onMessageReceived ------------------ ", msg.kind);

       
        if ( cmd == "setEventLog" )
        {
			eventLogDB.transaction(
				function(tx) {
					var sqlCmd = "SELECT COUNT(*) FROM log";
					var logs = tx.executeSql(sqlCmd);
					var logCount = logs.rows.item(0)["COUNT(*)"];
					
					if ((logCount + 1) > logLimit)
					{
						var sqlCmd = "DELETE FROM log WHERE log_id IN (SELECT log_id FROM log ORDER BY log_time ASC LIMIT ?)";
						tx.executeSql(sqlCmd, [(logCount + 1) - logLimit]);
					}
				}
			)

			eventLogDB.transaction(
				function(tx) {
					var sqlCmd = "INSERT INTO log VALUES(null, ?, ?, ?, ?, ?)";
					var logDate = new Date();
					var logTime = logDate.getTime();

					tx.executeSql(sqlCmd, [logType, logTime, args1.level, args1.module, args1.message]);

					var logLevelString = "";
					
					switch (args1.level)
					{
						case 1:
							logLevelString = "Debug";
							break;
						case 2:
							logLevelString = "Info";
							break;
						case 4:
							logLevelString = "Warn";
							break;
						case 8:
							logLevelString = "Error";
							break;
					}

					var logTimeString = Qt.formatDateTime(logDate, "yyyy-MM-dd AP hh:mm");
					//console.log(logTimeString + " " + logLevelString + " " + msg.module + " " + msg.message);
				}
			)
        }
    }


	Component.onCompleted: {
		console.log(">> start Event Log Service ... ");

        wemx.conn.executed.connect(executed);

        wemx.setService("/local/EventLogService", wemx);        
		
        initService();
     }
}

