.pragma library

var API_VERSION = "2.0"

var TAG_TYPE = { INVALID: -1,BOOLEAN: 8, REAL:9, INTEGER:10, STRING:11, CLASS:12, VARIANT:13, CONSTANT:14, EXPRESSION:15};

function isEmpty(str) {
    return (!str || 0 === str.length);
}

function getTagType(tag) {
	var type = tag.type();

	if (type === TAG_TYPE.BOOLEAN) {
		return "boolean";
	} else if (type === TAG_TYPE.REAL) {
		return "real";
	} else if (type === TAG_TYPE.INTEGER) {
		return "integer";
	} else if (type === TAG_TYPE.STRING) {
		return "string";
	} else if (type === TAG_TYPE.CLASS) {
		return "class";		
	} else if (type === TAG_TYPE.CONSTANT) {
		return "constant";		
	} else if (type === TAG_TYPE.EXPRESSION) {
		return "expression";		
	} else {
		return "unknown";
	}
}

function responseJSON(tagname, type, value) {
	var json = {};
	
	json["version"] = API_VERSION;
	json["tagname"] = tagname;
	json["type"] = type;
	
	if (type == "string") {
		json["value"] = encodeURIComponent(value);
	} else {
		json["value"] = value;
	}
	
	return JSON.stringify(json);
}

function error(response, tagname, type, error) {
	var json = {};
	
	json["version"] = API_VERSION;
	json["tagname"] = tagname;
	json["type"] = type;
	json["error"] = error;
	
	var str = JSON.stringify(json)
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", "Keep-Alive");
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);	
	
	response.write(str);
	response.end();
	
	return 200;
}

/**
GET https://10.0.0.1/feeds/tag
?tagname=tagA
*/
function doGet(http, wemx, parames, request, response) {
	
	var tagname = parames["tagname"];
	var isArray = false;
	
	if (isEmpty(tagname)) {
		console.log("405 Not found tagname");
		return 405;
	}

	var tag = wemx.get(tagname);

	if (tag == null) {
		console.log("Invalid tag : " + tagname);
		return error(response, tagname, "Invalid tagname");
	}
	
	var value;

	var type = getTagType(tag);

	if (type === "boolean") {
		value = tag.readBit();
	} else if (type === "real") {
		value = tag.readReal();
	} else if (type === "integer") {
		value = tag.readInteger();
	} else if (type === "string") {
		value = tag.readString();
	} else {
		value = tag.readReal();
	}

	var str = responseJSON(tagname, type, value);
	
	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);	
	
	//console.log("response body :" + str);
	response.write(str);	
	response.end();
	
	return 200;
}

/**
Content-Type: application/json; charset=UTF-8

PUT https://10.0.0.1/feeds/tag
?tagname=tagA

{
    "value": "yoyo"
}

*/
function doPut(http, wemx, parames, request, response) {
	var tagname = parames["tagname"];
	
	if (isEmpty(tagname)) {
		console.log("405 Not found tagname");
		return 405;
	}

	var tag = wemx.get(tagname);
	if (tag == null) {
		console.log("Invalid tag : " + tagname);
		return error(response, tagname, "Invalid tag");
	}	
	
	var json = JSON.parse(request.body);

	var type = getTagType(tag);
	var value = json["value"];

	if (type === "boolean") {
		var boolVal = value == "true" ? true : ((value == 1) ? true : false);
		value = boolVal;
		tag.writeBit(boolVal);
	} else if (type === "real") {
		tag.writeReal(value);
	} else if (type === "integer") {
		value = parseInt(value);
		tag.writeInteger(value);
	} else if (type === "string") {
		value = decodeURIComponent(value);
		tag.writeString(value);
	} else {
		return error(response, tagname, type, "Invalid type");
	}

	var str = responseJSON(tagname, type, value);

	response.setHeader("Content-Type", "application/json; charset=UTF-8;");
	response.setHeader("Connection", request.getHeaderHash("Connection"));
	response.setHeader("Keep-Alive", "timeout=5, max=100");
	response.setHeader("Content-Length", str.length);
	response.writeHead(200);
	
	response.write(str);
	//console.log("response body :" + str);
	response.end();

	return 200
}

function doPost(http, wemx, parames, request, response) {
	return 410
}

function doDelete(http, wemx, parames, request, response) {
	return 410
}