/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.0
import com.wemx 1.0

WEMXService {
	id: wemx
    
	width: 1920
	height: 1080

	property int indexWidth: 60	
	property int indexHeight: 60
	
	Text {
		width: 120
		height: 120
		anchors.horizontalCenter: parent.horizontalCenter
		anchors.verticalCenter: parent.verticalCenter
		horizontalAlignment: Text.AlignHCenter
 		verticalAlignment: Text.AlignVCenter
		font.bold: true
		font.pointSize: 64
		clip: true
		text: "+"
		color: "#FF000000"
	}

	Rectangle {
		id: rightIndex
		width: indexWidth
		height: indexHeight
		color: "#FFFF0000"
		border.color: "#00000000"
		border.width: 2
		radius: width * 0.5
		opacity: 0.8
		visible: false
	}

	function executed(cmd, args1, args2, cb, connId)
	{
		if (cmd == "Right.Fingers.Index.Information") {
			var jsonObject = JSON.parse(args1);
			rightIndex.x = jsonObject["Right.Fingers.Index.X"];
			rightIndex.y = jsonObject["Right.Fingers.Index.Y"];

			var isVisible = false;

			if (jsonObject["Right.Fingers.Index.isExisted"] && jsonObject["Right.Fingers.Index.isExtended"]) {
				isVisible = true
			}

			rightIndex.visible = isVisible
		} else if (cmd == "Right.Fingers.Index.PointColor") {
			rightIndex.color = args1
		} 
	}

	Component.onCompleted: {
		wemx.conn.executed.connect(executed);
		wemx.setService("/local/LeapMotionConfigurationService", wemx);     
	}
}
