var QT = { UnChecked:0, Checked:2 };

var LIMIT_OP = { OLDDELETE:0, STOP:1 };
var BACKUP_SAVE_PATH = { USB:0, SDCARD:1, APP_DATA:2};
var OPERATION_STATUS_TYPE = { OPERATION_STATUS_NONE:0, OPERATION_STATUS_SAVING:1, OPERATION_STOP_DATE:2, OPERATION_STOP_SIZE:3, OPERATION_WRITE_ERROR:4, OPERATION_UNKNOWN_ERROR:99};
var OPERATION_LOG_DATA  = ['BIT_SWICTH' , 'WORD_SWICTH' , 'SELECTOR_SWICTH' , 'SCREEN_SWITCH', 'SCRIPT_SWITCH', 'DATA_DISPLAY', 'SLIDER', 'SAMPLING_DATA', 'SYSTEM_MENU', 'LAUNCH', 'OFFLINE', 'LANGUAGE_CHANGE' ];	

var codec = "EUC-KR";
var currentSize;
var oldFileTime = 0;
var limitSize;
var limitDay;
var onLimitOp;

var operationFileList = [];

var currentFileItem = {};
var saveStorage = "";
var dateFormat = "yyyy/MM/dd";
var timeFormat = "hh:mm:ss";

var isOperationEnable = QT.UnChecked;
var operationCtrlHAddr = null;
var operationStatusHAddr = null;
var targetList;

var operationFolder = "/data/operation";


function tagReadInteger(tag)
{
    var value = 0;

    if (tag != null)
    {
        value = tag.tagReadInt64();
    }

    return value;
}


function tagWriteInteger(tag, value)
{
    if (tag != null)
    {
        tag.tagWriteInt64(value);
    }
}


function tagReadBit(tag)
{
    var value = false;

    if (tag != null)
    {
        value = tag.tagReadBit();
    }

    return value;
}

function tagWriteBit(tag, value)
{
    if (tag != null)
    {
        tag.tagWriteBit(value);
    }
}


function tagReadString(tag)
{
    var value = "";

    if (tag != null)
    {
        value = tag.tagReadString();
    }

    return value;
}


function tagWriteString(tag, value)
{
    if (tag != null)
    {
        tag.tagWriteString(value);
    }
}


function tagRegisterChangeNotify(tag, func, parm1, parm2)
{
    if (tag != null)
    {
        tag.tagRegisterChangeNotify(func, parm1, parm2)
    }
}


function operationControlCallback(hAddr, parm2, changedValue)
{
    isOperationEnable = hAddr.readBit();
}


function init() 
{
    var settings = wemx.getSettings();

    var wemxDataPath = settings.getWeMXDataPath();
    operationFolder = wemxDataPath + "/operation";

    isOperationEnable = settings.getEnableOperationLog();
    if ( isOperationEnable == QT.UnChecked )
    {
        return;
    }
    else
    {
        // TODO: 아래 주석처리 , 2.0에 맞춰 구현 필요함

        var tagService = wemx.getService("/local/TagService", true);

        operationCtrlHAddr = null;
        operationStatusHAddr = null;

        if ( settings.getEnableOperationTrigger() == QT.Checked )
        {
            var operationCtrlTag = settings.getOperationLogControlTag();

            if (operationCtrlTag.length != 0)
            {
                operationCtrlHAddr = wemx.getTag(operationCtrlTag);
                tagRegisterChangeNotify(operationCtrlHAddr, operationControlCallback, operationCtrlHAddr, null);
                isOperationEnable = tagReadBit(operationCtrlHAddr);
            }
        }

        if ( settings.getEnableOperationRecordState() == QT.Checked )
        {
            var operationStatusTag = settings.getOperationLogStatusTag();

            if (operationStatusTag.length != 0)
            {
                operationStatusHAddr = wemx.getTag(operationStatusTag);
            }
        }
    }

    limitDay = settings.getOperationDayLimit();
    limitSize = settings.getOperationSizeLimit() * 1024 * 1024;
    onLimitOp = settings.getOperationLimitCondition();
    targetList = settings.getEnableOperationLogTarget();

    var savePath = settings.getOperationSavePath();

    if ( savePath == BACKUP_SAVE_PATH.USB )
    {
        saveStorage = "USB";
    }
    else if ( savePath == BACKUP_SAVE_PATH.SDCARD )
    {
        saveStorage = "SDCARD";
    }
    else
    {
        saveStorage = "APP_DATA";
    }

    if ( settings.getEnableOperationLogDateFormat() == QT.Checked )
    {
        dateFormat = settings.getOperationDaySetting();
    }

    if ( settings.getEnableOperationLogTimeFormat() == QT.Checked )
    {
        timeFormat = settings.getOperationTimeSetting();
        if(timeFormat == 0)
        {
            timeFormat = "AP h:mm";
        }
        else if(timeFormat == 1)
        {
            timeFormat = "hh:mm";
        }
    }

    var file = wemx.allocFile();

    if(file.setStorage(saveStorage))
    {
        currentSize = file.size(operationFolder);

        var fileList = file.fileList(operationFolder);

        for(var i=0; i < fileList.length; i++)
        {
            var operationFile = {};
            operationFile.used = false;
            operationFile.fileName = fileList[i];
            operationFile.lastModified = file.lastModified(operationFolder + "/" + fileList[i]);

            operationFileList.push(operationFile);
        }

        operationFileList.sort(function(a, b) { return a.lastModified - b.lastModified } );
        if( fileList.length > 0 )
        {
            oldFileTime = operationFileList[0].lastModified;
            var dateStr = new Date(oldFileTime).toLocaleDateString(Qt.locale(), "yyyyMMdd");
        }
    }

    wemx.freeFile(file);
}


function openFile(fileItem)
{
    var file = fileItem.file;
    var dateStr = new Date().toLocaleDateString(Qt.locale(), "yyyyMMdd");
    var fileName = "wemx-operation-" + dateStr + ".csv";

    var result = file.open(operationFolder + "/" + fileName);

    if(!result)
    {
        tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_WRITE_ERROR);
        return;
    }

    var fileSize = file.size();
    if ( fileSize == 0 )
    {
        var backupFileHeader = "Date, Time, User, Level, Screen Number, Object Name, Target, Operation, Tag, Tag Value\n";

        file.write(backupFileHeader, backupFileHeader.length);
        currentSize += backupFileHeader.length;

        var operationFile = {};
        operationFile.used = true;
        operationFile.fileName = fileName;
        operationFile.lastModified = file.lastModified(operationFolder + "/" + fileName);

        operationFileList.push(operationFile);

        fileItem.operationFile = operationFile;
    }
    else
    {
        file.seek(fileSize);

        for(var i=0; i < operationFileList.length; i++)
        {
            if ( operationFileList[i].fileName == fileName )
            {
                operationFileList[i].used = true;
                fileItem.operationFile = operationFileList[i];
                break;
            }
        }
    }
}


function closeFile(fileItem)
{
    fileItem.file.close();
    fileItem.operationFile.used = false;
    fileItem.operationFile = null;
}


function deleteOldFile()
{
    if(operationFileList.length == 0)
    {
        return;
    }

    if ( operationFileList[0].used == true )
    {
        // old file이 사용중일대 는 지우지 않는다.
        return;
    }

    var file = wemx.allocFile();

    if (file.setStorage(saveStorage))
    {
        currentSize -= file.size(operationFolder + "/" + operationFileList[0].fileName);
        file.remove(operationFolder + "/" + operationFileList[0].fileName);
        operationFileList.splice(0, 1);

        oldFileTime = operationFileList[0].lastModified;
    }

    wemx.freeFile(file);
}


function writeFile(storeData)
{
    var currentTime = new Date().getTime();

    if ( currentFileItem.file == undefined )
    {
        var file = wemx.allocFile();

        if (file.setStorage(saveStorage))
        {
            file.setCodec(codec);

            currentFileItem.file = file;
            currentFileItem.toDay = Math.floor(currentTime / (1000*60*60*24));

            openFile(currentFileItem);

            if ( oldFileTime == 0 )
            {
                oldFileTime = operationFileList[0].lastModified;
            }
        }
    }
    else
    {
        if (!currentFileItem.file.setStorage(saveStorage))
        {
            tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_WRITE_ERROR);
            return;
        }
    }

    if ( currentSize >= limitSize )
    {
        if ( onLimitOp == LIMIT_OP.STOP )
        {
            tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_STOP_SIZE);
            return;
        }
        else
        {	// old data삭제 후 저장
            deleteOldFile();
        }
    }


    var elapsedDay = Math.floor( (currentTime - oldFileTime) / (1000*60*60*24) );

    if ( elapsedDay >= limitDay )
    {
        if ( onLimitOp == LIMIT_OP.STOP )
        {
            tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_STOP_DATE);
            return;
        }
        else
        {
            // old data삭제 후 저장
            deleteOldFile();
        }
    }

    var currentDay = Math.floor(currentTime / (1000*60*60*24));

    openFile(currentFileItem);

    if ( currentDay != currentFileItem.toDay )
    {
        currentFileItem.toDay = currentDay;
        closeFile(currentFileItem);
        openFile(currentFileItem);
    }

    var writeResult = currentFileItem.file.write(storeData, storeData.length);
    if(writeResult <= 0)
    {
        tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_WRITE_ERROR);
    }
    else
    {
        tagWriteInteger(operationStatusHAddr, OPERATION_STATUS_TYPE.OPERATION_STATUS_SAVING);
    }

    closeFile(currentFileItem);

    currentSize += storeData.length;
}


function setOperationLogData(dateTime, user, level, screenNumber, objectName, target, operation, address, addressValue)
{
    var dateStr;
    var date;

    date = new Date(dateTime * 1000);
    dateStr = date.toLocaleDateString(Qt.locale(), dateFormat) + ", " + date.toLocaleTimeString(Qt.locale(), timeFormat);
    var logData = dateStr + ", " + user + ", " + level + ", " + screenNumber + ", " + objectName + ", "  + target + ", "  + operation + ", "  + address + ", " + addressValue + "\n";

    try
    {
        writeFile(logData);
    }
    catch (err)
    {
    }
}

function setOperationLog(dateTime, user, level, screenNumber, objectName, target, operation, address, addressValue)
{
    if(isOperationEnable == QT.UnChecked)
    {
        return;
    }

    if(targetList[target] == QT.Unchecked)
    {
        return;
    }
    setOperationLogData(dateTime, user, level, screenNumber, objectName, OPERATION_LOG_DATA[target], operation, address, addressValue)
}


function executed(cmd, args1, args2, cb, connId)    
{		
    if ( cmd == "setOperationLog" )
    {
        //console.log(" --- Op Log Service: ", args1.operation, args1.objectName);
        setOperationLog(args1.dateTime, args1.user, args1.level, args1.screenNumber, args1.objectName, args1.target, args1.operation, args1.address, args1.addressValue)
    }
}


function initService()
{
    wemx.conn.executed.connect(executed);
    wemx.setService("/local/OperationLogService", wemx);
    init();
}


