.import QtQuick.LocalStorage 2.0 as Sql

var TAG_TYPE = { INVALID: -1,BOOLEAN: 1, REAL:9, INTEGER:10, STRING:11, CLASS:12, VARIANT:13, CONSTANT:14, EXPRESSION:15};
var RECIPE_STATE = { RECIPE_STATE_START:0, RECIPE_STATE_INPROGRESS:1, RECIPE_STATE_COMPLETE:2 };
var RECIPE_ERROR = { RECIPE_ERROR_NOERROR:0, RECIPE_ERROR_NORMAL:1, RECIPE_ERROR_IOERROR:2 };
var RECIPE_DRIVER = { RECIPE_DRIVER_INTERNAL:0 , RECIPE_DRIVER_CSV:1};

function RecipeGroup(_db, _id, _driver) {
    var db = null;
    var blockId = -1;
    var driver = RECIPE_DRIVER.RECIPE_DRIVER_INTERNAL;
    var settingData = {};
    var blockCount = 0;

    var controlAddr = null;
    var stateAddr = null;
    var errorAddr = null;

    var dataInfos = [];

    var isControlAddrCallbackCalled = false;

    //console.log("RecipeGroup()", blockId);

    if (_db == undefined || _id == undefined) {
        console.log("RecipeGroup() : Parameter Missing or Invalid!!!");
    }

    db = _db;
    blockId = _id;

    if (_driver != undefined) {
        driver = _driver;
    }

    function tagReadInteger(tag) {
        var value = 0;

        if (tag != null) {
            value = tag.tagReadInt64();
        }

        return value;
    }


    function tagWriteInteger(tag, value) {
        if (tag != null) {
            tag.tagWriteInt64(value);
        }
    }


    function tagReadBit(tag) {
        var value = false;

        if (tag != null) {
            value = tag.tagReadBit();
        }

        return value;
    }

    function tagWriteBit(tag, value) {
        if (tag != null) {
            tag.tagWriteBit(value);
        }
    }


    function tagReadString(tag) {
        var value = "";

        if (tag != null) {
            value = tag.tagReadString();
        }

        return value;
    }


    function tagWriteString(tag, value) {
        if (tag != null) {
            tag.tagWriteString(value);
        }
    }


    function tagRegisterChangeNotify(tag, func, parm1, parm2) {
        if (tag != null) {
            tag.tagRegisterChangeNotify(func, parm1, parm2)
        }
    }

    function empty(str) {
        if (typeof str == 'undefined' || !str || str.length === 0 || str === "" || !/[^\s]/.test(str) || /^\s*$/.test(str) || str.replace(/\s/g,"") === "") {
            return true;
        }
        else {
            return false;
        }
    }


    function transferRecipeToAddress(dataInfo, blockKey) {
        console.log("transferRecipeToAddress():", JSON.stringify(dataInfo));
        console.log("blockKey:", blockKey);

        if (dataInfo.tag == null) {
            return;
        }

        //console.log("transferRecipeToAddress() abstractType: " ,dataInfo.tag.abstractType());

        if (dataInfo.tag.abstractType() == TAG_TYPE.INTEGER
                || dataInfo.tag.abstractType() == TAG_TYPE.BOOLEAN) {
            var iVal = parseInt(dataInfo[blockKey]);
            tagWriteInteger(dataInfo.tag, iVal);
        }
        else if (dataInfo.hAddr.abstractType() == TAG_TYPE.STRING) {
            tagWriteString(dataInfo.tag, dataInfo[blockKey]);
        }
    }

    function transferAddressToRecipe(dataInfo, blockKey)
    {
        console.log("transferAddressToRecipe():", JSON.stringify(dataInfo));
        console.log("blockKey:", blockKey);

        if (dataInfo.tag == null) {
            return;
        }

        if (dataInfo.tag.abstractType() == TAG_TYPE.INTEGER
                || dataInfo.tag.abstractType() == TAG_TYPE.BOOLEAN) {
            var iVal = tagReadInteger(dataInfo.tag);

            dataInfo[blockKey] = iVal.toString();
        } else if (dataInfo.hAddr.abstractType() == TAG_TYPE.STRING) {
            dataInfo[blockKey] = tagReadString(dataInfo.tag);
        }

        updateCommBlockMapperSettingDb(dataInfo, blockKey);
    }

    function updateCommBlockMapperSettingDb(dataInfo, blockKey) {
        db.transaction(
                    function(tx) {
                        tx.executeSql("UPDATE comm_block_mapper_setting SET value = ? WHERE comm_block_mapper_id = ? AND key = ?",
                                      [dataInfo[blockKey], dataInfo.commBlockMapperId, blockKey]);
                    });
    }

    function recipeControlCallback(hAddr, parm2, changedValue) {
        var rcpControl = tagReadInteger(controlAddr);

        console.log("recipeControlCallback() rcpControl = ", rcpControl);

        if (isControlAddrCallbackCalled == false) {
            isControlAddrCallbackCalled = true;
            return;
        }

        if (rcpControl > 0 && rcpControl < blockCount + 1) {
            var blockKey = "Block" + rcpControl;
            var blockCustomKey = settingData[blockKey];

            //console.log("blockCustomKey: ", blockCustomKey);

            tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_START);
            //console.log("dataInfos length: ", dataInfos.length);

            for (var i=0; i < dataInfos.length; i++) {
                transferRecipeToAddress(dataInfos[i], blockCustomKey);
                tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_INPROGRESS);
            }

            tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_COMPLETE);
            tagWriteInteger(errorAddr, RECIPE_ERROR.RECIPE_ERROR_NOERROR);
        } else if (rcpControl > 10000 && rcpControl < 10000 + blockCount + 1) {
            var blockKey = "Block" + (rcpControl - 10000);
            var blockCustomKey = settingData[blockKey];

            tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_START);

            for (var i=0; i < dataInfos.length; i++) {
                transferAddressToRecipe(dataInfos[i], blockCustomKey);
                tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_INPROGRESS);
            }

            tagWriteInteger(stateAddr, RECIPE_STATE.RECIPE_STATE_COMPLETE);
            tagWriteInteger(errorAddr, RECIPE_ERROR.RECIPE_ERROR_NOERROR);
        } else {
            tagWriteInteger(errorAddr, RECIPE_ERROR.RECIPE_ERROR_NORMAL);
        }
    }


    this.init = function(){
        console.log("init() blockId = ", blockId);

        try {
            db.transaction(
                        function(tx) {
                            var commBlockSettingDb = tx.executeSql("SELECT key, value FROM comm_block_setting WHERE comm_block_id = ?", blockId);

                            if (commBlockSettingDb.rows.length == 0) {
                                return;
                            }

                            for(var i=0; i < commBlockSettingDb.rows.length; i++) {
                                settingData[commBlockSettingDb.rows.item(i).key] = commBlockSettingDb.rows.item(i).value;
                            }

                            console.log("settingData ", JSON.stringify(settingData));

                            blockCount = parseInt(settingData["BlockCount"]);

                            console.log("blockCount ", blockCount);

                            if (!empty(settingData["Control"])) {
                                controlAddr = wemx.getTag(settingData["Control"]);
                                tagRegisterChangeNotify(controlAddr, recipeControlCallback, controlAddr, null);
                            }

                            if (!empty(settingData["State"])) {
                                stateAddr = wemx.getTag(settingData["State"]);
                            }

                            if (!empty(settingData["Error"])) {
                                errorAddr = wemx.getTag(settingData["Error"]);
                            }

                            var commBlockMapperDb = tx.executeSql("SELECT comm_block_mapper_id, comm_block_id, number, tag, resource_url \
		                														FROM comm_block_mapper WHERE comm_block_id = ? ORDER BY number ASC", [blockId]);

                            if (commBlockMapperDb.rows.length == 0) {
                                return;
                            }

                            for(var i=0; i < commBlockMapperDb.rows.length; i++) {
                                var dataInfo = {};

                                dataInfo.commBlockMapperId = commBlockMapperDb.rows.item(i).comm_block_mapper_id;
                                dataInfo.commBlockId = commBlockMapperDb.rows.item(i).comm_block_id;
                                dataInfo.number = commBlockMapperDb.rows.item(i).number;
                                dataInfo.tagName = commBlockMapperDb.rows.item(i).tag;

                                if (!empty(dataInfo.tagName)) {
                                    dataInfo.tag = wemx.getTag(dataInfo.tagName);
                                } else {
                                    dataInfo.tag = null;
                                }

                                dataInfo.resource_url = commBlockMapperDb.rows.item(i).resource_url;

                                var commBlockMapperSettingDb = tx.executeSql("SELECT comm_block_mapper_setting_id, comm_block_mapper_id, type, flag, key, value \
		                								FROM comm_block_mapper_setting WHERE comm_block_mapper_id = ?  ORDER BY comm_block_mapper_setting_id ASC", [dataInfo.commBlockMapperId]);

                                for(var j=0; j < commBlockMapperSettingDb.rows.length; j++) {
                                    dataInfo[commBlockMapperSettingDb.rows.item(j).key] = commBlockMapperSettingDb.rows.item(j).value;

                                }
                                console.log("dataInfo ", i);
                                console.log("dataInfo ", JSON.stringify(dataInfo));

                                dataInfos.push(dataInfo);
                                console.log("dataInfo ledgth: ", dataInfos.length);
                            }
                        });
        } catch (err) {
            console.log("ERROR ", err);
            tagWriteInteger(errorAddr, RECIPE_ERROR.RECIPE_ERROR_IOERROR);
            return;
        }
    }
}
