.import QtQuick.LocalStorage 2.0 as Sql
.import "WEMXRecipeGroup.js" as Recipe

var COMM_CATEGORY = "Recipe";
var COMM_DRIVER_INTERNAL = "Internal";
var COMM_BLOCK_SETTING_DESCRIPTION_KEY = "Description";

var recipeGroups = [];

function initService() {
    recipeService.executed.connect(executed);


//    var localService = wemx.getService("/local");
//    localService.setService("RecipeService", recipeService);
    wemx.setService("/local/RecipeService", recipeService);

    var db = Sql.LocalStorage.openDatabaseSync("aromaHMIDevice", "1.0", "", 100000000);

    try {
        
        db.transaction(
            function(tx) {

                var commCategoryDb = tx.executeSql("SELECT comm_category_id, number, name FROM comm_category WHERE name = ?", COMM_CATEGORY);

                if (commCategoryDb.rows.length == 0) {
                    return;
                }

                var commCategoryData = {};

                commCategoryData.commCategoryId = commCategoryDb.rows.item(0).comm_category_id;
                commCategoryData.number = commCategoryDb.rows.item(0).number;
                commCategoryData.name = commCategoryDb.rows.item(0).name;

                console.log("commCategoryData ", JSON.stringify(commCategoryData));

                var commDriverDb = tx.executeSql("SELECT comm_driver_id, comm_category_id, number, name FROM comm_driver WHERE comm_category_id = ? AND name = ?", 
                                        [ commCategoryData.commCategoryId, COMM_DRIVER_INTERNAL ]);

                if (commDriverDb.rows.length == 0) {
                    return;
                }

                var commDriverData = {};

                commDriverData.commDriverId = commDriverDb.rows.item(0).comm_driver_id;
                commDriverData.commCategoryId = commDriverDb.rows.item(0).comm_category_id;
                commDriverData.number = commDriverDb.rows.item(0).number;
                commDriverData.name = commDriverDb.rows.item(0).name;

                console.log("commDriverData ", JSON.stringify(commDriverData));

                var commBlockDb = tx.executeSql("SELECT comm_block.comm_block_id, comm_block.comm_driver_id, comm_block.number, comm_block.name, comm_block_setting.value \
                                        FROM comm_block \
                                        JOIN comm_block_setting ON comm_block.comm_driver_id = ? AND comm_block.comm_block_id = comm_block_setting.comm_block_id AND comm_block_setting.key = ? \
                                        ORDER BY comm_block.number ASC",
                                        [ commDriverData.commDriverId, COMM_BLOCK_SETTING_DESCRIPTION_KEY ]);

                var commBlockData = {};

                console.log("commBlockDb.rows.length ", commBlockDb.rows.length);

                for(var i=0; i < commBlockDb.rows.length; i++) {
                    commBlockData.commBlockId = commBlockDb.rows.item(i).comm_block_id;
                    commBlockData.commDriverId = commBlockDb.rows.item(i).comm_driver_id;
                    commBlockData.number = commBlockDb.rows.item(i).number;
                    commBlockData.name = commBlockDb.rows.item(i).name;
                    commBlockData.value = commBlockDb.rows.item(i).value;

                    //console.log("commDriverDb.rows.item(0) ", JSON.stringify(commBlockDb.rows.item(0)));
                    console.log("commBlockData ", JSON.stringify(commBlockData));

                    var recipeGroup = new Recipe.RecipeGroup(db, commBlockData.commBlockId);

                    recipeGroups.push(recipeGroup);
                }
            }
        )
            
    } catch (err) {
        //console.log("ERROR ", err);
        return;
    }

    for(var i=0; i < recipeGroups.length; i++) {
        var recipeGroup = recipeGroups[i];
        recipeGroup.init();
    }
}

function executed(cmd, args1, args2, cb, connId) {       
    //console.log("WEMX Recipe Service  onMessageReceived .........", msg.kind);

    if ( cmd == "transfer" ) {
        //transferRecipe(args1.control, args1.group, args1.block, args1.index, args1.count);
    }       
}
